/* can.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *  RCSHeader
 */

/*   This files contains the definition of the MPI CAN object.
 *   The CAN object allows the user to control and monitor a set of CANOpen
 *   nodes connected to the controller.
 */

#if !defined(_CAN_H)
#define _CAN_H

/* Can handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEICan *MEICan;
#if defined(__cplusplus)
}
#endif

#include "meidef.h"
#include "control.h"

#if defined(__cplusplus)
extern "C" {
#endif

/* The maximum number of CANnetworks that can exist on a controller. */
#define MEICanNetworkMAX  (1)

/* The maximum number of nodes that can exist on a CANopen network. */
/* This is defined in the CANOpen specification DS301. */
#define MEICanNodeCountMAX   (127)

typedef long (MPI_DEF2 *MEICanCallback)(long percentage);

typedef enum {
    MEICanBitRateINVALID = -1,

    MEICanBitRate1000K = 0,
    MEICanBitRate800K,
    MEICanBitRate500K,
    MEICanBitRate250K,
    MEICanBitRate125K,
    MEICanBitRate50K,
    MEICanBitRate20K,
    MEICanBitRate10K,

    MEICanBitRateLAST,
    MEICanBitRateFIRST = MEICanBitRateINVALID + 1
} MEICanBitRate;

typedef enum {
    MEICanTransmissionTypeINVALID = -1,

    MEICanTransmissionTypeCYCLIC = 0,
    MEICanTransmissionTypeEVENT  = 1,

    MEICanTransmissionTypeLAST,
    MEICanTransmissionTypeFIRST = MEICanTransmissionTypeINVALID + 1
} MEICanTransmissionType;

typedef struct MEICanNodeConfig {
    MEICanTransmissionType digitalOutTransmissionType;
    MEICanTransmissionType analogOutTransmissionType;
    MEICanTransmissionType digitalInTransmissionType;
    MEICanTransmissionType analogInTransmissionType;
} MEICanNodeConfig;

typedef struct MEICanConfig {
    MEICanBitRate bitRate;
    unsigned long cyclicPeriod;
    unsigned long healthPeriod;
    unsigned long nodeNumber;
    unsigned long inhibitTime;
} MEICanConfig;

typedef enum {
    MEICanCommandTypeINVALID = -1,

    MEICanCommandTypeSDO_READ,
    MEICanCommandTypeSDO_WRITE,
    MEICanCommandTypeCLEAR_STATUS_BITS,
    MEICanCommandTypeBUS_START,
    MEICanCommandTypeBUS_STOP,
    MEICanCommandTypeNMT_ENTER_PRE_OPERATIONAL,
    MEICanCommandTypeNMT_START_REMOTE_NODE,
    MEICanCommandTypeNMT_STOP_REMOTE_NODE,
    MEICanCommandTypeNMT_RESET_NODE,
    MEICanCommandTypeNMT_RESET_COMMUNICATION,

    MEICanCommandTypeLAST,
    MEICanCommandTypeFIRST = MEICanCommandTypeINVALID + 1
} MEICanCommandType;

typedef struct MEICanCommand {
    MEICanCommandType type;
    long data[6];
} MEICanCommand;

typedef enum {
    MEICanNodeTypeNONE    = 0,
    MEICanNodeTypeIO      = 401
} MEICanNodeType;

typedef enum {
	MEICanNodeInfoProductCodeUNKNOWN = 0,
	MEICanNodeInfoProductCodeMEI_SLICE_IO = 0x0204
} MEICanNodeInfoProductCode;

typedef enum {
    MEICanHealthTypeINVALID = -1,

    MEICanHealthTypeNODE_GUARDING,
    MEICanHealthTypeHEART_BEATING,

    MEICanHealthTypeLAST,
    MEICanHealthTypeFIRST = MEICanHealthTypeINVALID + 1
} MEICanHealthType;

typedef enum {
	MEICanNodeInfoVendorUNKNOWN = 0,
	MEICanNodeInfoVendorMEI = 0x014F
} MEICanNodeInfoVendor;

typedef struct MEICanNodeInfo {
    MEICanNodeType				type;
    unsigned long				digitalInputCount;
    unsigned long				digitalOutputCount;
    unsigned long				analogInputCount;
    unsigned long				analogOutputCount;
    MEICanHealthType			healthType;
	MEICanNodeInfoVendor		vendorID;
	MEICanNodeInfoProductCode	productCode;
	unsigned long				versionNumber;
	unsigned long				serialNumber;
} MEICanNodeInfo;

typedef enum {
    MEICanNMTStateINVALID = -1,

    MEICanNMTStateBOOT_UP,
    MEICanNMTStateSTOPPED,
    MEICanNMTStateOPERATIONAL,
    MEICanNMTStatePRE_OPERATIONAL,
    MEICanNMTStateUNKNOWN,

    MEICanNMTStateLAST,
    MEICanNMTStateFIRST = MEICanNMTStateINVALID + 1
} MEICanNMTState;

typedef struct MEICanNodeStatus {
    unsigned long live;
    MEICanNMTState nmtState;
} MEICanNodeStatus;

typedef enum {
    MEICanBusStateINVALID     = -1,

    MEICanBusStateOFF,
    MEICanBusStatePASSIVE,
    MEICanBusStateOPERATIONAL,

    MEICanBusStateLAST,
    MEICanBusStateFIRST = MEICanBusStateINVALID + 1
} MEICanBusState;

typedef struct MEICanStatus {
    MEICanBusState  busState;
    long transmitErrorCounter;
    long receiveErrorCounter;
    long messageRate;
    long tick;
    long softwareReceiveOverflow;
    long hardwareReceiveOverflow;
} MEICanStatus;

typedef struct MEICanVersion {
    long bootloaderVersion;
    long firmwareVersion;
    char firmwareRevision;
    long firmwareSubRevision;
} MEICanVersion;

typedef enum {
    MEICanMessageFIRST = mpiMessageID(MEIModuleIdCAN, 0),

    MEICanMessageFIRMWARE_INVALID,
    MEICanMessageFIRMWARE_VERSION,
    MEICanMessageNOT_INITIALIZED,
    MEICanMessageCAN_INVALID,
    MEICanMessageIO_NOT_SUPPORTED,
    MEICanMessageFILE_FORMAT_ERROR,
    MEICanMessageUSER_ABORT,
    MEICanMessageCOMMAND_PROTOCOL,
    MEICanMessageINTERFACE_NOT_FOUND,
    MEICanMessageNODE_DEAD,
    MEICanMessageSDO_TIMEOUT,
    MEICanMessageSDO_ABORT,
    MEICanMessageSDO_PROTOCOL,
    MEICanMessageTX_OVERFLOW,
    MEICanMessageRTR_TX_OVERFLOW,
    MEICanMessageRX_BUFFER_EMPTY,
    MEICanMessageBUS_OFF,
    MEICanMessageSIGNATURE_INVALID,

    MEICanMessageLAST
} MEICanMessage;

/* Create/Delete/Validate */
MPI_DECL1 MEICan MPI_DECL2
    meiCanCreate(MPIControl control,
                 long       number);

MPI_DECL1 long MPI_DECL2
    meiCanDelete(MEICan can);

MPI_DECL1 long MPI_DECL2
    meiCanValidate(MEICan  can);

/* CAN */
MPI_DECL1 long MPI_DECL2
    meiCanCommand(MEICan         can,
                  MEICanCommand* command);

MPI_DECL1 long MPI_DECL2
    meiCanConfigGet(MEICan        can,
                    MEICanConfig* config);

MPI_DECL1 long MPI_DECL2
    meiCanConfigSet(MEICan        can,
                    MEICanConfig* config);

MPI_DECL1 MPIControl MPI_DECL2
	meiCanControl(MEICan can);

MPI_DECL1 long MPI_DECL2
	meiCanInit(MEICan can);

MPI_DECL1 long MPI_DECL2
    meiCanEventNotifyGet(MEICan           can,
						 MPIEventMask *eventMask,
						 void		 *external);

MPI_DECL1 long MPI_DECL2
    meiCanEventNotifySet(MEICan           can,
						 MPIEventMask eventMask,
						 void		 *external);

MPI_DECL1 long MPI_DECL2
    meiCanFirmwareDownload(MEICan         can,
                           const char*    filename,
                           MEICanCallback callback);

MPI_DECL1 long MPI_DECL2
    meiCanFirmwareErase(MEICan         can);

MPI_DECL1 long MPI_DECL2
    meiCanFirmwareUpload(MEICan         can,
                         const char*    filename,
                         MEICanCallback callback);

MPI_DECL1 long MPI_DECL2
    meiCanFlashConfigGet(MEICan        can,
                         void*         flash,
                         MEICanConfig* config);

MPI_DECL1 long MPI_DECL2
    meiCanFlashConfigSet(MEICan        can,
                         void*         flash,
                         MEICanConfig* config);

MPI_DECL1 long MPI_DECL2
    meiCanMemory(MEICan can,
                 void** memory);

MPI_DECL1 long MPI_DECL2
    meiCanMemoryGet(MEICan      can,
                    void*       dst,
                    const void* src,
                    long        count); 

MPI_DECL1 long MPI_DECL2
    meiCanMemorySet(MEICan      can,
                    void*       dst,
                    const void* src,
                    long        count); 

MPI_DECL1 long MPI_DECL2
	meiCanNodeAnalogIn(MEICan  can,
                       long    node,
					   long	   channel,
					   long	   *state);

MPI_DECL1 long MPI_DECL2
	meiCanNodeAnalogOutSet(MEICan		can,
							long		node,
							long		channel,
							long		state,
							MPI_BOOL	wait);

MPI_DECL1 long MPI_DECL2
	meiCanNodeAnalogOutGet(MEICan	can,
							long	node,
							long	channel,
							long	*state);

MPI_DECL1 long MPI_DECL2
    meiCanNodeConfigGet(MEICan            can,
                        long              node,
                        MEICanNodeConfig* nodeConfig);

MPI_DECL1 long MPI_DECL2
    meiCanNodeConfigSet(MEICan            can,
                        long              node,
                        MEICanNodeConfig* nodeConfig);

MPI_DECL1 long MPI_DECL2
	meiCanNodeDigitalIn(MEICan			can,
                               long   node,
						long			bitStart,
						long			bitCount,
						unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	meiCanNodeDigitalOutGet(MEICan			can,
                                long             node,
							long			bitStart,
							long			bitCount,
							unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	meiCanNodeDigitalOutSet(MEICan			can,
                                long             node,
							long			bitStart,
							long			bitCount,
							unsigned long	state,
							MPI_BOOL		wait);

MPI_DECL1 long MPI_DECL2
    meiCanNodeFlashConfigGet(MEICan            can,
                             void*             flash,
                             long              node,
                             MEICanNodeConfig* nodeConfig);

MPI_DECL1 long MPI_DECL2
    meiCanNodeFlashConfigSet(MEICan            can,
                             void*             flash,
                             long              node,
                             MEICanNodeConfig* nodeConfig);

MPI_DECL1 long MPI_DECL2
    meiCanNodeInfo(MEICan          can,
                   long            node,
                   MEICanNodeInfo* nodeInfo);


MPI_DECL1 long MPI_DECL2
    meiCanNodeStatus(MEICan            can,
                     long              node,
                     MEICanNodeStatus* nodeStatus);

MPI_DECL1 long MPI_DECL2
	meiCanNumber(MEICan	can,
				 long	*number);

MPI_DECL1 long MPI_DECL2
    meiCanStatus(MEICan        can,
                 MEICanStatus* status);

MPI_DECL1 long MPI_DECL2
    meiCanVersion(MEICan         can,
                  MEICanVersion* version);

#if defined(__cplusplus)
}
#endif

#endif
