/* capture.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_CAPTURE_H)
#define	_CAPTURE_H

/* Capture handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPICapture *MPICapture;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "motor.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define MPICaptureNOT_MAPPED	(-1)

/* Capture Config */
typedef struct MPICaptureTrigger {
	MPI_BOOL enabled;	/* TRUE/FALSE */
	MPI_BOOL invert;	/* TRUE = invert, FALSE = normal */
} MPICaptureTrigger;

typedef enum MPICaptureSource {
	MPICaptureSourceINVALID = -1,

	MPICaptureSourceMOTOR_IO_0,
	MPICaptureSourceMOTOR_IO_1,
	MPICaptureSourceMOTOR_IO_2,
	MPICaptureSourceMOTOR_IO_3,
	MPICaptureSourceMOTOR_IO_4,
	MPICaptureSourceMOTOR_IO_5,
	MPICaptureSourceMOTOR_IO_6,
	MPICaptureSourceMOTOR_IO_7,
	MPICaptureSourceHOME,
	MPICaptureSourceINDEX,
	MPICaptureSourceLIMIT_HW_NEG,
	MPICaptureSourceLIMIT_HW_POS,
	MPICaptureSourceGLOBAL,
	MPICaptureSourceINDEX_SECONDARY,

	MPICaptureSourceLAST,
	MPICaptureSourceFIRST = MPICaptureSourceINVALID + 1,

	MPICaptureSourceCOUNT = MPICaptureSourceLAST
} MPICaptureSource;

typedef enum MPICaptureEdge {
	MPICaptureEdgeINVALID = -1,

	MPICaptureEdgeNONE,
	MPICaptureEdgeRISING,
	MPICaptureEdgeFALLING,
	MPICaptureEdgeEITHER,

	MPICaptureEdgeLAST,
	MPICaptureEdgeFIRST = MPICaptureEdgeINVALID + 1
} MPICaptureEdge;

typedef enum  {
	MPICaptureTypeINVALID = -1,
	
	MPICaptureTypePOSITION,
	MPICaptureTypeTIME,

	MPICaptureTypeLAST,
	MPICaptureTypeFIRST = MPICaptureTypeINVALID +1 
} MPICaptureType;

typedef struct MPICaptureTriggerGlobal {
	MPI_BOOL enabled;	/* TRUE/FALSE */
} MPICaptureTriggerGlobal;

typedef struct MPICaptureConfig {
	
	MPICaptureTrigger		source[MPICaptureSourceCOUNT]; /* use MPICaptureSource to index */
	MPICaptureEdge			edge;
	MPICaptureTriggerGlobal	global;	

	MPICaptureType			type;
	long					captureMotorNumber;
	long					feedbackMotorNumber;  /* the same as captureMotorNumber for POSITION capture */
	MPIMotorEncoder			encoder;
	long					captureIndex;		  /* 0,1,... */

} MPICaptureConfig;

/* Capture Status */
typedef enum {
	MPICaptureStateINVALID = -1,

	MPICaptureStateIDLE,
	MPICaptureStateARMED,
	MPICaptureStateCAPTURED,
	MPICaptureStateCLEAR,

	MPICaptureStateLAST,
	MPICaptureStateFIRST = MPICaptureStateINVALID + 1
} MPICaptureState;

typedef struct MPICaptureStatus {
	MPICaptureState	 state;
	double			 latchedValue;
} MPICaptureStatus;

typedef enum {
	MPICaptureMessageFIRST = mpiMessageID(MPIModuleIdCAPTURE, 0),

	MPICaptureMessageMOTOR_INVALID,
	MPICaptureMessageCAPTURE_TYPE_INVALID,
	MPICaptureMessageCAPTURE_INVALID,
	MPICaptureMessageENCODER_INVALID,
	MPICaptureMessageLAST
} MPICaptureMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPICapture MPI_DECL2
	mpiCaptureCreate(MPIControl			control,
					 long				number);

MPI_DECL1 long MPI_DECL2
	mpiCaptureDelete(MPICapture	capture);

MPI_DECL1 long MPI_DECL2
	mpiCaptureValidate(MPICapture	capture);

/* Capture */
MPI_DECL1 long MPI_DECL2
	mpiCaptureArm(MPICapture	capture,
				  MPI_BOOL	    arm);	/* TRUE/FALSE */

MPI_DECL1 long MPI_DECL2
	mpiCaptureConfigGet(MPICapture			capture,
						MPICaptureConfig	*config,
						void				*external);

MPI_DECL1 long MPI_DECL2
	mpiCaptureConfigSet(MPICapture			capture,
						MPICaptureConfig	*config,
						void				*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiCaptureControl(MPICapture capture);

MPI_DECL1 long MPI_DECL2
	mpiCaptureMemory(MPICapture	capture,
					 void		**memory);

MPI_DECL1 long MPI_DECL2
	mpiCaptureMemoryGet(MPICapture	capture,
						void		*dst,
						const void	*src,
						long		count);

MPI_DECL1 long MPI_DECL2
	mpiCaptureMemorySet(MPICapture	capture,
						void		*dst,
						const void	*src,
						long		count);


MPI_DECL1 long MPI_DECL2
	mpiCaptureNumber(MPICapture	capture,
					 long 		*number);

MPI_DECL1 long MPI_DECL2
	mpiCaptureStatus(MPICapture			capture,
					 MPICaptureStatus	*status,
					 void				*external);

MPI_DECL1 long MPI_DECL2
	mpiCaptureConfigReset(MPICapture capture);

#if defined(__cplusplus)
}
#endif

#endif
