/* command.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_COMMAND_H)
#define	_COMMAND_H

/* Command handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPICommand *MPICommand;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"
#include "event.h"
#include "eventmgr.h"
#include "motion.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum {
	MPICommandTypeINVALID = -1,

	MPICommandTypeASSIGN,
	MPICommandTypeBRANCH,
	MPICommandTypeBRANCH_EVENT,
	MPICommandTypeBRANCH_IO,
	MPICommandTypeCOMPUTE,
	MPICommandTypeCOMPUTE_IO,
	MPICommandTypeCOPY,
	MPICommandTypeDELAY,
	MPICommandTypeEVENT,
	MPICommandTypeMOTION,
	MPICommandTypeWAIT,
	MPICommandTypeWAIT_EVENT,
	MPICommandTypeWAIT_IO,

	MPICommandTypeLAST,
	MPICommandTypeFIRST = MPICommandTypeINVALID + 1,

	MPICommandTypeBIT_COUNT = sizeof(int) * 8,	/* sizeof(enum) */

	MPICommandTypeREF	= 0x1 << (MPICommandTypeBIT_COUNT - 1),
	MPICommandTypeFLOAT	= 0x1 << (MPICommandTypeBIT_COUNT - 2),

	MPICommandTypeMASK	= ~(MPICommandTypeREF |
							MPICommandTypeFLOAT),

	MPICommandTypeASSIGN_FLOAT		= MPICommandTypeASSIGN | MPICommandTypeFLOAT,

	MPICommandTypeBRANCH_REF		= MPICommandTypeBRANCH | MPICommandTypeREF,
	MPICommandTypeBRANCH_FLOAT		= MPICommandTypeBRANCH | MPICommandTypeFLOAT,
	MPICommandTypeBRANCH_FLOAT_REF	= MPICommandTypeBRANCH_FLOAT | MPICommandTypeREF,

	MPICommandTypeCOMPUTE_REF		= MPICommandTypeCOMPUTE | MPICommandTypeREF,
	MPICommandTypeCOMPUTE_FLOAT		= MPICommandTypeCOMPUTE | MPICommandTypeFLOAT,
	MPICommandTypeCOMPUTE_FLOAT_REF	= MPICommandTypeCOMPUTE_FLOAT | MPICommandTypeREF,

	MPICommandTypeWAIT_REF			= MPICommandTypeWAIT | MPICommandTypeREF,
	MPICommandTypeWAIT_FLOAT		= MPICommandTypeWAIT | MPICommandTypeFLOAT,
	MPICommandTypeWAIT_FLOAT_REF	= MPICommandTypeWAIT_FLOAT | MPICommandTypeREF
} MPICommandType;

typedef enum {
	MPICommandMotionINVALID = -1,

	MPICommandMotionABORT,
	MPICommandMotionE_STOP,
	MPICommandMotionE_STOP_MODIFY,
	MPICommandMotionE_STOP_ABORT,
	MPICommandMotionE_STOP_CMD_EQ_ACT,
	MPICommandMotionMODIFY,
	MPICommandMotionRESET,
	MPICommandMotionRESUME,
	MPICommandMotionSTART,
	MPICommandMotionSTOP,

	MPICommandMotionLAST,
	MPICommandMotionFIRST = MPICommandMotionINVALID + 1
} MPICommandMotion;

typedef enum {
	MPICommandOperatorINVALID = -1,

	/* Arithmetic operators */
	MPICommandOperatorADD,
	MPICommandOperatorSUBTRACT,
	MPICommandOperatorMULTIPLY,
	MPICommandOperatorDIVIDE,

	MPICommandOperatorAND,
	MPICommandOperatorOR,
	MPICommandOperatorXOR,

	/* Logical operators */
	MPICommandOperatorALWAYS,

	MPICommandOperatorEQUAL,
	MPICommandOperatorNOT_EQUAL,

	MPICommandOperatorGREATER_OR_EQUAL,
	MPICommandOperatorGREATER,

	MPICommandOperatorLESS_OR_EQUAL,
	MPICommandOperatorLESS,

	MPICommandOperatorBIT_CLEAR,
	MPICommandOperatorBIT_SET,

	MPICommandOperatorLAST,
	MPICommandOperatorFIRST = MPICommandOperatorINVALID + 1,

	MPICommandOperatorArithmeticFIRST	= MPICommandOperatorADD,
	MPICommandOperatorArithmeticLAST	= MPICommandOperatorXOR + 1,

	MPICommandOperatorLogicalFIRST	= MPICommandOperatorALWAYS,
	MPICommandOperatorLogicalLAST	= MPICommandOperatorBIT_SET + 1
} MPICommandOperator;

typedef union {
	long	*l;
	float	*f;
} MPICommandAddress;

typedef union {
	long	l;
	float	f;
} MPICommandConstant;

typedef struct MPICommandExpr {
	MPICommandOperator	oper;
	MPICommandAddress	address;
	union {
		MPICommandConstant	value;	/* [*'address'] 'oper' ['value'] */
		MPICommandAddress	ref;	/* [*'address'] 'oper' [*'ref']  */
	} by;
} MPICommandExpr;

typedef union {
	struct {	/* *'dst' = 'value' */
		MPICommandAddress	dst;
		MPICommandConstant	value;
		MPIControl			control;	/* Ignored by Sequence */
	} assign;

	struct {	/* branch to 'label' on 'expr' */
		char			*label;		/* NULL => stop sequence */
		MPICommandExpr	expr;		/* expr.oper => MPICommandOperatorLogical */
		MPIControl		control;	/* Ignored by Sequence */
	} branch;

	struct {	/* branch to 'label' on MPIEventMask('handle') 'oper' 'mask' */
		char				*label;		/* NULL => stop sequence */
		MPIHandle			handle;		/* [MPIMotor|MPIMotion|...] */
		MPICommandOperator	oper;		/* EQUAL/NOT_EQUAL/BIT_CLEAR/BIT_SET */
		MPIEventMask		mask;		/* MPIEventMask('handle') 'oper' 'mask' */
	} branchEvent;

	struct {	/* branch to 'label' on Io.input 'oper' 'mask' */
		char				*label;		/* NULL => stop sequence */
		MPIIoType			type;		/* MOTOR,   USER */
		MPIIoSource			source;		/* MPIMotor	index */
		MPICommandOperator	oper;		/* EQUAL/NOT_EQUAL/BIT_CLEAR/BIT_SET */
		long				mask;		/* [motor|user]Io.input 'oper' 'mask' */
	} branchIO;

	struct {	/* *'dst' = 'expr' */
		MPICommandAddress	dst;
		MPICommandExpr		expr;		/* expr.oper => MPICommandOperatorArithmetic */
		MPIControl			control;	/* Ignored by Sequence */
	} compute;

	struct {	/* Io.output = Io.output 'oper' 'mask' */
		MPIIoType			type;		/* MOTOR,   USER */
		MPIIoSource			source;		/* MPIMotor	index */
		MPICommandOperator	oper;		/* AND/OR/XOR */
		long				mask;
	} computeIO;

	struct {	/* memcpy(dst, src, count) */
		void		*dst;
		void		*src;
		long		count;
		MPIControl	control;	/* Ignored by Sequence */
	} copy;

	float	delay;	/* seconds */

	struct {
		long		value;	/* MPIEventStatus.type    = MPIEventTypeEXTERNAL */
							/*				 .source  = MPISequence/MPIProgram */
							/*				 .info[0] = value */
		MPIEventMgr	eventMgr;	/* Ignored by Sequence */
	} event;

	struct {	/* mpiMotion[Abort|EStop|Reset|Resume|Start|Stop](motion[, type, params]) */
		MPICommandMotion	motionCommand;
		MPIMotion			motion;
		MPIMotionType		type;	/* MPICommandMotionSTART */
		MPIMotionParams		params;	/* MPICommandMotionSTART */
	} motion;

	struct {	/* wait until 'expr' */
		MPICommandExpr	expr;		/* expr.oper => MPICommandOperatorLogical */
		MPIControl		control;	/* Ignored by Sequence */
	} wait;

	struct {	/* wait until MPIEventMask('handle') 'oper' 'mask' */
		MPIHandle			handle;		/* [MPIMotor|MPIMotion|...] */
		MPICommandOperator	oper;		/* EQUAL/NOT_EQUAL/BIT_CLEAR/BIT_SET */
		MPIEventMask		mask;		/* MPIEventMask('handle') 'oper' 'mask' */
	} waitEvent;

	struct {	/* wait until Io.input 'oper' 'mask' */
		MPIIoType			type;	/* MOTOR,   USER */
		MPIIoSource			source;	/* MPIMotor	index */
		MPICommandOperator	oper;	/* EQUAL/NOT_EQUAL/BIT_CLEAR/BIT_SET */
		long				mask;	/* [motor|user]Io.input 'oper' 'mask' */
	} waitIO;
} MPICommandParams;

typedef enum {
	MPICommandMessageFIRST = mpiMessageID(MPIModuleIdCOMMAND, 0),

	MPICommandMessageCOMMAND_INVALID,
	MPICommandMessageTYPE_INVALID,
	MPICommandMessagePARAM_INVALID,

	MPICommandMessageLAST
} MPICommandMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPICommand MPI_DECL2
	mpiCommandCreate(MPICommandType		type,
					 MPICommandParams	*params,
					 const char			*label);

MPI_DECL1 long MPI_DECL2
	mpiCommandDelete(MPICommand command);

MPI_DECL1 long MPI_DECL2
	mpiCommandValidate(MPICommand command);

/* Command */
MPI_DECL1 long MPI_DECL2
	mpiCommandLabel(MPICommand	command,
					char		**label);

MPI_DECL1 long MPI_DECL2
	mpiCommandParams(MPICommand			command,
					 MPICommandParams	*params);

MPI_DECL1 long MPI_DECL2
	mpiCommandType(MPICommand		command,
				   MPICommandType	*type);

#if defined(__cplusplus)
}
#endif

#endif
