/* compare.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_COMPARE_H)
#define	_COMPARE_H

/* Compare handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPICompare *MPICompare;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum {
	MPICompareStateINVALID = -1,

	MPICompareStateIDLE,
	MPICompareStateARMED,
	MPICompareStateCOMPARED,

	MPICompareStateLAST,
	MPICompareStateFIRST = MPICompareStateINVALID + 1
} MPICompareState;

typedef enum {
	MPICompareMessageFIRST = mpiMessageID(MPIModuleIdCOMPARE, 0),

	MPICompareMessageCOMPARE_INVALID,

	MPICompareMessageLAST
} MPICompareMessage;

#define MPIComparePositionCountMAX	(10)	/* Maximum latches/compare */

typedef struct MPIComparePosition {
	long	number;
	long	motorNumber;
	long	position;
	MPICommandOperator  commandOperator;
} MPIComparePosition;

typedef struct MPICompareConfig {
	MPIIoTrigger	trigger;			/* which output to use... */
} MPICompareConfig;

typedef struct MPICompareParams {
	long	position;
	long	outputState;
	MPICommandOperator	commandOperator;
} MPICompareParams;

typedef struct MPICompareStatus {
	MPICompareState	state;
	double			position[MPIComparePositionCountMAX];
} MPICompareStatus;

/* Create/Delete/Validate */
MPI_DECL1 MPICompare MPI_DECL2
	mpiCompareCreate(MPIControl	control,
					 long		number);

MPI_DECL1 long MPI_DECL2
	mpiCompareDelete(MPICompare	compare);

MPI_DECL1 long MPI_DECL2
	mpiCompareValidate(MPICompare	compare);

/* Compare */
MPI_DECL1 long MPI_DECL2
	mpiCompareArm(MPICompare	compare,
				  long			arm);

MPI_DECL1 long MPI_DECL2
	mpiCompareLoad(MPICompare			compare,
				   MPICompareParams		*params,
				   void					*external);

MPI_DECL1 long MPI_DECL2
	mpiCompareConfigGet(MPICompare			compare,
						MPICompareConfig	*config,
						void				*external);

MPI_DECL1 long MPI_DECL2
	mpiCompareConfigSet(MPICompare			compare,
						MPICompareConfig	*config,
						void				*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiCompareControl(MPICompare	compare);

MPI_DECL1 long MPI_DECL2
	mpiCompareFlashConfigGet(MPICompare			compare,
							 void				*flash,
							 MPICompareConfig	*config,
							 void				*external);

MPI_DECL1 long MPI_DECL2
	mpiCompareFlashConfigSet(MPICompare			compare,
							 void				*flash,
							 MPICompareConfig	*config,
							 void				*external);

MPI_DECL1 long MPI_DECL2
	mpiCompareMemory(MPICompare	compare,
					 void		**memory);

MPI_DECL1 long MPI_DECL2
	mpiCompareMemoryGet(MPICompare	compare,
						void		*dst,
						const void	*src,
						long		count);

MPI_DECL1 long MPI_DECL2
	mpiCompareMemorySet(MPICompare	compare,
						void		*dst,
						const void	*src,
						long		count);

MPI_DECL1 long MPI_DECL2
	mpiCompareNumber(MPICompare	compare,
					 long 		*number);

MPI_DECL1 long MPI_DECL2
	mpiCompareStatus(MPICompare			compare,
					 MPICompareStatus	*status,
					 void				*external);

#if defined(__cplusplus)
}
#endif

#endif
