/* compensator.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_COMPENSATOR_H)
#define	_COMPENSATOR_H

/* Compensator handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPICompensator *MPICompensator;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum MPICompensatorDimension {
	MPICompensatorDimensionINVALID = -1,

	MPICompensatorDimensionX,
	MPICompensatorDimensionY,

	MPICompensatorDimensionLAST,
	MPICompensatorDimensionFIRST = MPICompensatorDimensionINVALID + 1
} MPICompensatorDimension;

#define MPICompensatorDimensionsMAX (MPICompensatorDimensionLAST)

typedef struct MPICompensatorRange {
	double positionMin;
	double positionMax;
} MPICompensatorRange;

typedef struct MPICompensatorInputAxis {
	long				axisNumber;
	MPICompensatorRange	range;
	long				positionDelta;
} MPICompensatorInputAxis;

typedef struct MPICompensatorConfig {
	long					dimensionCount;
	MPICompensatorInputAxis	inputAxis[MPICompensatorDimensionsMAX];
	long					outputAxisNumber;
} MPICompensatorConfig;

typedef struct MPICompensatorInfo {
	long tableDimensions[MPICompensatorDimensionsMAX];
	long tableSizeBytes;
} MPICompensatorInfo;

typedef enum {
	MPICompensatorMessageFIRST = mpiMessageID(MPIModuleIdCOMPENSATOR, 0),

	MPICompensatorMessageCOMPENSATOR_INVALID,
	MPICompensatorMessageNOT_CONFIGURED,
	MPICompensatorMessageNOT_ENABLED,
	MPICompensatorMessageAXIS_NOT_ENABLED,
	MPICompensatorMessageTABLE_SIZE_ERROR,
	MPICompensatorMessagePOSITION_DELTA_INVALID,
	MPICompensatorMessageDIMENSION_NOT_SUPPORTED,

	MPICompensatorMessageLAST
} MPICompensatorMessage;


/* Create/Delete/Validate */
MPI_DECL1 MPICompensator MPI_DECL2
	mpiCompensatorCreate(MPIControl	control,
						 long		number);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorDelete(MPICompensator compensator);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorValidate(MPICompensator compensator);

/* Compensator */
MPI_DECL1 long MPI_DECL2
	mpiCompensatorConfigGet(MPICompensator		 compensator,
							MPICompensatorConfig *config,
							void				 *external);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorConfigSet(MPICompensator		 compensator,
							MPICompensatorConfig *config,
							void				 *external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiCompensatorControl(MPICompensator compensator);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorInfo(MPICompensator     compensator,
					   MPICompensatorInfo *info);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorMemory(MPICompensator	compensator,
						 void			**memory);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorMemoryGet(MPICompensator	compensator,
							void			*dst,
							const void		*src,
							long			count);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorMemorySet(MPICompensator	compensator,
							void			*dst,
							const void		*src,
							long			count);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorNumber(MPICompensator	compensator,
						 long			*number);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorTableGet(MPICompensator compensator,
						   long			  *table);

MPI_DECL1 long MPI_DECL2
	mpiCompensatorTableSet(MPICompensator compensator,
				           long			  *table);

#if defined(__cplusplus)
}
#endif

#endif
