/* control.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *  RCSHeader
 */

#if !defined(_CONTROL_H)
#define _CONTROL_H

/* Controller handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIControl *MPIControl;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define MPI_VERSION_STRINGIZE_INTERNALX(id) #id
#define MPI_VERSION_STRINGIZE(id) MPI_VERSION_STRINGIZE_INTERNALX(id)

#define MPI_VERSION_MAJOR_ID	03
#define MPI_VERSION_MINOR_ID	04
#define MPI_VERSION_RELEASE		"29"
/*
MPI_VERSION_IS_BRANCH is TRUE if this library is a branch. Otherwise, it is
FALSE.

MPI_VERSION_BRANCH includes MPI_VERSION_RELEASE from the parent library plus
the branch number. A leading period, ".", is also necessary. For example, the 
following definitions would be used to make branch B1 from version 03.03.02:
	
	MPI_VERSION_MAJOR_ID	03
	MPI_VERSION_MINOR_ID	03
	MPI_VERSION_RELEASE		"00"
	MPI_VERSION_BRANCH		".02.B1"
	MPI_INTERFACE_VERSION	"03.03.02.B1"
	MPI_VERSION				"03.03.02.B1.00"

NOTE: MPI_VERSION_RELEASE refers to the current branch, not the parent library.
The MPI_VERSION_RELEASE of the parent library is located in MPI_VERSION_BRANCH.
*/
#define MPI_VERSION_IS_BRANCH	FALSE
#define MPI_VERSION_BRANCH		""
	
#define MPI_VERSION_MAJOR		MPI_VERSION_STRINGIZE(MPI_VERSION_MAJOR_ID)
#define MPI_VERSION_MINOR		MPI_VERSION_STRINGIZE(MPI_VERSION_MINOR_ID)

#define MPI_INTERFACE_VERSION	MPI_VERSION_MAJOR "." MPI_VERSION_MINOR MPI_VERSION_BRANCH
#define MPI_VERSION				MPI_INTERFACE_VERSION "." MPI_VERSION_RELEASE

#define MPIControlMAX_RECORDERS			(32)
#define MPIControlRECORD_COUNT_DEFAULT	(16384)
#define MPIControlMAX_COMPENSATORS		(12)
#define MPIControlMAX_AXES				(32)
#define MPIControlMIN_AXIS_FRAME_COUNT	(128)
#define MPIControlMIN_SAMPLE_RATE		(1000)
#define MPIControlMAX_SAMPLE_RATE		(100000)
#define MPIControlMAX_STATUS_CACHE_INTERVAL	(5000)

typedef struct MPIControlConfig {
	long    axisCount;
	long 	axisFrameCount[MPIControlMAX_AXES];
	long    captureCount;
	long    compareCount;
	long	compensatorCount;
	long 	compensatorPointCount[MPIControlMAX_COMPENSATORS];
	long    filterCount;
	long    motionCount;
	long    motorCount;
	long	recorderCount;
	long    recordCount[MPIControlMAX_RECORDERS];
	long    sequenceCount;
	long    userVersion;
	long    sampleRate;
} MPIControlConfig;

typedef enum {
	MPIControlTypeINVALID = -1,

	MPIControlTypeDEFAULT,
	MPIControlTypeMAPPED,
	MPIControlTypeIOPORT,
	MPIControlTypeDEVICE,
	MPIControlTypeCLIENT,
	MPIControlTypeFILE,

	MPIControlTypeLAST,
	MPIControlTypeFIRST = MPIControlTypeINVALID + 1

} MPIControlType;

typedef enum {
	MPIControlFileTypeINVALID = -1,
	MPIControlFileTypeLAST,
	MPIControlFileTypeFIRST = MPIControlFileTypeINVALID + 1
} MPIControlFileType;

typedef struct MPIControlAddress {
	long    number;									/* controller number */

	union {
		void                        *mapped;        /* memory address     */
		unsigned long               ioPort;         /* I/O port number    */
		const char                  *device;        /* device driver name */
		struct {
			const char              *name;          /* image file name    */
			MPIControlFileType      type;           /* image file type    */
		} file;
		struct {
			const char              *server;        /* IP address: host.domain.com */
			long                    port;           /* socket number      */
		} client;
	} type;
} MPIControlAddress;

typedef enum {
	MPIControlMemoryTypeINVALID = -1,

	MPIControlMemoryTypeUSER,

	MPIControlMemoryTypeLAST,
	MPIControlMemoryTypeFIRST = MPIControlMemoryTypeINVALID + 1,
	MPIControlMemoryTypeDEFAULT = MPIControlMemoryTypeUSER
} MPIControlMemoryType;


/* status */
typedef enum {
	MPIControlFanStatusFlagINVALID = -1,

	MPIControlFanStatusFlagSTATUS_NOT_AVAILABLE,	/* 0 */
	MPIControlFanStatusFlagFAN_OK,					/* 1 */
	MPIControlFanStatusFlagFAN_ERROR,				/* 2 */
	MPIControlFanStatusFlagOVER_TEMP_LIMIT,			/* 3 */

	MPIControlFanStatusFlagLAST,
	MPIControlFanStatusFlagFIRST = MPIControlFanStatusFlagINVALID + 1
} MPIControlFanStatusFlag;

#define	mpiControlFanStatusMaskBIT(flag)	(int)(0x1 << (flag))

typedef enum {
	MPIControlFanStatusMaskNONE	= 0x0,

	MPIControlFanStatusMaskSTATUS_NOT_AVAILABLE	= mpiControlFanStatusMaskBIT(MPIControlFanStatusFlagSTATUS_NOT_AVAILABLE),/* 0x00000001 */
	MPIControlFanStatusMaskFAN_OK				= mpiControlFanStatusMaskBIT(MPIControlFanStatusFlagFAN_OK),				/* 0x00000002 */
	MPIControlFanStatusMaskFAN_ERROR			= mpiControlFanStatusMaskBIT(MPIControlFanStatusFlagFAN_ERROR),			/* 0x00000004 */
	MPIControlFanStatusMaskOVER_TEMP_LIMIT		= mpiControlFanStatusMaskBIT(MPIControlFanStatusFlagOVER_TEMP_LIMIT),		/* 0x00000008 */	

	MPIControlFanStatusMaskALL	= mpiControlFanStatusMaskBIT(MPIControlFanStatusFlagLAST) - 1			/* 0x0000000F */
} MPIControlFanStatusMask;

typedef struct MPIControlStatus {
	MPIEventMask			eventMask;		/* see meiEventMaskCONTROL() */
	MPIControlFanStatusMask	fanStatus;
} MPIControlStatus;

typedef struct MPIControlStatusCacheStatistics {
	unsigned long	count;
	double			min;
	double			max;
	double			mean;
	double			standardDeviation;
} MPIControlStatusCacheStatistics;

#define MPIControlInMAX 6
#define MPIControlOutMAX 5

typedef enum {
	MPIControlMessageFIRST = mpiMessageID(MPIModuleIdCONTROL, 0),

	MPIControlMessageLIBRARY_VERSION,       /* Keep as first control message */
	MPIControlMessageADDRESS_INVALID,
	MPIControlMessageCONTROL_INVALID,
	MPIControlMessageCONTROL_NUMBER_INVALID,
	MPIControlMessageTYPE_INVALID,  
	MPIControlMessageINTERRUPTS_DISABLED,
	MPIControlMessageEXTERNAL_MEMORY_OVERFLOW,
	MPIControlMessageADC_COUNT_INVALID,                                                
	MPIControlMessageAXIS_COUNT_INVALID,   
	MPIControlMessageAXIS_FRAME_COUNT_INVALID,
	MPIControlMessageCAPTURE_COUNT_INVALID,                                    
	MPIControlMessageCOMPARE_COUNT_INVALID,                                    
	MPIControlMessageFILTER_COUNT_INVALID,                                     
	MPIControlMessageMOTION_COUNT_INVALID,                                     
	MPIControlMessageMOTOR_COUNT_INVALID,                                      
	MPIControlMessageSAMPLE_RATE_TO_LOW,
	MPIControlMessageSAMPLE_RATE_TO_HIGH,
	MPIControlMessageRECORDER_COUNT_INVALID,
	MPIControlMessageCOMPENSATOR_COUNT_INVALID,
	MPIControlMessageAXIS_RUNNING,
	MPIControlMessageRECORDER_RUNNING,
	MPIControlMessageRECORD_COUNT_MEMORY_OVERFLOW,
	MPIControlMessagePACK_ALIGNMENT,

	MPIControlMessageLAST
} MPIControlMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIControl MPI_DECL2
	mpiControlCreate(MPIControlType         type,
					 MPIControlAddress      *address);

MPI_DECL1 long MPI_DECL2
	mpiControlDelete(MPIControl control);

MPI_DECL1 long MPI_DECL2
	mpiControlValidate(MPIControl control);

/* Control */
MPI_DECL1 long MPI_DECL2
	mpiControlAddress(MPIControl        control,
					  MPIControlAddress *address);

MPI_DECL1 long MPI_DECL2
	mpiControlConfigGet(MPIControl       control,
						MPIControlConfig *config,
						void             *external);

MPI_DECL1 long MPI_DECL2
	mpiControlConfigSet(MPIControl       control,
						MPIControlConfig *config,
						void             *external);

MPI_DECL1 long MPI_DECL2
	mpiControlError(MPIControl control,
					long       *error);

MPI_DECL1 long MPI_DECL2
	mpiControlEventNotifyGet(MPIControl		control,
							MPIEventMask	*eventMask,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiControlEventNotifySet(MPIControl		control,
							MPIEventMask	eventMask,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiControlEventReset(MPIControl		control,
						MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	mpiControlFlashConfigGet(MPIControl       control,
							 void             *flash,
							 MPIControlConfig *config,
							 void             *external);

MPI_DECL1 long MPI_DECL2
	mpiControlFlashConfigSet(MPIControl       control,
							 void             *flash,
							 MPIControlConfig *config,
							 void             *external);

MPI_DECL1 long MPI_DECL2
	mpiControlInit(MPIControl control);

MPI_DECL1 long MPI_DECL2
	mpiControlInterruptEnable(MPIControl control,
							  long       enable);

MPI_DECL1 long MPI_DECL2
	mpiControlInterruptWait(MPIControl control,
							MPI_BOOL   *interrupted,
							MPIWait    timeout);

MPI_DECL1 long MPI_DECL2
	mpiControlInterruptWake(MPIControl control);

MPI_DECL1 long MPI_DECL2
	mpiControlDigitalIn(MPIControl   control,
						long			bitStart,
						long			bitCount,
						unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	mpiControlDigitalOutSet(MPIControl   control,
						long			bitStart,
						long			bitCount,
						unsigned long	state,
						MPI_BOOL		wait);

MPI_DECL1 long MPI_DECL2
	mpiControlDigitalOutGet(MPIControl   control,
						long			bitStart,
						long			bitCount,
						unsigned long	*state);

MPI_DECL1 long MPI_DECL2
	mpiControlMemory(MPIControl control,
					 void       **memory,
					 void       **external);

MPI_DECL1 long MPI_DECL2
	mpiControlMemoryAlloc(MPIControl           control,
						  MPIControlMemoryType type,
						  long                 count,
						  void                 **memory);

MPI_DECL1 long MPI_DECL2
	mpiControlMemoryCount(MPIControl           control,
						  MPIControlMemoryType type,
						  long                 *count);

MPI_DECL1 long MPI_DECL2
	mpiControlMemoryFree(MPIControl           control,
						 MPIControlMemoryType type,
						 long                 count,
						 void                 *memory);

MPI_DECL1 long MPI_DECL2
	mpiControlMemoryGet(MPIControl control,
						void       *dst,
						const void *src,
						long       count);

MPI_DECL1 long MPI_DECL2
	mpiControlMemorySet(MPIControl control,
						void       *dst,
						const void *src,
						long       count);

MPI_DECL1 long MPI_DECL2
	mpiControlReset(MPIControl control);

MPI_DECL1 long MPI_DECL2
	mpiControlStatus(MPIControl			control,
					 MPIControlStatus	*status,
					 void				*external);

MPI_DECL1 long MPI_DECL2
	mpiControlClientCountGet(MPIControl control, unsigned long* clientCount);

MPI_DECL1 long MPI_DECL2
	mpiControlClientCountSet(MPIControl control, unsigned long clientCount);

/* mpiControlStatusCacheIntervalGet:
 * Return the current cache refresh interval.
 * refreshInterval is in milliseconds. Zero means caching is disabled.
 */

typedef unsigned long MPIControlStatusCacheRefreshInterval;
MPI_DECL1 long MPI_DECL2
	mpiControlStatusCacheIntervalGet(MPIControl		control,
			MPIControlStatusCacheRefreshInterval	*refreshInterval);
	
/* mpiControlStatusCacheIntervalSet:
 * Set the current cache refresh interval.
 * This call implicitly enables or disables caching.
 * refreshInterval is in milliseconds. A zero refreshInterval disables caching.
 * The valid range for refreshInterval is from 0 to 1000.
 *
 */
MPI_DECL1 long MPI_DECL2
	mpiControlStatusCacheIntervalSet(MPIControl		control,
			MPIControlStatusCacheRefreshInterval	refreshInterval);

/* mpiControlStatusCacheRefresh:
 * The user calls this to force a refresh of the cached data.
 * This call does not enable the use of the cache. To do this call,
 * mpiControlStatusCacheIntervalSet.
 */
MPI_DECL1 long MPI_DECL2
	mpiControlStatusCacheRefresh(MPIControl	control);

MPI_DECL1 long MPI_DECL2
	mpiControlStatusCacheStatistics(MPIControl	control,
			MPIControlStatusCacheStatistics		*statistics);

MPI_DECL1 long MPI_DECL2
	mpiControlStatusCacheStatisticsReset(MPIControl control);

MPI_DECL1 long MPI_DECL2
	mpiControlType(MPIControl     control,
				   MPIControlType *type);

/* Reserved for future use */
typedef struct MPIControlUserRpcData {
    long    id;
    long    dataSize;
    void*   data;
} MPIControlUserRpcData;

MPI_DECL1 long MPI_DECL2
	mpiControlInitVerify(MPIControl	control,
						 long		methodVersion,
						 ...);

#define mpiControlInit(control)							 \
	mpiControlInitVerify(control,						 \
						 _MPI_INIT_CHECK_METHOD_VERSION, \
						 MPI_INTERFACE_VERSION,			 \
						 _MPI_PACK_ALIGNMENT_TEST)

#if defined(__cplusplus)
}
#endif

#endif
