/* drivemap.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_DRIVEMAP_H)
#define	_DRIVEMAP_H

/* Map handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEIDriveMap *MEIDriveMap;
#if defined(__cplusplus)
}
#endif

#include "meidef.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum {
	MEIDriveMapMessageFIRST = mpiMessageID(MEIModuleIdDRIVE_MAP, 0),

	MEIDriveMapMessageMAP_FILE_OPEN_ERROR,
	MEIDriveMapMessageMAP_FILE_FORMAT_INVALID,
	MEIDriveMapMessageNODE_NOT_FOUND_IN_MAP,
	MEIDriveMapMessageVERSION_NOT_FOUND_IN_MAP,
	MEIDriveMapMessageDRIVE_PARAM_READ_ONLY,

	MEIDriveMapMessageLAST
} MEIDriveMapMessage;

/* SqNode Drive Parameters */
typedef enum MEIDriveMapParamType {
    MEIDriveMapParamTypeSIGNED32,
    MEIDriveMapParamTypeSIGNED16,
    MEIDriveMapParamTypeSIGNED8,
    MEIDriveMapParamTypeUNSIGNED32,
    MEIDriveMapParamTypeUNSIGNED16,
    MEIDriveMapParamTypeUNSIGNED8,
    MEIDriveMapParamTypeHEX,
    MEIDriveMapParamTypeENUMERATED,
    MEIDriveMapParamTypeMASK,
    MEIDriveMapParamTypeCHARACTER,
    MEIDriveMapParamTypeSTRING,
    MEIDriveMapParamTypeSINGLE,
    MEIDriveMapParamTypeACTION
} MEIDriveMapParamType;

typedef enum MEIDriveMapParamAccess {
    MEIDriveMapParamAccessREAD_WRITE,
    MEIDriveMapParamAccessREAD_ONLY
} MEIDriveMapParamAccess;

#define MEIDriveMapParamInfoMAX_STRING_LENGTH	(256)
#define MEIDriveMapParamMAX_STRING_LENGTH	(256)

typedef struct MEIDriveMapParamInfo {
	long				    parameter;
	char				    name[MEIDriveMapParamInfoMAX_STRING_LENGTH];
	MEIDriveMapParamAccess access;
	MEIDriveMapParamType	type;
	char				    validValues[MEIDriveMapParamInfoMAX_STRING_LENGTH];
	long				    defaultValue;
	char				    help[MEIDriveMapParamInfoMAX_STRING_LENGTH];
} MEIDriveMapParamInfo;

typedef union {
	long           signed32;
	short          signed16;
	char           signed8;
	unsigned long  unsigned32;
	unsigned short unsigned16;
	unsigned char  unsigned8;
	unsigned long  enumerated;
	unsigned long  hex;
	unsigned long  mask;
	char           character;
	char           string[MEIDriveMapParamMAX_STRING_LENGTH+1];
	float          single;
} MEIDriveMapParamValue;

/* Create/Delete/Validate */

MPI_DECL1 MEIDriveMap MPI_DECL2
	meiDriveMapCreate(char		*fileName);

MPI_DECL1 long MPI_DECL2
	meiDriveMapDelete(MEIDriveMap	mapHandle);

MPI_DECL1 long MPI_DECL2
	meiDriveMapValidate(MEIDriveMap	mapHandle);

/* Identity */

MPI_DECL1 const char * MPI_DECL2
	meiDriveMapFileName(MEIDriveMap	mapHandle);

/* Methods */
MPI_DECL1 long MPI_DECL2
	meiDriveMapParamCount(MEIDriveMap	driveMap,
							char		*nodeName,
							char		*firmwareVersion,
							long		*paramsCount);

MPI_DECL1 long MPI_DECL2
	meiDriveMapParamList(MEIDriveMap	driveMap,
						char			*nodeName,
						char			*firmwareVersion,
						long			paramsCount,
						MEIDriveMapParamInfo *driveParamInfo);

MPI_DECL1 long MPI_DECL2
	meiDriveMapConfigCount(MEIDriveMap	driveMap,
							char		*nodeName,
							char		*firmwareVersion,
							long		*configCount);

MPI_DECL1 long MPI_DECL2
	meiDriveMapConfigList(MEIDriveMap	driveMap,
							char		*nodeName,
							char		*firmwareVersion,
							long		configCount,
							long		*configList);

#if defined(__cplusplus)
}
#endif

#endif
