/* element.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_ELEMENT_H)
#define	_ELEMENT_H

/* Element handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEIElement *MEIElement;
#if defined(__cplusplus)
}
#endif

#include "meidef.h"
#include "list.h"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 *	Element object
 *
 *	Available here for efficiency -
 *		similar to a C++ friend
 *		use macros at bottom
 */
typedef struct _MEIElement {
	_MEIObject	object;		/* MUST be first! */

	MEIObjectLink	link;
	MEIList			list;

	void	*handle;

	MPI_BOOL allocated;	/* free on delete */
} _MEIElement;

typedef enum {
	MEIElementMessageFIRST = mpiMessageID(MEIModuleIdELEMENT, 0),

	MEIElementMessageELEMENT_INVALID,

	MEIElementMessageLAST
} MEIElementMessage;

typedef enum {
	MEIElementTraceFIRST = MEITraceLAST << 1,

	MEIElementTraceLAST  = MEIElementTraceFIRST << 1
} MEIElementTrace;

/* Create/Delete/Validate */
MPI_DECL1 MEIElement MPI_DECL2
	meiElementCreate(MEIElement element,
					 void *object);

MPI_DECL1 long MPI_DECL2
	meiElementDelete(MEIElement element);

MPI_DECL1 long MPI_DECL2
	meiElementValidate(MEIElement element);

/* Element */
MPI_DECL1 MEIList MPI_DECL2
	meiElementList(MEIElement element);

MPI_DECL1 MEIElement MPI_DECL2
	meiElementNextGet(MEIElement element);

MPI_DECL1 long MPI_DECL2
	meiElementNextSet(MEIElement element,
					  MEIElement next);

MPI_DECL1 void * MPI_DECL2
	meiElementObject(MEIElement element);

MPI_DECL1 MEIElement MPI_DECL2
	meiElementPreviousGet(MEIElement element);

MPI_DECL1 long MPI_DECL2
	meiElementPreviousSet(MEIElement element,
						  MEIElement previous);

/*
 *	Macros for efficiency
 */

#define meiElementVALIDATE(element)	\
			((((element) != MPIHandleVOID) && \
			  (mpiMessageMODULE(meiObjectMessageGET((MEIObject)(element))) != MPIModuleIdMESSAGE) && \
			  (mpiMessageNUMBER(meiObjectMessageGET((MEIObject)(element))) == MPIMessageOK)) \
						? MPIMessageOK \
						: MEIElementMessageELEMENT_INVALID)

#if defined(MEI_VALIDATE)
#define meiElementLIST(element) \
			((meiElementVALIDATE(element) == MPIMessageOK) \
						? (element)->list \
						: MPIHandleVOID)

#define meiElementNEXT(element) \
			((meiElementVALIDATE(element) == MPIMessageOK) \
						? (element)->link.next \
						: MPIHandleVOID)

#define meiElementNextSET(element,nextElement) \
			((meiElementVALIDATE(element) == MPIMessageOK) \
						? ((element)->link.next = (nextElement), MPIMessageOK) \
						: MEIElementMessageELEMENT_INVALID)

#define	meiElementOBJECT(element) \
			((meiElementVALIDATE(element) == MPIMessageOK) \
						? (element)->handle \
						: NULL)

#define meiElementPREVIOUS(element) \
			((meiElementVALIDATE(element) == MPIMessageOK) \
						? (element)->link.previous \
						: MPIHandleVOID)

#define meiElementPreviousSET(element,previousElement) \
			((meiElementVALIDATE(element) == MPIMessageOK) \
						? ((element)->link.previous = (previousElement), MPIMessageOK) \
						: MEIElementMessageELEMENT_INVALID)
#else
#define meiElementLIST(element)		((element)->list)
#define meiElementNEXT(element)		((element)->link.next)
#define meiElementNextSET(element,nextElement) \
									((element)->link.next = (nextElement), MPIMessageOK)
#define	meiElementOBJECT(element)	((element)->handle)
#define meiElementPREVIOUS(element)	((element)->link.previous)
#define meiElementPreviousSET(element,previousElement) \
									((element)->link.previous = (previousElement), MPIMessageOK)
#endif

#if defined(__cplusplus)
}
#endif

#endif

