/* event.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_EVENT_H)
#define	_EVENT_H

/* Event handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIEvent *MPIEvent;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum {
	MPIEventTypeINVALID = -1,

	MPIEventTypeNONE,					/*  0 */

	/* Motor events */
	MPIEventTypeAMP_FAULT,				/*  1 */
	MPIEventTypeHOME,					/*  2 */
	MPIEventTypeLIMIT_ERROR,			/*  3 */
	MPIEventTypeLIMIT_HW_NEG,			/*  4 */
	MPIEventTypeLIMIT_HW_POS,			/*  5 */
	MPIEventTypeLIMIT_SW_NEG,			/*  6 */
	MPIEventTypeLIMIT_SW_POS,			/*  7 */
	MPIEventTypeENCODER_FAULT,			/*  8 */
	MPIEventTypeAMP_WARNING,			/*  9 */

	/* Motion events */
	MPIEventTypeMOTION_DONE,			/* 10 */
	MPIEventTypeMOTION_AT_VELOCITY,		/* 11 */

	/* Recorder events */
	MPIEventTypeRECORDER_HIGH,			/* 12 */
	MPIEventTypeRECORDER_FULL,			/* 13 */
	MPIEventTypeRECORDER_DONE,			/* 14 */

	/* External events */
	MPIEventTypeEXTERNAL,				/* 15 */

	MPIEventTypeLAST,
	MPIEventTypeFIRST = MPIEventTypeINVALID + 1,

	MPIEventTypeMOTOR_FIRST	= MPIEventTypeAMP_FAULT,
	MPIEventTypeMOTOR_LAST	= MPIEventTypeAMP_WARNING + 1,

	/* Events that can be filtered by the direction of motion */
	MPIEventTypeDIR_FIRST	= MPIEventTypeLIMIT_HW_NEG,	   
	MPIEventTypeDIR_LAST	= MPIEventTypeLIMIT_SW_POS + 1,  

	MPIEventTypeMOTION_FIRST	= MPIEventTypeMOTION_DONE,
	MPIEventTypeMOTION_LAST		= MPIEventTypeMOTION_AT_VELOCITY + 1,

	MPIEventTypeRECORDER_FIRST	= MPIEventTypeRECORDER_HIGH,
	MPIEventTypeRECORDER_LAST	= MPIEventTypeRECORDER_DONE + 1
} MPIEventType;

#define MPIEventMaskELEMENTS	(2)
typedef unsigned long MPIEventMaskELEMENT_TYPE;
typedef MPIEventMaskELEMENT_TYPE MPIEventMask[MPIEventMaskELEMENTS];

#define MEIEventMaskBITS_IN_ELEMENT			((unsigned long)(sizeof(MPIEventMaskELEMENT_TYPE) * 8))
#define mpiEventMaskBITS_IN_ELEMENT			(MEIEventMaskBITS_IN_ELEMENT) /* deprecated */
#define mpiEventMaskBIT_POSITION(type)		(((unsigned long)(type) < MEIEventMaskBITS_IN_ELEMENT) ? \
												(type) : \
												((unsigned long)(type) - MEIEventMaskBITS_IN_ELEMENT))
#define mpiEventMaskBIT_POSITION_MASK(type)	((MPIEventMaskELEMENT_TYPE)(0x1 << mpiEventMaskBIT_POSITION(type)))
#define mpiEventMaskWORD(type)				(((unsigned long)(type) < MEIEventMaskBITS_IN_ELEMENT) ? \
												(0) : \
												(1))

#define	mpiEventMaskBIT(mask,type)			(((mask)[0] = 0); ((mask)[1] = 0); \
												((mask)[mpiEventMaskWORD(type)] = mpiEventMaskBIT_POSITION_MASK(type)))


#define	mpiEventMaskAND_ASSIGN(dst,src)		((dst)[0] = ((dst)[0] & (src)[0])), \
												((dst)[1] = ((dst)[1] & (src)[1]))
#define	mpiEventMaskASSIGN(dst,src)			((dst)[0] = (src)[0]), \
												((dst)[1] = (src)[1])
#define	mpiEventMaskCLEAR(mask)				((mask)[0] = 0), \
												((mask)[1] = 0)
#define	mpiEventMaskCOMPLEMENT(mask)		((mask)[0] = ~(mask)[0]), \
												((mask)[1] = ~(mask)[1])

#define	mpiEventMaskOR_ASSIGN(dst,src)		((dst)[0] = ((dst)[0] | (src)[0])), \
												((dst)[1] = ((dst)[1] | (src)[1]))

#define	mpiEventMaskIS_CLEAR(mask)			(((mask)[0] == 0) && ((mask)[1] == 0))
#define	mpiEventMaskIS_EQUAL(mask1,mask2)	(((mask1)[0] == (mask2)[0]) && ((mask1)[1] == (mask2)[1]))


#define	mpiEventMaskBitGET(mask,bit)		(((mask)[mpiEventMaskWORD(bit)] & mpiEventMaskBIT_POSITION_MASK(bit)) ? 1 : 0)

#define	mpiEventMaskBitSET(mask,bit,value)	(((value) == 0) ? \
												((mask)[mpiEventMaskWORD(bit)] &= ~mpiEventMaskBIT_POSITION_MASK(bit)) : \
												((mask)[mpiEventMaskWORD(bit)] |= mpiEventMaskBIT_POSITION_MASK(bit)))

#define	mpiEventMaskGET(mask,type)			mpiEventMaskBitGET(mask,type)
#define	mpiEventMaskSET(mask,type)			mpiEventMaskBitSET(mask,type,1)

#define	mpiEventMaskSET_ALL(mask,type)		((mpiEventMaskWORD(type) == 0) ? \
												((mask)[0] |= mpiEventMaskBIT_POSITION_MASK(type) - 1) : \
										  		((mask)[0] |= 0xFFFFFFFF)), \
											((mpiEventMaskWORD(type) == 0) ? \
												((mask)[1] |= 0x0) : \
												((mask)[1] |= mpiEventMaskBIT_POSITION_MASK(type) - 1))

#define mpiEventMaskALL(mask)		mpiEventMaskSET_ALL((mask), MPIEventTypeLAST)
#define mpiEventMaskCONTROL(mask)	(mask)
#define mpiEventMaskAXIS(mask)		(mask)
#define mpiEventMaskEXTERNAL(mask)	mpiEventMaskBitSET((mask), MPIEventTypeEXTERNAL, 1)
#define mpiEventMaskMOTION(mask)	mpiEventMaskBitSET((mask), MPIEventTypeMOTION_DONE, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeMOTION_AT_VELOCITY, 1)
#define mpiEventMaskMOTOR(mask)		mpiEventMaskBitSET((mask), MPIEventTypeAMP_FAULT, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeHOME, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeLIMIT_ERROR, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeLIMIT_HW_NEG, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeLIMIT_HW_POS, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeLIMIT_SW_NEG, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeLIMIT_SW_POS, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeENCODER_FAULT, 1), \
										mpiEventMaskBitSET((mask), MPIEventTypeAMP_WARNING, 1)
#define mpiEventMaskRECORDER(mask)	mpiEventMaskBitSET((mask), MPIEventTypeRECORDER_HIGH, 1), \
									mpiEventMaskBitSET((mask), MPIEventTypeRECORDER_FULL, 1), \
									mpiEventMaskBitSET((mask), MPIEventTypeRECORDER_DONE, 1)


#define MPIEventStatusINFO_COUNT_MAX	(16)

typedef struct MPIEventStatus {
	MPIEventType	type;
	void			*source;

	long	info[MPIEventStatusINFO_COUNT_MAX];
} MPIEventStatus;

typedef enum {
	MPIEventMessageFIRST = mpiMessageID(MPIModuleIdEVENT, 0),

	MPIEventMessageEVENT_INVALID,

	MPIEventMessageLAST
} MPIEventMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIEvent MPI_DECL2
	mpiEventCreate(MPIEventStatus *status);

MPI_DECL1 long MPI_DECL2
	mpiEventDelete(MPIEvent event);

MPI_DECL1 long MPI_DECL2
	mpiEventValidate(MPIEvent event);

/* Event */
MPI_DECL1 long MPI_DECL2
	mpiEventStatusGet(MPIEvent			event,
					  MPIEventStatus	*status);

MPI_DECL1 long MPI_DECL2
	mpiEventStatusSet(MPIEvent			event,
					  MPIEventStatus	*status);

MPI_DECL1 const char* MPI_DECL2
    mpiEventTypeName(MPIEventType eventType);

#if defined(__cplusplus)
}
#endif

#endif
