/* meidef.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_MEIDEF_H)
#define _MEIDEF_H

/*
 *	This file defines symbols, typedefs, modules etc.
 *	for use with the internal MEI implementation of the MPI.
 *	
 *	This file is included by all MEI source and header files.
 */

#include "mpidef.h"

#if defined(MEI_ASSERT)
#define	meiASSERT(exp)	if ((exp) == FALSE) meiPlatformAssert(__FILE__, __LINE__)
#else
#define	meiASSERT(exp)
#endif

#if defined(MEI_STATIC)
#define	Static	MEI_STATIC
#else
#define	Static	static
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/* MEIAction */
typedef enum {
	MEIActionMAP = MPIActionLAST,

	MEIActionLAST,
	MEIActionFIRST = MPIActionLAST
} MEIAction;

typedef enum {
	MEIModuleIdPLATFORM = MPIModuleIdLAST,

	MEIModuleIdCAN,
	MEIModuleIdCLIENT,
	MEIModuleIdELEMENT,
	MEIModuleIdFLASH,
	MEIModuleIdLIST,
	MEIModuleIdMAP,
	MEIModuleIdPACKET,
	MEIModuleIdSERVER,
	MEIModuleIdSYNQNET,
	MEIModuleIdSQNODE,
	MEIModuleIdDRIVE_MAP,

	MEIModuleIdLAST,
	MEIModuleIdFIRST = MPIModuleIdLAST,

	MEIModuleIdBLOCK = MEIModuleIdSQNODE
} MEIModuleId;

/* Network */
typedef enum MEINetworkPort {
	MEINetworkPortINVALID = -1,

	MEINetworkPortOUT0,
	MEINetworkPortIN0,

	MEINetworkPortLAST,
	MEINetworkPortFIRST = MEINetworkPortINVALID + 1
} MEINetworkPort;

typedef enum MEINetworkType {
	MEINetworkTypeINVALID = -1,			/* no nodes found */

	MEINetworkTypeSTRING,
	MEINetworkTypeSTRING_DUAL,			
	MEINetworkTypeRING,

	MEINetworkTypeLAST,
	MEINetworkTypeFIRST = MEINetworkTypeINVALID + 1
} MEINetworkType;

typedef enum MEINetworkObjectType {
	MEINetworkObjectTypeINVALID = -1,

	MEINetworkObjectTypeNONE,			/* no object */
	MEINetworkObjectTypeTERMINATOR,	
	MEINetworkObjectTypeSQNODE,
	MEINetworkObjectTypeSYNQNET,			

	MEINetworkObjectTypeLAST,
	MEINetworkObjectTypeFIRST = MEINetworkObjectTypeINVALID + 1,
} MEINetworkObjectType;

typedef struct MEINetworkObjectInfo {
	MEINetworkObjectType type;
	long                 number;
} MEINetworkObjectInfo;

#include "trace.h"

#define	MEIModuleTraceALL	((MEITraceMask)(-1 & ~MEITraceALL))

typedef struct _MEIObject *MEIObject;

typedef struct MEIObjectLink {
	void	*next;
	void	*previous;
} MEIObjectLink;

typedef struct _MEIObject {
	long   			message;	/* MPIMessageXXX */
	MEITraceMask	trace;		/* MEITraceXXX */
	MPIWait			timeout;	/* MEI_MULTI */

	MEIObjectLink	link;		/* doubly-linked list (internal use) */
} _MEIObject;

typedef enum {
	MEIStatusFlagBROKEN_WIRE	= MPIStatusFlagLAST,	/* 0 */
	MEIStatusFlagILLEGAL_STATE, 						/* 1 */
	MEIStatusFlagABS_ENCODER_FAULT,						/* 2 */
	MEIStatusFlagABS_ENCODER_TIMEOUT,					/* 3 */
	MEIStatusFlagBROKEN_WIRE_SECONDARY,					/* 4 */
	MEIStatusFlagILLEGAL_STATE_SECONDARY,				/* 5 */

	MEIStatusFlagLAST,
	MEIStatusFlagFIRST = MPIStatusFlagLAST,

	MEIStatusFlagMOTOR_FIRST	= MEIStatusFlagBROKEN_WIRE,
	MEIStatusFlagMOTOR_LAST		= MEIStatusFlagABS_ENCODER_TIMEOUT + 1
} MEIStatusFlag;

typedef enum {
	MEIStatusMaskBROKEN_WIRE			= mpiStatusMaskBIT(MEIStatusFlagBROKEN_WIRE),			/* 0x00000002 */
	MEIStatusMaskILLEGAL_STATE			= mpiStatusMaskBIT(MEIStatusFlagILLEGAL_STATE),			/* 0x00000004 */
	MEIStatusMaskABS_ENCODER_FAULT		= mpiStatusMaskBIT(MEIStatusFlagABS_ENCODER_FAULT),		/* 0x00000008 */
	MEIStatusMaskABS_ENCODER_TIMEOUT	= mpiStatusMaskBIT(MEIStatusFlagABS_ENCODER_TIMEOUT), 	/* 0x00000010 */
	MEIStatusMaskBROKEN_WIRE_SECONDARY	= mpiStatusMaskBIT(MEIStatusFlagBROKEN_WIRE_SECONDARY),	/* 0x00000020 */
	MEIStatusMaskILLEGAL_STATE_SECONDARY	= mpiStatusMaskBIT(MEIStatusFlagILLEGAL_STATE_SECONDARY),	/* 0x00000040 */

	MEIStatusMaskMOTOR =																		/* 0x0000001E */
								   (MEIStatusMaskBROKEN_WIRE	|
									MEIStatusMaskILLEGAL_STATE |
									MEIStatusMaskABS_ENCODER_FAULT |
									MEIStatusMaskABS_ENCODER_TIMEOUT),

	MEIStatusMaskALL	=																		/* 0x0000001E */
		(mpiStatusMaskBIT(MEIStatusFlagLAST) - 1) & ~MPIStatusMaskALL
} MEIStatusMask;

typedef struct MEIStatus {
	long	word[8];
} MEIStatus;

#if defined(__cplusplus)
}
#endif

#endif
