/* meimap.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_MEIMAP_H)
#define	_MEIMAP_H

/* Map handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEIMap *MEIMap;
#if defined(__cplusplus)
}
#endif

#include "meidef.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define	MEIMapNameSizeMAX	(1024)

typedef enum MEIMapDataType {
	MEIMapDataTypeINVALID = -1,

	MEIMapDataTypeDOUBLE,
	MEIMapDataTypeFLOAT,
	MEIMapDataTypeLONG,
	MEIMapDataTypePOINTER,
	MEIMapDataTypeSTRUCT,
	MEIMapDataTypeUNION,

	MEIMapDataTypeLAST,
	MEIMapDataTypeFIRST = MEIMapDataTypeINVALID + 1
} MEIMapDataType;

#define	meiMapDataTypeTERMINAL(type) \
			(((type) != MEIMapDataTypeSTRUCT) && \
			 ((type) != MEIMapDataTypeUNION))

typedef enum  {
	MEIMapGroupINVALID = -1,

	MEIMapGroupMPI_AXIS_CONFIG,
	MEIMapGroupMEI_AXIS_CONFIG,
	MEIMapGroupMEI_CAN_CONFIG,
	MEIMapGroupMPI_CAPTURE_CONFIG,
	MEIMapGroupMPI_CONTROL_CONFIG,
	MEIMapGroupMEI_CONTROL_CONFIG,
	MEIMapGroupMPI_EVENTMGR_CONFIG,
	MEIMapGroupMEI_EVENTMGR_CONFIG,
	MEIMapGroupMPI_FILTER_CONFIG,
	MEIMapGroupMEI_FILTER_CONFIG,
	MEIMapGroupMPI_MOTION_CONFIG,
	MEIMapGroupMEI_MOTION_CONFIG,
	MEIMapGroupMPI_MOTOR_CONFIG,
	MEIMapGroupMEI_MOTOR_CONFIG,
	MEIMapGroupMPI_RECORDER_CONFIG,
	MEIMapGroupMEI_RECORDER_CONFIG,
	MEIMapGroupMPI_SEQUENCE_CONFIG,
	MEIMapGroupMEI_SEQUENCE_CONFIG,
									/* MEIMapGroupCONFIG_LAST */
	MEIMapGroupMEI_XMP_BUFFER_DATA,	/* MEIMapGroupXMP_FIRST */
	MEIMapGroupMEI_XMP_DATA,
	MEIMapGroupMEI_XMP_SYNQNET_DATA,
	MEIMapGroupMEI_XMP_PLD,

	MEIMapGroupMEI_XMP_RINCON_REG,

	MEIMapGroupMEI_XMP_CAN,

	MEIMapGroupMEI_XMP_FRAME_BUFFER,
	MEIMapGroupMEI_XMP_RECORD_BUFFER,
	MEIMapGroupMEI_XMP_PACKET_CONFIG,
	MEIMapGroupMEI_XMP_RINCON_BUFFER,
	MEIMapGroupMEI_XMP_RINCON_INTERNAL,
	MEIMapGroupMEI_XMP_ASYNQ_BUFFER,

									/* MEIMapGroupXMP_LAST */
	MEIMapGroupLAST,
	MEIMapGroupFIRST = MEIMapGroupINVALID + 1,

	MEIMapGroupCONFIG_FIRST = MEIMapGroupMPI_AXIS_CONFIG,
	MEIMapGroupCONFIG_LAST	= MEIMapGroupMEI_SEQUENCE_CONFIG + 1,

	MEIMapGroupXMP_FIRST	= MEIMapGroupMEI_XMP_BUFFER_DATA,
	MEIMapGroupXMP_LAST		= MEIMapGroupMEI_XMP_RINCON_REG + 1
} MEIMapGroup;

typedef struct MEIMapNameParams {
	MEIMapGroup	group;
	long		index;	/* -1 => single instance of group's structure */
	long		offset;	/* byte offset within group's structure */

	MEIMapDataType	type;

	void	*addressHost;		/* NULL => group is host-resident */
	void	*addressFirmware;	/* NULL => group is host-resident */
} MEIMapNameParams;

typedef enum {
	MEIMapMessageFIRST = mpiMessageID(MEIModuleIdMAP, 0),

	MEIMapMessageNAME_INVALID,
	MEIMapMessageNAME_NOT_FOUND,
	MEIMapMessageINDEX_INVALID,
	MEIMapMessageFILE_INVALID,

	MEIMapMessageLAST
} MEIMapMessage;

typedef enum {
	MEIMapTraceFIRST = MEITraceLAST << 1,

	MEIMapTraceLAST  = MEIMapTraceFIRST << 1
} MEIMapTrace;

/* Create/Delete/Validate */

MPI_DECL1 MEIMap MPI_DECL2
	meiMapCreate(MPIControl	control,
				 char		*fileName);

MPI_DECL1 long MPI_DECL2
	meiMapDelete(MEIMap	mapHandle);

MPI_DECL1 long MPI_DECL2
	meiMapValidate(MEIMap	mapHandle);

/* Identity */

MPI_DECL1 MPIControl MPI_DECL2
	meiMapControl(MEIMap	mapHandle);

MPI_DECL1 const char * MPI_DECL2
	meiMapFileName(MEIMap	mapHandle);

/* Map */

MPI_DECL1 long MPI_DECL2
	meiMapNameFromAddress(MEIMap			mapHandle,
						  char				*name,
						  void				*address,
						  MEIMapNameParams	*params);

MPI_DECL1 long MPI_DECL2
	meiMapNameFromParams(MEIMap				mapHandle,
						 char				*name,
						 MEIMapNameParams	*params);

MPI_DECL1 long MPI_DECL2
	meiMapNameToParams(MEIMap			mapHandle,
					   char				*name,
					   MEIMapNameParams	*params);

#if defined(__cplusplus)
}
#endif

#endif
