/* motion.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_MOTION_H)
#define	_MOTION_H

/* Motion handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIMotion *MPIMotion;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"
#include "axis.h"
#include "event.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef struct MPIMotionDecelTime {
	float	stop;	/* seconds */
	float	eStop;	/* seconds */
} MPIMotionDecelTime;	

typedef struct MPIMotionConfig {
	MPIMotionDecelTime	decelTime;
	float				normalFeedrate;
	float				pauseFeedrate;
} MPIMotionConfig;

/* Motion Attributes */
typedef enum {
	MPIMotionAttrINVALID			= -1,
	MPIMotionAttrLAST				= 32,

	MPIMotionAttrAPPEND				= MPIMotionAttrLAST - 1,
	MPIMotionAttrAUTO_START			= MPIMotionAttrLAST - 2,
	MPIMotionAttrDELAY				= MPIMotionAttrLAST - 3,
	MPIMotionAttrID					= MPIMotionAttrLAST - 4,
	MPIMotionAttrELEMENT_ID			= MPIMotionAttrLAST - 5,
	MPIMotionAttrRELATIVE			= MPIMotionAttrLAST - 6,
	MPIMotionAttrSYNC_END			= MPIMotionAttrLAST - 7,
	MPIMotionAttrSYNC_START			= MPIMotionAttrLAST - 8,
	MPIMotionAttrREPEAT				= MPIMotionAttrLAST - 9,
	MPIMotionAttrMASTER_START		= MPIMotionAttrLAST - 10,
	MPIMotionAttrNO_BACKTRACK		= MPIMotionAttrLAST - 11,
	MPIMotionAttrNO_BACKTRACK_HOLD	= MPIMotionAttrLAST - 12,

	MPIMotionAttrFIRST				= MPIMotionAttrLAST - 12,
	MPIMotionAttrCOUNT				= MPIMotionAttrLAST - MPIMotionAttrFIRST
} MPIMotionAttr;

#define	mpiMotionAttrMaskBIT(attr)	(0x1 << (attr))

typedef	enum {
	MPIMotionAttrMaskAPPEND				= mpiMotionAttrMaskBIT(MPIMotionAttrAPPEND),
	MPIMotionAttrMaskAUTO_START			= mpiMotionAttrMaskBIT(MPIMotionAttrAUTO_START),
	MPIMotionAttrMaskDELAY				= mpiMotionAttrMaskBIT(MPIMotionAttrDELAY),
	MPIMotionAttrMaskID					= mpiMotionAttrMaskBIT(MPIMotionAttrID),
	MPIMotionAttrMaskELEMENT_ID			= mpiMotionAttrMaskBIT(MPIMotionAttrELEMENT_ID),
	MPIMotionAttrMaskRELATIVE			= mpiMotionAttrMaskBIT(MPIMotionAttrRELATIVE),
	MPIMotionAttrMaskSYNC_END			= mpiMotionAttrMaskBIT(MPIMotionAttrSYNC_END),
	MPIMotionAttrMaskSYNC_START			= mpiMotionAttrMaskBIT(MPIMotionAttrSYNC_START),
	MPIMotionAttrMaskREPEAT				= mpiMotionAttrMaskBIT(MPIMotionAttrREPEAT),
	MPIMotionAttrMaskMASTER_START		= mpiMotionAttrMaskBIT(MPIMotionAttrMASTER_START),
	MPIMotionAttrMaskNO_BACKTRACK		= mpiMotionAttrMaskBIT(MPIMotionAttrNO_BACKTRACK),
	MPIMotionAttrMaskNO_BACKTRACK_HOLD	= mpiMotionAttrMaskBIT(MPIMotionAttrNO_BACKTRACK_HOLD),

	MPIMotionAttrMaskALL				= -1 << MPIMotionAttrFIRST
} MPIMotionAttrMask;

typedef struct MPIMotionAttributes {
	double	*delay;			/* MPIMotionAttrMaskDELAY */
	long	id;				/* MPIMotionAttrMaskID */
	long	*elementId;		/* MPIMotionAttrMaskELEMENT_ID */
    double	masterStart;    /* MPIMotionAttrMaskMASTER_START */
	long	repeatFrom;		/* MPIMotionAttrMaskREPEAT */
} MPIMotionAttributes;

/* Basic Motion Types */

typedef enum {
	MPIMotionTypeINVALID = -1,

	MPIMotionTypePT,
	MPIMotionTypePTF,
	MPIMotionTypePVT,
	MPIMotionTypePVTF,
	MPIMotionTypeSPLINE,
	MPIMotionTypeBESSEL,
	MPIMotionTypeBSPLINE,
	MPIMotionTypeBSPLINE2,

	MPIMotionTypeS_CURVE,
	MPIMotionTypeTRAPEZOIDAL,
	MPIMotionTypeS_CURVE_JERK,

	MPIMotionTypeVELOCITY,
	MPIMotionTypeVELOCITY_JERK,

	MPIMotionTypeCAM_LINEAR,
	MPIMotionTypeCAM_CUBIC,

#if 0
	/* Reserved for future use */
	MPIMotionTypeP,
	MPIMotionTypePF,
	MPIMotionTypeJOG,
	MPIMotionTypePARABOLIC,
	MPIMotionTypeCOORD_ARC,
	MPIMotionTypeCOORD_ARC_FINAL_RADIUS,
	MPIMotionTypeCOORD_HELICAL,
	MPIMotionTypeCOORD_LINEAR,
#endif

	MPIMotionTypeLAST,
	MPIMotionTypeFIRST	= MPIMotionTypeINVALID + 1,
	MPIMotionTypeMASK	= 0xFF
} MPIMotionType;

#define	mpiMotionTYPE(type)			((type) & MPIMotionTypeMASK)
#define mpiMotionATTR(type,attr)	((type) |= mpiMotionAttrMaskBIT(attr))

/* MPIMotionParams structures for MPIMotionType */

typedef struct MPIMotionPoint {
	MPI_BOOL retain;		/* FALSE => flush points after use */
	MPI_BOOL final;		/* FALSE => more points to come */
	long	 emptyCount;	/* # of remaining points to trigger empty event, -1 => disable */
} MPIMotionPoint;

typedef	struct MPIMotionPT {
	long		pointCount;
	double		*position;
	double		*time;
	
	MPIMotionPoint	point;
} MPIMotionPT;

typedef	struct MPIMotionPTF {
	long		pointCount;
	double		*position;
	double		*feedforward;
	double		*time;
	
	MPIMotionPoint	point;
} MPIMotionPTF;

typedef	struct MPIMotionPVT {
	long		pointCount;
	double		*position;
	double		*velocity;
	double		*time;

	MPIMotionPoint	point;
} MPIMotionPVT;

typedef	struct MPIMotionPVTF {
	long		pointCount;
	double		*position;
	double		*velocity;
	double		*feedforward;
	double		*time;
	
	MPIMotionPoint	point;
} MPIMotionPVTF;

typedef	struct MPIMotionSPLINE {
	long		pointCount;
	double		*position;
	double		*time;

	MPIMotionPoint	point;
} MPIMotionSPLINE;

typedef	struct MPIMotionBESSEL {
	long		pointCount;
	double		*position;
	double		*time;

	MPIMotionPoint	point;
} MPIMotionBESSEL;

typedef	struct MPIMotionBSPLINE {
	long		pointCount;
	double		*position;
	double		*time;

	MPIMotionPoint	point;
} MPIMotionBSPLINE;

typedef struct MPIMotionSCurve {
	MPITrajectory	*trajectory;
	double	*position;
} MPIMotionSCurve;

typedef MPIMotionSCurve MPIMotionSCurveJerk;

typedef MPIMotionSCurve MPIMotionTrapezoidal;

typedef struct MPIMotionVelocity {
	MPITrajectory	*trajectory;
} MPIMotionVelocity;

typedef struct MPIMotionCam {
	long	pointCount;
	double	**slavePosition;
	double	*masterDistance;
	double	**gearRatio;		/* This field is only used with MPIMotionTypeCUBIC_CAM. */
} MPIMotionCam;

typedef	struct MPIMotionParams {
	MPIMotionPT				pt;
	MPIMotionPTF   			ptf;
	MPIMotionPVT			pvt;
	MPIMotionPVTF			pvtf;
	MPIMotionSPLINE			spline;
	MPIMotionBESSEL			bessel;
	MPIMotionBSPLINE	  	bspline;

	MPIMotionSCurve			sCurve;
	MPIMotionSCurve			sCurveJerk;
	MPIMotionTrapezoidal 	trapezoidal;

	MPIMotionVelocity		velocity;
	MPIMotionVelocity		velocityJerk;

	MPIMotionCam			cam;

	MPIMotionAttributes	attributes;

	void	*external;
} MPIMotionParams;

typedef enum {
	MPIMotionMessageFIRST = mpiMessageID(MPIModuleIdMOTION, 0),

	MPIMotionMessageMOTION_INVALID,
	MPIMotionMessageAXIS_NOT_FOUND,
	MPIMotionMessageAXIS_COUNT,
	MPIMotionMessageAXIS_FRAME_COUNT,
	MPIMotionMessageTYPE_INVALID,
	MPIMotionMessageATTRIBUTE_INVALID,
	MPIMotionMessageIDLE,
	MPIMotionMessageMOVING,
	MPIMotionMessageSTOPPING,
	MPIMotionMessageSTOPPED,
	MPIMotionMessageSTOPPING_ERROR,
	MPIMotionMessageERROR,
	MPIMotionMessageAUTO_START,
	MPIMotionMessageILLEGAL_DELAY,
	MPIMotionMessagePROFILE_ERROR,
	MPIMotionMessagePROFILE_NOT_SUPPORTED,
	MPIMotionMessagePATH_ERROR,
	MPIMotionMessageFRAMES_LOW,
	MPIMotionMessageFRAMES_EMPTY,
	MPIMotionMessageFRAME_BUFFER_TOO_SMALL,

	MPIMotionMessageLAST
} MPIMotionMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIMotion MPI_DECL2
	mpiMotionCreate(MPIControl	control,
					long		number,
					MPIAxis		axis);

MPI_DECL1 long MPI_DECL2
	mpiMotionDelete(MPIMotion motion);

MPI_DECL1 long MPI_DECL2
	mpiMotionValidate(MPIMotion motion);

/* Motion */
MPI_DECL1 long MPI_DECL2
	mpiMotionAction(MPIMotion	motion,
					MPIAction	action);

MPI_DECL1 long MPI_DECL2
	mpiMotionConfigGet(MPIMotion		motion,
					   MPIMotionConfig	*config,
					   void				*external);

MPI_DECL1 long MPI_DECL2
	mpiMotionConfigSet(MPIMotion		motion,
					   MPIMotionConfig	*config,
					   void				*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiMotionControl(MPIMotion motion);

MPI_DECL1 long MPI_DECL2
	mpiMotionEventNotifyGet(MPIMotion		motion,
							MPIEventMask	*eventMask,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiMotionEventNotifySet(MPIMotion		motion,
							MPIEventMask	eventMask,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiMotionEventReset(MPIMotion		motion,
						MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	mpiMotionFlashConfigGet(MPIMotion		motion,
							void			*flash,
							MPIMotionConfig	*config,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiMotionFlashConfigSet(MPIMotion		motion,
							void			*flash,
							MPIMotionConfig	*config,
							void			*external);

MPI_DECL1 long MPI_DECL2
	mpiMotionMemory(MPIMotion	motion,
					void		**memory);

MPI_DECL1 long MPI_DECL2
	mpiMotionMemoryGet(MPIMotion	motion,
					   void			*dst,
					   const void	*src,
					   long			count);

MPI_DECL1 long MPI_DECL2
	mpiMotionMemorySet(MPIMotion	motion,
					   void			*dst,
					   const void	*src,
					   long			count);

MPI_DECL1 long MPI_DECL2
	mpiMotionModify(MPIMotion		motion,
					MPIMotionType	type,
					MPIMotionParams	*params);

#define _mpiMotionStart(a,b,c) _meiMotionExecute(a,b,c,TRUE)
#define _mpiMotionModify(a,b,c) _meiMotionExecute(a,b,c,FALSE)

MPI_DECL1 long MPI_DECL2
	_meiMotionExecute(MPIMotion		motion,
					MPIMotionType	type,
					MPIMotionParams	*params,
					long			startFlag);

MPI_DECL1 long MPI_DECL2
	mpiMotionNumber(MPIMotion	motion,
					long		*number);

MPI_DECL1 long MPI_DECL2
	mpiMotionParamsGet(MPIMotion		motion,
					   MPIMotionParams	*params);

MPI_DECL1 long MPI_DECL2
	mpiMotionParamsSet(MPIMotion		motion,
					   MPIMotionParams	*params);

MPI_DECL1 long MPI_DECL2
	mpiMotionPositionGet(MPIMotion	motion,
						 double		*actual,
						 double		*command);

MPI_DECL1 long MPI_DECL2
	mpiMotionPositionSet(MPIMotion	motion,
						 double		*actual,
						 double		*command);

MPI_DECL1 long MPI_DECL2
	mpiMotionStart(MPIMotion		motion,
				   MPIMotionType	type,
				   MPIMotionParams	*params);

MPI_DECL1 long MPI_DECL2
	mpiMotionStatus(MPIMotion	motion,
					MPIStatus	*status,
					void		*external);

MPI_DECL1 long MPI_DECL2
	mpiMotionTrajectory(MPIMotion		motion,
					    MPITrajectory	*trajectory);

/* List */
MPI_DECL1 MPIAxis MPI_DECL2
	mpiMotionAxis(MPIMotion	motion,
				 long		index);

MPI_DECL1 long MPI_DECL2
	mpiMotionAxisAppend(MPIMotion	motion,
						MPIAxis		axis);

MPI_DECL1 long MPI_DECL2
	mpiMotionAxisCount(MPIMotion motion);

MPI_DECL1 MPIAxis MPI_DECL2
	mpiMotionAxisFirst(MPIMotion motion);

MPI_DECL1 long MPI_DECL2
	mpiMotionAxisIndex(MPIMotion	motion,
					   MPIAxis		axis);

MPI_DECL1 long MPI_DECL2
	mpiMotionAxisInsert(MPIMotion	motion,
						MPIAxis		axis,
						MPIAxis		insert);

MPI_DECL1 MPIAxis MPI_DECL2
	mpiMotionAxisLast(MPIMotion motion);

MPI_DECL1 long MPI_DECL2
	mpiMotionAxisListGet(MPIMotion	motion,
						 long		*axisCount,
						 MPIAxis	*axisList);

MPI_DECL1 long MPI_DECL2
	mpiMotionAxisListSet(MPIMotion	motion,
						 long		axisCount,
						 MPIAxis	*axisList);

MPI_DECL1 MPIAxis MPI_DECL2
	mpiMotionAxisNext(MPIMotion	motion,
					  MPIAxis	axis);

MPI_DECL1 MPIAxis MPI_DECL2
	mpiMotionAxisPrevious(MPIMotion	motion,
						  MPIAxis	axis);

MPI_DECL1 long MPI_DECL2
	mpiMotionAxisRemove(MPIMotion	motion,
						MPIAxis		axis);

#if defined(__cplusplus)
}
#endif

#endif
