/* platform.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_PLATFORM_H)
#define _PLATFORM_H

#include <stdarg.h>

/* Platform handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MEIPlatform	*MEIPlatform;
#if defined(__cplusplus)
}
#endif

#if defined(MEI_PLATFORM_WINNT) || \
	defined(MEI_PLATFORM_WIN95) || \
	defined(MEI_PLATFORM_WINRTSS)

/*#define MEIInt64 __int64*/
typedef __int64 MEIInt64;

#else

/*#define MEIInt64 long long*/
typedef long long MEIInt64;

#endif

#include "meidef.h"
#include "control.h"
#include "flash.h"
#include "message.h"
#include "synq.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define  MEIPlatformSocketInfoNA	(0xFFFFFFFF)
#define  MEIPlatformFlashKeySize	(7)  /* "?MP\n\r\0\x1A" */
#define  MEIPlatformFileOpenError	(0)
#define  MEIPlatformControlCountMax	(8) /* maximum number of controllers the driver will currently support */

#define swapBytes(x)    (((x) << 24) |\
                        (((x) & 0x0000ff00) << 8) |\
                        (((x) & 0x00ff0000) >> 8) |\
                        (((unsigned long)(x)) >> 24))
	
#define	MEIPlatformPldREG_SELECT		(0x2)

typedef enum {	/* These are 1 byte wide */
	MEIPlatformEEPromTableTypeNONE   = (int)0x0,
	MEIPlatformEEPromTableTypeSOCKET = (int)0x1,
	MEIPlatformEEPromTableTypeNA     = (int)0xFF
} MEIPlatformEEPromTableType;


typedef enum {
	MEIPlatformBoardTypeINVALID = -1,

	MEIPlatformBoardTypeUNKNOWN,
	MEIPlatformBoardTypeXMP,
	MEIPlatformBoardTypeZMP,
	
	MEIPlatformBoardTypeLAST,
	MEIPlatformBoardTypeFIRST = MEIPlatformBoardTypeINVALID + 1
} MEIPlatformBoardType;

typedef enum {
	MEIPlatformFileModeREAD		= (int)(0x0),		/* default */
	MEIPlatformFileModeWRITE	= (int)(0x1 << 0),

	MEIPlatformFileModeTEXT		= (int)(0x0),		/* default */
	MEIPlatformFileModeBINARY	= (int)(0x1 << 1),

	MEIPlatformFileModeTRUNC	= (int)(0x1 << 2),
	MEIPlatformFileModeAPPEND	= (int)(0x1 << 3)

} MEIPlatformFileMode;

/* Bit 4 of the HardwareSocketType is used to determine which FPGAFileType to load */
typedef enum {
	MEIPlatformHardwareSocketTypeNONE      = (int)0x0,
	MEIPlatformHardwareSocketTypeRINCON    = (int)0x12,
	MEIPlatformHardwareSocketTypeRINCONZ   = (int)0x14,
	MEIPlatformHardwareSocketTypeNonMBMask = (int)0x10
} MEIPlatformHardwareSocketType;

/* MEIHardwareICType is used as an index into MEIFPGAFilesXXX[] */
typedef enum {
	MEIPlatformHardwareICTypeNONE = (int)0x0,
	MEIPlatformHardwareICTypeASIC = (int)0x7,
	/* for MEIPlatformHardwareSocketTypeRINCON */
	MEIPlatformHardwareICTypeXC2S50 = (int)0x1,
	MEIPlatformHardwareICTypeXC2S100 = (int)0x2,
	MEIPlatformHardwareICTypeXC2S150 = (int)0x3,
	MEIPlatformHardwareICTypeXC2S200 = (int)0x4,
	MEIPlatformHardwareICTypeXC2S50BGA = (int)0x5,
	MEIPlatformHardwareICTypeXC2S100BGA = (int)0x6,
	/* for MEIPlatformHardwareSocketTypeRINCONZ */
	MEIPlatformHardwareICTypeXC2S300 = (int)0x1,
	MEIPlatformHardwareICTypeXC2S400 = (int)0x2,
	MEIPlatformHardwareICTypeXC3S200 = (int)0x3,
	MEIPlatformHardwareICTypeXC3S400 = (int)0x4,
	MEIPlatformHardwareICTypeXC3S1000 = (int)0x5
} MEIPlatformHardwareICType;

typedef struct MEIPlatformSocketInfo {
	long sockets;
	MEIPlatformHardwareSocketType socketType;
	MEIPlatformHardwareICType ICType;
} MEIPlatformSocketInfo;

#define	MEIPlatformInfoCHAR_MAX	(128)
typedef struct MEIPlatformInfo {
	char OSInfo[MEIPlatformInfoCHAR_MAX];
	char CPUInfo[MEIPlatformInfoCHAR_MAX];
	long CPUMHz;
} MEIPlatformInfo;

typedef enum {
	MEIPlatformFPGAOptionID_NONE = (int)0x0,
	/* for MEIControlHardwareSocketTypeRINCON */
	MEIPlatformFPGAOptionID_RINCON = (int)0x1,
	/* for MEIControlHardwareSocketTypeRINCONZ */
	MEIPlatformFPGAOptionID_RINCONZ_PCI = (int)0x1,
	MEIPlatformFPGAOptionID_RINCONZ = (int)0x2
} MEIPlatformFPGAOptionID;

typedef enum {
	MEIPlatformMessageFIRST = mpiMessageID(MEIModuleIdPLATFORM, 0),

	MEIPlatformMessagePLATFORM_INVALID,
	MEIPlatformMessageDEVICE_INVALID,
	MEIPlatformMessageDEVICE_ERROR,
	MEIPlatformMessageDEVICE_MAP_ERROR,
	MEIPlatformMessageCOPY64_FAILURE,
	MEIPlatformMessageCLIENT_ACCESS,
	MEIPlatformMessageALREADY_INITIALIZED,

	MEIPlatformMessageLAST
} MEIPlatformMessage;

typedef enum {
	MEIPlatformTraceFIRST = MEITraceLAST << 1,

	MEIPlatformTraceINIT = MEIPlatformTraceFIRST << 0,
	MEIPlatformTraceQUIT = MEIPlatformTraceFIRST << 1,

	MEIPlatformTraceLAST  = MEIPlatformTraceFIRST << 2
} MEIPlatformTrace;

/* Create/Delete/Validate */
MPI_DECL1 long MPI_DECL2
	meiPlatformValidate(MEIPlatform platform);

/* Platform */

MPI_DECL1 long MPI_DECL2
	meiPlatformBoardType(MEIPlatform			platform,
						 MEIPlatformBoardType	*boardType);

MPI_DECL1 long MPI_DECL2
	meiPlatformDriverVersion(MEIPlatform platform,
							 char		 **version);

MPI_DECL1 long MPI_DECL2
	meiPlatformError(MEIPlatform	platform,
					 long			*error);

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareEnable(MEIPlatform	platform,
							  MPI_BOOL		enable);

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareMemoryGet(MEIPlatform	platform,
						         void			*dst,
						         const void		*src,
						         long			count);

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareMemorySet(MEIPlatform	platform,
						         void			*dst,
						         const void		*src,
						         long			count);

MPI_DECL1 long MPI_DECL2
	meiPlatformFlashConfigGet(MEIPlatform		platform,
							  MEIFlashConfig	*config);

MPI_DECL1 long MPI_DECL2
    meiPlatformFlashCopy(MEIPlatform platform,
                         void *dst,
                         void *src,
                         long  count);

MPI_DECL1 long MPI_DECL2
	meiPlatformFlashMemoryGet(MEIPlatform	platform,
							  void			*dst,
							  const void	*src,
							  long			count);

MPI_DECL1 long MPI_DECL2
	meiPlatformFlashMemorySet(MEIPlatform	platform,
							  void			*dst,
							  const void	*src,
							  long			count);

MPI_DECL1 long MPI_DECL2
	meiPlatformFlashMemoryWriteSector(MEIPlatform		platform,
										unsigned char	*dst,
										unsigned char	*src);

MPI_DECL1 long MPI_DECL2
	meiPlatformFlashSectorErase(MEIPlatform platform,
								long		sector,
								MPI_BOOL	erase,
								MPI_BOOL	verify);

MPI_DECL1 long MPI_DECL2
	meiPlatformInit(MEIPlatform platform);

MPI_DECL1 long MPI_DECL2
	meiPlatformInterruptEnable(MEIPlatform	platform,
							   long			enable);

MPI_DECL1 long MPI_DECL2
	meiPlatformInterruptWait(MEIPlatform	platform,
							 MPI_BOOL		*interrupted,
							 MPIWait		timeout);

MPI_DECL1 long MPI_DECL2
	meiPlatformInterruptWake(MEIPlatform platform);

MPI_DECL1 long MPI_DECL2
	meiPlatformMapController(MEIPlatform platform, 
							void *baseAddress, 
							long region);

MPI_DECL1 long MPI_DECL2
	meiPlatformMemory(MEIPlatform	platform,
					  void			**base,
					  void			**memory,
					  void			**external);

MPI_DECL1 long MPI_DECL2
	meiPlatformMemoryGet(MEIPlatform	platform,
						 void			*dst,
						 const void		*src,
						 long			count); 

/* for getting position values */
MPI_DECL1 long MPI_DECL2
	meiPlatformMemoryGet64(MEIPlatform	platform,
						 void			*dst,
						 const void		*src);

MPI_DECL1 long MPI_DECL2
	meiPlatformMemorySet(MEIPlatform	platform,
						 void			*dst,
						 const void		*src,
						 long			count); 

/* for setting position values */
MPI_DECL1 long MPI_DECL2
	meiPlatformMemorySet64(MEIPlatform	platform,
						 void			*dst,
						 const void		*src);

MPI_DECL1 long MPI_DECL2
	meiPlatformWord64Orient(MEIPlatform	platform,
							 MEIInt64	*dst,
							 MEIInt64 	*src);

MPI_DECL1 long MPI_DECL2
	meiPlatformMemoryCounters(MEIPlatform	platform,
						   long *readBytes,
						   long *writeBytes);

MPI_DECL1 long MPI_DECL2
	meiPlatformMemoryToFirmware(MEIPlatform	platform,
								const void	*host,
								void		**address);

MPI_DECL1 long MPI_DECL2
	meiPlatformMemoryToHost(MEIPlatform	platform,
							const void	*firmware,
							void		**host);

MPI_DECL1 long MPI_DECL2
	meiPlatformObjectLockGive(MEIPlatform	platform,
							  MPIModuleId	moduleId,
							  long			number);

MPI_DECL1 long MPI_DECL2
	meiPlatformObjectLockTake(MEIPlatform	platform,
							  MPIModuleId	moduleId,
							  long			number,
							  MPIWait		timeout);

MPI_DECL1 long MPI_DECL2
	meiPlatformPciWindowAddress(MEIPlatform	platform, 
							    long		windowNumber,
								void		**address);

MPI_DECL1 long MPI_DECL2
	meiPlatformPort(MEIPlatform		platform,
					unsigned short	*port);

MPI_DECL1 unsigned char MPI_DECL2
	meiPlatformPortInChar(MEIPlatform		platform,
						  unsigned short	port);

MPI_DECL1 unsigned long MPI_DECL2
	meiPlatformPortInLong(MEIPlatform		platform,
						  unsigned short	port);

MPI_DECL1 unsigned short MPI_DECL2
	meiPlatformPortInShort(MEIPlatform		platform,
						   unsigned short	port);

MPI_DECL1 void MPI_DECL2
	meiPlatformPortOutChar(MEIPlatform		platform,
						   unsigned short	port,
						   unsigned char	value);

MPI_DECL1 void MPI_DECL2
	meiPlatformPortOutLong(MEIPlatform		platform,
						   unsigned short	port,
						   unsigned long	value);

MPI_DECL1 void MPI_DECL2
	meiPlatformPortOutShort(MEIPlatform		platform,
						    unsigned short	port,
						    unsigned short	value);

MPI_DECL1 long MPI_DECL2
	meiPlatformReset(MEIPlatform platform);

/* Functions */
MPI_DECL1 void * MPI_DECL2
	meiPlatformAllocMethod(long        size,
						   long        line,
						   const char* fileName);

#define meiPlatformAlloc(size) \
	meiPlatformAllocMethod(size, __LINE__, __FILE__)

MPI_DECL1 void MPI_DECL2
	meiPlatformAssert(const char	*file,
					  long			line);

MPI_DECL1 void MPI_DECL2
	meiPlatformAssertSet(void (MPI_DEF2 *func)(const char *file, long line));

MPI_DECL1 double MPI_DECL2
	meiPlatformAtof(const char *ascii);

MPI_DECL1 long MPI_DECL2
	meiPlatformAtol(const char *ascii);

MPI_DECL1 long /* MPI_DECL2 */
	meiPlatformConsole(const char *format, ...);

MPI_DECL1 void MPI_DECL2
	meiPlatformControlDefault(MPIControlType	*type,
							  MPIControlAddress	*address);

MPI_DECL1 long MPI_DECL2
	meiPlatformExmpTempInit(MEIPlatform		platform);

MPI_DECL1 long MPI_DECL2
	meiPlatformExmpTempGet(MEIPlatform		platform,
						   long*			temp);

MPI_DECL1 long MPI_DECL2
	meiPlatformFileClose(long file);

MPI_DECL1 long MPI_DECL2
	meiPlatformFileOpen(const char			*fileName,
						MEIPlatformFileMode	mode);

MPI_DECL1 long MPI_DECL2
	meiPlatformFileOpenValidate(long file);

MPI_DECL1 long MPI_DECL2
	meiPlatformFileRead(long	file,
						char	*buffer,
						long	byteCount);

MPI_DECL1 long MPI_DECL2
	meiPlatformFileWrite(long		file,
						 const char	*buffer,
						 long		byteCount);

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareAddress16To32(MEIPlatform	platform,
									void			**firmwareAddress);

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareAddress32To16(MEIPlatform	platform,
									void			**firmwareAddress,
									long			lowWord);

/* calculates base address of 64 bit entity given a pointer to one of the 32 bit words within the 64 bit entity */
MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareAddress32To64(MEIPlatform	platform,
									void			**firmwareAddress,	/* address of 32 bit entity */
									long			lowWord);			/* set to 1 if given address is the low word */

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareAddress64To32(MEIPlatform	platform,
									void			**firmwareAddress,	/* address of generic64 entity */
									long			lowWord);

/* calculates base address of 64 bit entity given a pointer to one of the 32 bit words within the 64 bit entity */
MPI_DECL1 long MPI_DECL2
	meiPlatformHostAddress32To64(MEIPlatform	platform,
									void		**hostAddress,		/* address of 32 bit entity */
									long		lowWord);			/* set to 1 if given address is the low word */

MPI_DECL1 long MPI_DECL2
	meiPlatformHostAddress64To32(MEIPlatform	platform,
				   				void			**hostAddress,	/* address of generic64 entity */
				   				long			lowWord);

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareAddressModify(MEIPlatform	platform,
									void			**firmwareAddress,
									long			byteCount);

MPI_DECL1 long MPI_DECL2
	meiPlatformFirmwareAddressModify16(MEIPlatform	platform,
									void			**firmwareAddress,
									long			byteCount);

MPI_DECL1 long MPI_DECL2
	meiPlatformFlashFileCheck(MEIPlatform	platform,
								long		file);

MPI_DECL1 long MPI_DECL2
	meiPlatformFlashKeyWrite(MEIPlatform	platform,
								long		file);

MPI_DECL1 long MPI_DECL2
	meiPlatformFreeMethod(void*	      alloc,
						  long	      size,
						  long        line,
						  const char* fileName);

#define meiPlatformFree(alloc, size) \
	meiPlatformFreeMethod(alloc, size, __LINE__, __FILE__)

MPI_DECL1 char * MPI_DECL2
	meiPlatformGetenv(const char *variable);

MPI_DECL1 long MPI_DECL2
	meiPlatformKey(MPIWait wait);

MPI_DECL1 long MPI_DECL2
	meiPlatformLockCreate(const char	*name,
						  void			**lock);

MPI_DECL1 long MPI_DECL2
	meiPlatformLockDelete(void *lock);

MPI_DECL1 long MPI_DECL2
	meiPlatformLockGive(void *lock);

MPI_DECL1 long MPI_DECL2
	meiPlatformLockTake(void	*lock,
						MPIWait	timeout);
MPI_DECL1 double MPI_DECL2
	meiPlatformLogb(double data);

MPI_DECL1 long MPI_DECL2
	meiPlatformMessageLockCreate(const char *name,
								 void **lock);

MPI_DECL1 long MPI_DECL2
    meiPlatformThreadId(void);

MPI_DECL1 long MPI_DECL2
	meiPlatformProcessId(void);

MPI_DECL1 long MPI_DECL2
	meiPlatformSignalCreate(const char	*name,
							void		**signal);

MPI_DECL1 long MPI_DECL2
	meiPlatformSignalDelete(void *signal);

MPI_DECL1 long MPI_DECL2
	meiPlatformSignalWait(void		*signal,
						  MPIWait	timeout);

MPI_DECL1 long MPI_DECL2
	meiPlatformSignalWake(void *signal);

MPI_DECL1 void MPI_DECL2
	meiPlatformSleep(long milliseconds);

MPI_DECL1 long MPI_DECL2
	meiPlatformSocketInfoGet(MEIPlatform platform,
							MEIPlatformSocketInfo *socketInfo);

MPI_DECL1 long MPI_DECL2
	meiPlatformSocketInfoSet(MEIPlatform platform,
							MEIPlatformSocketInfo *socketInfo);

MPI_DECL1 long MPI_DECL2
	meiPlatformSynqNetCopyMap(MEIPlatform	platform,
								long		txSrcAddr,
								long		txDstAddr,
								long		txSize,
								long		rxSrcAddr,
								long		rxDstAddr,
								long		rxSize,
								MEIXmpSynqNetCopy *synqNetCopy);

MPI_DECL1 long MPI_DECL2
	meiPlatformTimerCount(long *ticks);

MPI_DECL1 long MPI_DECL2
	meiPlatformTimerFrequency(long *frequency);

MPI_DECL1 long MPI_DECL2
	meiPlatformPerformanceTimerCount(long *ticks);

MPI_DECL1 long MPI_DECL2
	meiPlatformPerformanceTimerFrequency(long *frequency);

MPI_DECL1 long MPI_DECL2
	meiPlatformTrace(const char *format,
					 ...);

MPI_DECL1 char MPI_DECL2
	meiPlatformTraceEol(char eol);

MPI_DECL1 long MPI_DECL2
	meiPlatformTraceFile(const char *fileName);

MPI_DECL1 MEITraceFunction MPI_DECL2
	meiPlatformTraceFunction(MEITraceFunction traceFunction);

MPI_DECL1 long MPI_DECL2
	meiPlatformInfo(MEIPlatform		platform,
					MEIPlatformInfo	*platformInfo);


MPI_DECL1 long MPI_DECL2
	meiPlatformVsnprintf(char* buffer, long bufferSize, long count, char** end, const char* format, va_list args);

MPI_DECL1 long MPI_DECL2
	meiPlatformSnprintf(char* buffer, long bufferSize, long count, char** end, const char* format, ...);

MPI_DECL1 long MPI_DECL2
	meiPlatformVsprintf(char* buffer, long bufferSize, char** end, const char* format, va_list args);

MPI_DECL1 long MPI_DECL2
	meiPlatformSprintf(char* buffer, long bufferSize, char** end, const char* format, ...);


MPI_DECL1 long MPI_DECL2
	meiPlatformStrncpy(char* buffer, long bufferSize, char** end, const char* source, long count);

MPI_DECL1 long MPI_DECL2
	meiPlatformStrcpy(char* buffer, long bufferSize, char** end, const char* source);


MPI_DECL1 long MPI_DECL2
	meiPlatformStrncat(char* buffer, long bufferSize, char** end, const char* source, long count);

MPI_DECL1 long MPI_DECL2
	meiPlatformStrcat(char* buffer, long bufferSize, char** end, const char* source);

MPI_DECL1 long MPI_DECL2
	meiPlatformStricmp(const char* str1, const char* str2);

#if defined(__cplusplus)
}
#endif

#endif
