/* recorder.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_RECORDER_H)
#define	_RECORDER_H

/* Recorder handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPIRecorder *MPIRecorder;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "axis.h"
#include "event.h"
#include "filter.h"

#if defined(__cplusplus)
extern "C" {
#endif

#define	MPIRecorderADDRESS_COUNT_MAX	(32)
#define	MPIRecorderRECORDERS_MAX		(32)

typedef struct MPIRecorderConfig {
	long	 period;		  /* collect 1 record every `period` milliseconds */
	long	 highCount;	  /* >0 => record count to trigger high buffer    */
	MPI_BOOL bufferWrap;   /* TRUE/FALSE */

	long	addressCount; /* number of data point addresses in address[]  */
	void	*address[MPIRecorderADDRESS_COUNT_MAX];
} MPIRecorderConfig;

typedef long MPIRecorderRecordPoint;

typedef union {
	MPIRecorderRecordPoint	point[MPIRecorderADDRESS_COUNT_MAX];
} MPIRecorderRecord;

typedef enum {
	MPIRecorderRecordTypeINVALID = -1,

	MPIRecorderRecordTypePOINT,

	MPIRecorderRecordTypeLAST,
	MPIRecorderRecordTypeFIRST = MPIRecorderRecordTypeINVALID + 1
} MPIRecorderRecordType;

typedef struct MPIRecorderStatus {
	MPI_BOOL enabled;
	MPI_BOOL full;
	long     recordCount;
	long     recordCountMax;
	MPI_BOOL reserved;
} MPIRecorderStatus;

typedef enum {
	MPIRecorderMessageFIRST = mpiMessageID(MPIModuleIdRECORDER, 0),

	MPIRecorderMessageRECORDER_INVALID,
	MPIRecorderMessageSTARTED,
	MPIRecorderMessageSTOPPED,
	MPIRecorderMessageNOT_CONFIGURED,
	MPIRecorderMessageNO_RECORDERS_AVAIL,
	MPIRecorderMessageNOT_ENABLED,
	MPIRecorderMessageRUNNING,
	MPIRecorderMessageRECORD_COUNT_INVALID,
	MPIRecorderMessageRESERVED,
	MPIRecorderMessageNOT_RESERVED,

	MPIRecorderMessageLAST
} MPIRecorderMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPIRecorder MPI_DECL2
	mpiRecorderCreate(MPIControl	control,
					  long			number);

MPI_DECL1 long MPI_DECL2
	mpiRecorderDelete(MPIRecorder recorder);

MPI_DECL1 long MPI_DECL2
	mpiRecorderValidate(MPIRecorder recorder);

/* Recorder */
MPI_DECL1 long MPI_DECL2
	mpiRecorderConfigGet(MPIRecorder		recorder,
						 MPIRecorderConfig	*config,
						 void				*external);

MPI_DECL1 long MPI_DECL2
	mpiRecorderConfigSet(MPIRecorder		recorder,
						 MPIRecorderConfig	*config,
						 void				*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiRecorderControl(MPIRecorder recorder);

MPI_DECL1 long MPI_DECL2
	mpiRecorderEventNotifyGet(MPIRecorder	recorder,
							  MPIEventMask	*eventMask,
							  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiRecorderEventNotifySet(MPIRecorder	recorder,
							  MPIEventMask	eventMask,
							  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiRecorderEventReset(MPIRecorder	recorder,
						  MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	mpiRecorderMemory(MPIRecorder	recorder,
					  void			**memory);

MPI_DECL1 long MPI_DECL2
	mpiRecorderMemoryGet(MPIRecorder	recorder,
						 void			*dst,
						 const void		*src,
						 long			count);

MPI_DECL1 long MPI_DECL2
	mpiRecorderMemorySet(MPIRecorder	recorder,
						 void			*dst,
						 const void		*src,
						 long			count);

MPI_DECL1 long MPI_DECL2
	mpiRecorderRecordConfig(MPIRecorder				recorder,
							MPIRecorderRecordType	type,
							long					count,
							void					*handle);

MPI_DECL1 long MPI_DECL2
	mpiRecorderRecordGet(MPIRecorder		recorder,
						 long				countMax,
						 MPIRecorderRecord	*record,
						 long				*count);

MPI_DECL1 long MPI_DECL2
	mpiRecorderStart(MPIRecorder	recorder,
					 long			count);		/* -1 => continuous, >0 => # of records */
					 

MPI_DECL1 long MPI_DECL2
	mpiRecorderNumber(MPIRecorder	recorder,
					  long		  	*number);

MPI_DECL1 long MPI_DECL2
	mpiRecorderStatus(MPIRecorder		recorder,
					  MPIRecorderStatus	*status,
					  void				*external);

MPI_DECL1 long MPI_DECL2
	mpiRecorderStop(MPIRecorder recorder);

#if defined(__cplusplus)
}
#endif

#endif
