/* Rincon.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

#if !defined (__RINCON__)
#define __RINCON__

#if defined(__cplusplus)
extern "C" {
#endif

#define MEIXmpRinconDMATxMessagesAsynq	(1+1)	/* 1 service command + end */
#define MEIXmpRinconDMARxMessagesAsynq	(1+1+1)	/* 1 service status + end (plus one more for reset with dual-string etc...) */

#define MEIXmpRinconDMATxMessages	(1+2*MEIXmpMaxMotionBlocks+1+1)	/* Synq message + 2 messages per block + test packet + end */
#define MEIXmpRinconDMARxMessages	(2*MEIXmpMaxMotionBlocks+1)		/* 2 messages per block + end */

#define RINCON_UPSTREAM_FAULT_THREASHOLD_BASE	((volatile long *)(0x00550100))	/* 100 - 11F = 32 entries */
#define RINCON_UPSTREAM_FAIL_THREASHOLD_BASE	((volatile long *)(0x00550200))	/* 200 - 21F = 32 entries */
#define RINCON_UPSTREAM_ERROR_COUNTER_BASE		((volatile long *)(0x00550300))	/* 300 - 31F = 32 entries */
#define RINCON_UPSTREAM_ERROR_RATE_COUNTER_BASE	((volatile long *)(0x00550400))	/* 400 - 41F = 32 entries */

#define RINCON_DMA_TX_TIMER_DEFAULT					(500)
#define RINCON_DMA_RX_TIMER_DEFAULT					(200)
#define RINCON_TX_MII_TIMER_DEFAULT					(200)
#define RINCON_RX_COPY_TIMER_DEFAULT				(0xFFFF)
#define RINCON_TX_PACKET_GAP_DEFAULT				(8)
#define RINCON_UPSTREAM_FAULT_THREASHOLD_DEFAULT	(4)
#define RINCON_UPSTREAM_FAIL_THREASHOLD_DEFAULT		(16)


typedef enum
{	MEIXmpRinconDMATimerControlDisable = (int)0x00000000,				/* Bit 0 - Reset state */
	MEIXmpRinconDMATimerControlEnable = (int)0x00000001,				/* Bit 0 - Enables timers */
	MEIXmpRinconDMATimerControlTriggerExt = (int)0x00000002,			/* Bit 1 - Reset state is cleared - used for XMP - trigger source is Texp from sharc */
	MEIXmpRinconDMATimerControlTriggerSingle = (int)0x00000002,			/* Bit 1 - Reset state is cleared - used for ZMP - trigger source is RinconZ timer */
	MEIXmpRinconDMATimerControlTriggerContinuous = (int)0x00000004,		/* Bit 2 - Reset state is cleared */
	MEIXmpRinconDMATimerControlTriggerForce = (int)0x00000008,			/* Bit 3 - Self clearing one shot - Reset state is cleared */
	MEIXmpRinconDMATimerControlRxCopyTimerEnable = (int)0x00000020,		/* Bit 5 - Rx Copy Timer should be enabled for cyclic operation and disabled during asynchronous operation */
	MEIXmpRinconDMATimerControlContinuousEnable = (int)0x00000200		/* Bit 9 - Enable Continuous Timer */
} MEIXmpRinconDMATimerControl;

typedef enum 
{	MEIXmpRinconDMATxControlDisable = (int)0x00000000,					/* Bit 0 - Reset state */
	MEIXmpRinconDMATxControlEnable = (int)0x00000001,					/* Bit 0 - Enables DMA transmit */
	MEIXmpRinconDMATxControlCRCErrorInjection = (int)0x00000002,		/* Bit 1 - Forces CRC fields to 0 while set */
	MEIXmpRinconDMATxControlCRCErrorInjectionOneShot = (int)0x00000004,	/* Bit 2 - Forces CRC field in packet after TxTimer triggers to 0 - self clearing */
	MEIXmpRinconDMATxControlReservedMask = (int)0x00007FF8,				/* Bits 3 - 14 Reserved */
	MEIXmpRinconDMATxControlCopyEnable = (int)0x00008000,				/* Bit 15 - Copy Enable - used for ZMP */
	MEIXmpRinconDMATxControlStatusBusy = (int)0x00010000,				/* Bit 16 - Busy Tx DMA active */
	MEIXmpRinconDMATxControlStatusFIFOUnderflow = (int)0x00020000,		/* Bit 17 - FIFO transmitted more than was loaded */
	MEIXmpRinconDMATxControlStatusFIFONotEmpty = (int)0x00040000,		/* Bit 18 - Not Empty, MII still transmitting */
	MEIXmpRinconDMATxControlTxEnableOUTPORT = (int)0x00080000,			/* Bit 19 - Tx enabled on MII Outport */
	MEIXmpRinconDMATxControlTxEnableINPORT = (int)0x00100000,			/* Bit 20 - Tx enabled on MII Inport */
	MEIXmpRinconDMATxControlStatusMask = (int)0x00070000				/* Bits 16 - 18 */
} MEIXmpRinconDMATxControl;

typedef enum
{	MEIXmpRinconDMARxControlDisable = (int)0x00000000,					/* Bit 2 - Reset state */
	MEIXmpRinconDMARxControlEnable = (int)0x00000004, 					/* Bit 2 - Enables DMA receive */
	MEIXmpRinconDMARxControlDMARequestEnable = (int)0x00000008,			/* Bit 3 - Enables Recieve handshaking for external DMA */
	MEIXmpRinconDMARxControlOutPortLink = (int)0x00000040,				/* Bit 6 - OutPortLinkStatus */
	MEIXmpRinconDMARxControlInPortLink = (int)0x00000080,				/* Bit 7 - InPortLinkStatus */
	MEIXmpRinconDMARxControlPropTimerBusy = (int)0x00000100,			/* Bit 8 - Propagation timer is busy */
	MEIXmpRinconDMARxControlCopyEnable = (int)0x00008000,				/* Bit 15 - Copy Enable - used for ZMP */
	MEIXmpRinconDMARxControlDMABusy = (int)0x00010000,					/* Bit 16 - DMA engine running */
	MEIXmpRinconDMARxControlInPortDisable = (int)0x00040000,			/* Bit 18 - InPort rx packets ignored */
	MEIXmpRinconDMARxControlOutPortDisable = (int)0x00080000,			/* Bit 19 - OutPort Rx packets ignored */
	MEIXmpRinconDMARxControlChangePacketEnable = (int)0x00100000,		/* Bit 20 - Change packets accepeted */
	MEIXmpRinconDMARxControlTestPacketEnable = (int)0x00200000,			/* Bit 21 - Test packets accepted */
	MEIXmpRinconDMARxControlResetCompletePacketEnable = (int)0x00400000,/* Bit 22 - ResetComplete packets accepted - defaulted to 1, clear for restart */
	MEIXmpRinconDMARxControlRestartRequestPacketEnable = (int)0x00800000,/* Bit 23 - RestartRequest packets accepted */
	MEIXmpRinconDMARxControlStatusError_Length = (int)0x10000000,		/* Bit 28 - Rx Cb Length Error */
	MEIXmpRinconDMARxControlStatusError_Dribble =0x20000000,			/* Bit 29 - Rx Cb Dribble on myself Error */
	MEIXmpRinconDMARxControlStatusError_CRC = (int)0x40000000,			/* Bit 30 - Rx Cb CRC Error */
	MEIXmpRinconDMARxControlStatusError = (int)0x80000000,				/* Bit 31 - Rx DMA Error */
	MEIXmpRinconDMARxControlReservedMask = (int)0x0FFE7E33				/* Bits 0, 1, 4, 5, 9 - 14, 17 - 27 Reserved */
} MEIXmpRinconDMARxControl;

typedef enum
{	MEIXmpRinconCRCCounterRESET = (int)0x00000001,						/* Bits ANY - a write to any bit resets the counter to 0 */
	MEIXmpRinconCRCCounterControlCountMask = (int)0x000000FF,			/* Bits 0 - 7 */
	MEIXmpRinconCRCCounterControlRolloverModeSATURATE = (int)0x00000000,/* Bit 26 - saturates at 0xFF */
	MEIXmpRinconCRCCounterControlRolloverModeROLLOVER = (int)0x04000000,/* Bit 26 - rolls over to 0 at 0xFF */
	MEIXmpRinconCRCCounterControlReservedMask = (int)0xFBFFFF00			/* Bits 8 - 25, 27 - 31 */
} MEIXmpRinconCRCCounterControl;

typedef enum
{	MEIXmpRinconDMARxStatusUpstreamFault = (int)0x00000001,				/* Bit 0 - RxError Rate exceeds RxError Fault Threashold */
	MEIXmpRinconDMARxStatusUpstreamFail = (int)0x00000002,				/* Bit 1 - RxError Rate exceeds RxError Fail Threashold */
	MEIXmpRinconDMARxStatusMIIRxInPort = (int)0x00000000,				/* Bit 4 - Last packet received on "In" port */
	MEIXmpRinconDMARxStatusMIIRxOutPort = (int)0x00000010,				/* Bit 4 - Last packet received on "out" port */
	MEIXmpRinconDMARxStatusNonZeroErrorRate = (int)0x0000200,			/* Bit 9 - Error rate is non-zero */
	MEIXmpRinconDMARxStatusUpstreamPacketSequenceError = (int)0x00000400/* Bit 10 - Packet out of sequence, using last good packet */
} MEIXmpRinconDMARxStatus;

typedef enum
{	MEIXmpRinconTestPacketStatusRxOutPort = (int)0x0000001,				/* Bit 0 - Test Packet received on Out Port */ 
	MEIXmpRinconTestPacketStatusRxInPort = (int)0x00000002,				/* Bit 1 - Test Packet received on In Port */
	MEIXmpRinconTestPacketStatusTxOutPort = (int)0x0000100,				/* Bit 8 - transmit Test Packet on Out Port */ 
	MEIXmpRinconTestPacketStatusTxInPort = (int)0x00000200,				/* Bit 9 - transmit Test Packet on In Port */
	MEIXmpRinconTestPacketStatusEchoTimerMask = (int)0x00FF0000,		/* Bits 16-23 - Echo Timer value */
	MEIXmpRinconTestPacketStatusEchoTimerMaskSHIFT = (int)16,
} MEIXmpRinconTestPacketStatus;

typedef enum
{	MEIXmpRinconRestartRequestStatusOutPort = (int)0x00000001,			/* Bit 0 - ResetRequest Packet on Out Port - must be cleared before sending */
	MEIXmpRinconRestartRequestStatusInPort = (int)0x00000002,			/* Bit 1 - ResetRequest Packet on In Port - must be cleared before sending */
} MEIXmpRinconRestartRequestStatus;

typedef enum
{	MEIXmpRinconResetCompleteStatusOutPort = (int)0x00000001,			/* Bit 0 - ResetComplete Packet on Out Port - must be cleared before sending */
	MEIXmpRinconResetCompleteStatusInPort = (int)0x00000002,			/* Bit 1 - ResetComplete Packet on In Port - must be cleared before sending */
} MEIXmpRinconResetCompleteStatus;

typedef enum 
{	MEIXmpRinconPropagationDelayMask = 0x0000FFFF,						/* Bits 0 - 15 - delay value in 40nS ticks (25MHz) */
	MEIXmpRinconPropagationDelayRxTypeMask = 0x00FF0000,				/* Bits 16 - 23 - packet type field from received packet */
	MEIXmpRinconPropagationDelayRxTypeShift = 16,
	MEIXmpRinconPropagationDelayRxAddrMask = 0xFF000000,				/* Bits 24 - 31 - addr field from received */
	MEIXmpRinconPropagationDelayRxAddrShift = 24,
} MEIXmpRinconPropagationDelay;

typedef enum
{	MEIXmpRinconPropagationControlPacketCountMask = (int)0x000000FF,	/* Bits 0 - 7 */
	MEIXmpRinconPropagationControlCycleDelayMask = (int)0x00000300,		/* Bits 8 - 9 */
	MEIXmpRinconPropagationControlRxFifoFlushError = (int)0x04000000,	/* Bit 26 */
	MEIXmpRinconPropagationControlUnderflowError = (int)0x08000000,		/* Bit 27 */
	MEIXmpRinconPropagationControlDribbleError = (int)0x10000000,		/* Bit 28 */
	MEIXmpRinconPropagationControlCRCError = (int)0x20000000,			/* Bit 29 */
	MEIXmpRinconPropagationControlError = (int)0x40000000,				/* Bit 30 */
	MEIXmpRinconPropagationControlBusy = (int)0x80000000,				/* Bit 31 */
} MEIXmpRinconPropagationControl;

typedef enum
{	MEIXmpRinconNetworkStateUNDISCOVERED = 0,
	MEIXmpRinconNetworkStateASYNQ = 1,
	MEIXmpRinconNetworkStateSYNQ = 2
} MEIXmpRinconNetworkState;

/* For Rincon TCBs, the address is the start address for the Tx or Rx data.  The length indicates the number of
	32 bit words to Tx or Rx.  There must be one TCB per message so that Rincon can insert or cut out the
	checksum for each message.  TCBs are chained automatically by going to the next TCB until an address and 
	length of zero is found.  There are two separate TCB lists for Tx and Rx.
*/
typedef struct MEIXmpRinconTCBTx
{	long *Address;
	long Length;
} MEIXmpRinconTCBTx;

typedef struct MEIXmpRinconTCBRx
{	long *Address;
	long Length;
	long Header;
	long Status;	/* Written by Rincon */
} MEIXmpRinconTCBRx;

typedef struct MEIXmpRinconDMAAsynq
{	MEIXmpRinconTCBTx Tx[MEIXmpRinconDMATxMessagesAsynq];
	MEIXmpRinconTCBRx Rx[MEIXmpRinconDMARxMessagesAsynq];
} MEIXmpRinconDMAAsynq;

typedef struct MEIXmpRinconAsynq
{	MEIXmpSynqNetMessageAsynq	Message;
	MEIXmpRinconDMAAsynq		DMA;
} MEIXmpRinconAsynq;


typedef struct MEIXmpRinconCopyControl	/* RinconZ only */
{	long SourceAddress;
	long DestAddress;
	long Length;
} MEIXmpRinconCopyControl;

typedef struct MEIXmpRinconCopyControlBlock	/* RinconZ only - on top of MEIXmpSynqNetCopy struct in flash */
{	MEIXmpRinconCopyControl Tx;
	MEIXmpRinconCopyControl Rx;
} MEIXmpRinconCopyControlBlock;

#if defined(__cplusplus)
}
#endif

#endif /* __RINCON__ */
