/* sequence.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_SEQUENCE_H)
#define	_SEQUENCE_H

/* Sequence handle */
#if defined(__cplusplus)
extern "C" {
#endif
typedef struct _MPISequence *MPISequence;
#if defined(__cplusplus)
}
#endif

#include "mpidef.h"
#include "control.h"
#include "command.h"
#include "event.h"

#if defined(__cplusplus)
extern "C" {
#endif

typedef MPIEmpty	MPISequenceConfig;

typedef enum {
	MPISequenceStateSTOPPED = 0,
	MPISequenceStateSTARTED
} MPISequenceState;

typedef struct MPISequenceStatus {
	MPICommand	command;

	MPISequenceState	state;
} MPISequenceStatus;

typedef enum {
	MPISequenceMessageFIRST = mpiMessageID(MPIModuleIdSEQUENCE, 0),

	MPISequenceMessageSEQUENCE_INVALID,
	MPISequenceMessageCOMMAND_COUNT,
	MPISequenceMessageCOMMAND_NOT_FOUND,
	MPISequenceMessageSTARTED,
	MPISequenceMessageSTOPPED,

	MPISequenceMessageLAST
} MPISequenceMessage;

/* Create/Delete/Validate */
MPI_DECL1 MPISequence MPI_DECL2
	mpiSequenceCreate(MPIControl	control,
					  long			number,
					  long			pageSize);

MPI_DECL1 long MPI_DECL2
	mpiSequenceDelete(MPISequence sequence);

MPI_DECL1 long MPI_DECL2
	mpiSequenceValidate(MPISequence sequence);

/* Sequence */
MPI_DECL1 long MPI_DECL2
	mpiSequenceConfigGet(MPISequence		sequence,
						 MPISequenceConfig	*config,
						 void				*external);

MPI_DECL1 long MPI_DECL2
	mpiSequenceConfigSet(MPISequence		sequence,
						 MPISequenceConfig	*config,
						 void				*external);

MPI_DECL1 MPIControl MPI_DECL2
	mpiSequenceControl(MPISequence sequence);

MPI_DECL1 long MPI_DECL2
	mpiSequenceEventNotifyGet(MPISequence	sequence,
							  MPIEventMask	*eventMask,
							  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiSequenceEventNotifySet(MPISequence	sequence,
							  MPIEventMask	eventMask,
							  void			*external);

MPI_DECL1 long MPI_DECL2
	mpiSequenceEventReset(MPISequence	sequence,
						  MPIEventMask	eventMask);

MPI_DECL1 long MPI_DECL2
	mpiSequenceFlashConfigGet(MPISequence		sequence,
							  void				*flash,
							  MPISequenceConfig	*config,
							  void				*external);

MPI_DECL1 long MPI_DECL2
	mpiSequenceFlashConfigSet(MPISequence		sequence,
							  void				*flash,
							  MPISequenceConfig	*config,
							  void				*external);

MPI_DECL1 long MPI_DECL2
	mpiSequenceLoad(MPISequence	sequence,
					MPICommand	command,
					MPI_BOOL	start);

MPI_DECL1 long MPI_DECL2
	mpiSequenceMemory(MPISequence	sequence,
					  void			**memory);

MPI_DECL1 long MPI_DECL2
	mpiSequenceMemoryGet(MPISequence	sequence,
						 void			*dst,
						 const void		*src,
						 long			count);

MPI_DECL1 long MPI_DECL2
	mpiSequenceMemorySet(MPISequence	sequence,
						 void			*dst,
						 const void		*src,
						 long			count);

MPI_DECL1 long MPI_DECL2
	mpiSequenceNumber(MPISequence	sequence,
					  long			*number);

MPI_DECL1 long MPI_DECL2
	mpiSequencePageSize(MPISequence	sequence,
						long		*pageSize);

MPI_DECL1 long MPI_DECL2
	mpiSequenceResume(MPISequence sequence);

MPI_DECL1 long MPI_DECL2
	mpiSequenceStart(MPISequence	sequence,
					 MPICommand		command);

MPI_DECL1 long MPI_DECL2
	mpiSequenceStatus(MPISequence		sequence,
					  MPISequenceStatus	*status,
					  void				*external);

MPI_DECL1 long MPI_DECL2
	mpiSequenceStep(MPISequence	sequence,
					long		count);

MPI_DECL1 long MPI_DECL2
	mpiSequenceStop(MPISequence sequence);

/* List */
MPI_DECL1 MPICommand MPI_DECL2
	mpiSequenceCommand(MPISequence	sequence,
					  long			index);

MPI_DECL1 long MPI_DECL2
	mpiSequenceCommandAppend(MPISequence	sequence,
							 MPICommand		command);

MPI_DECL1 long MPI_DECL2
	mpiSequenceCommandCount(MPISequence sequence);

MPI_DECL1 MPICommand MPI_DECL2
	mpiSequenceCommandFirst(MPISequence sequence);

MPI_DECL1 long MPI_DECL2
	mpiSequenceCommandIndex(MPISequence	sequence,
							MPICommand	command);

MPI_DECL1 long MPI_DECL2
	mpiSequenceCommandInsert(MPISequence	sequence,
							 MPICommand		command,
							 MPICommand		insert);

MPI_DECL1 MPICommand MPI_DECL2
	mpiSequenceCommandLast(MPISequence sequence);

MPI_DECL1 long MPI_DECL2
	mpiSequenceCommandListGet(MPISequence	sequence,
							  long			*commandCount,
							  MPICommand	*commandList);

MPI_DECL1 long MPI_DECL2
	mpiSequenceCommandListSet(MPISequence	sequence,
							  long			commandCount,
							  MPICommand	*commandList);

MPI_DECL1 MPICommand MPI_DECL2
	mpiSequenceCommandNext(MPISequence	sequence,
						   MPICommand	command);

MPI_DECL1 MPICommand MPI_DECL2
	mpiSequenceCommandPrevious(MPISequence	sequence,
							   MPICommand	command);

MPI_DECL1 long MPI_DECL2
	mpiSequenceCommandRemove(MPISequence	sequence,
							 MPICommand		command);

#if defined(__cplusplus)
}
#endif

#endif
