   /* stdmei.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_STDMEI_H)
#define	_STDMEI_H

/*
 *	This file declares the entire internal MEI implementation
 *	of the standard Motion Programming Interface.
 */

#define MEIObjectLabelCharMAX		(16)

#include "stdmpi.h"
#include "platform.h"
#include "meidef.h"

#include "can.h"
#include "sqNode.h"
#include "client.h"
#include "element.h"
#include "firmware.h"
#include "flash.h"
#include "list.h"
#include "meimap.h"
#include "packet.h"
#include "synqnet.h"
#include "drivemap.h"
#include "remote.h"
#include "server.h"

#if defined(__cplusplus)
extern "C" {
#endif

/* FPGA Revision defines */
#define MEIFPGARINCONREV			(0x0251)

#define MEIFpgaSqMACVersionDEFAULT	(0x0300)
#define MEIFpgaSqMACVersionMIN		(0x0300)
#define MEIFpgaSqMACVersionMAX		(0x03FF)

#define MEIFpgaSqNodeVersionDEFAULT	(0x0400)
#define MEIFpgaSqNodeVersionMIN		(0x0400)
#define MEIFpgaSqNodeVersionMAX		(0x04FF)

/* Axis */
typedef struct MEIAxisConfig {
	char					userLabel[MEIObjectLabelCharMAX+1]; /* +1 for NULL terminator */
  
	long					*FeedbackDeltaPtr[MEIXmpAxisPosInputs];
	MEIXmpAxisPreFilter		PreFilter;
	MEIXmpAxisGear			Gear;
	MEIXmpAxisGantryType	GantryType;
} MEIAxisConfig;

typedef enum {
	MEIAxisTraceFIRST = MEITraceLAST << 1,

	MEIAxisTraceLAST  = MEIAxisTraceFIRST << 1
} MEIAxisTrace;

typedef struct MEIAxisFrameBufferStatus {
	long	size;
	long	frameCount;
} MEIAxisFrameBufferStatus;

MPI_DECL1 long MPI_DECL2
	meiAxisFrameBufferStatus(MPIAxis axis,
							 MEIAxisFrameBufferStatus *status);
  

/* Capture */
typedef enum {
	MEICaptureMessageINVALID_EDGE = MPICaptureMessageLAST,
	MEICaptureMessageGLOBAL_CONFIG_ERR,
	MEICaptureMessageGLOBAL_ALREADY_ENABLED,
	MEICaptureMessageCAPTURE_NOT_ENABLED,
	MEICaptureMessageCAPTURE_STATE_INVALID,
	MEICaptureMessageNOT_MAPPED ,
	MEICaptureMessageUNSUPPORTED_PRIMARY,
	MEICaptureMessageUNSUPPORTED_SECONDARY,
	MEICaptureMessageSECONDARY_INDEX_INVALID,
	MEICaptureMessageCAPTURE_ARMED,
	MEICaptureMessageLAST
} MEICaptureMessage;

typedef enum {
	MEICaptureTraceFIRST = MEITraceLAST << 1,

	MEICaptureTraceLAST  = MEICaptureTraceFIRST << 1
} MEICaptureTrace;


/* Command */
typedef enum {
	MEICommandTraceFIRST = MEITraceLAST << 1,

	MEICommandTraceLAST  = MEICommandTraceFIRST << 1
} MEICommandTrace;

MPI_DECL1 long MPI_DECL2
	meiCommandAxisListGet(MPICommand	command,
						  long			*axisCount,
						  MPIAxis		*axisList);

MPI_DECL1 long MPI_DECL2
	meiCommandXmpIndexGet(MPICommand	command,
						  long			*index);

MPI_DECL1 long MPI_DECL2
	meiCommandXmpIndexSet(MPICommand	command,
						  long			index);



/* Flash */
#define MEIFlashFileMaxNameChars	(12)	/* 8.3 format */
#define MEIFlashFileMaxChars		(120)
#define MEIFlashFileMaxPathChars	(MEIFlashFileMaxChars - MEIFlashFileMaxNameChars)

typedef struct MEIFlashFiles {
	char binFile[MEIFlashFileMaxChars];
	char FPGAFile[MEIFlashFileMaxChars];
} MEIFlashFiles;


#define MEIPreFilterCoeffsMAX     (MEIXmpMAX_PreCoeffs) 
#define MEIPreFilterCountMAX      (MEIXmpMAX_PreFilters)

typedef enum {
		MEIPreFilterFormINVALID = -1,

		MEIPreFilterFormNONE,
		MEIPreFilterFormBOXCAR,
		MEIPreFilterFormSHAPING,

		MEIPreFilterFormLAST,
		MEIPreFilterFormFIRST = MEIPreFilterFormINVALID + 1

} MEIPreFilterForm;

typedef struct MEIPreFilter {
	long						axisNumber;
	MEIPreFilterForm			form;
	long						length;
	long						coeff[MEIPreFilterCoeffsMAX];
} MEIPreFilter;

MPI_DECL1 long MPI_DECL2
	meiFlashMemoryFromFile(MEIFlash flash,
						   MEIFlashFiles *filesIn,
						   MEIFlashFiles *filesOut);

/* Control */
typedef struct MEIControlFPGA {
	char					FileName[MEIFlashFileMaxChars];
} MEIControlFPGA;

typedef struct MEIControlConfig {
	char			    userLabel[MEIObjectLabelCharMAX+1]; /* +1 for NULL terminator */
	long				preFilterCount;
	long				TxTime;
	long				syncInterruptPeriod;
	MEIPreFilter		preFilter[MEIXmpMAX_PreFilters];
	MEIXmpUserBuffer	UserBuffer;
} MEIControlConfig;

typedef enum {
	MEIControlFileTypeFLASH_IMAGE = MPIControlFileTypeLAST,
	MEIControlFileTypeFULL_IMAGE,
	MEIControlFileTypePARTIAL_IMAGE,
	MEIControlFileTypeLAST,
	MEIControlFileTypeFIRST = MPIControlFileTypeLAST
} MEIControlFileType;

typedef enum {
	MEIControlInputUSER_0	= MEIXmpControlIOMaskUSER0_IN,
	MEIControlInputUSER_1	= MEIXmpControlIOMaskUSER1_IN,
	MEIControlInputUSER_2	= MEIXmpControlIOMaskUSER2_IN,
	MEIControlInputUSER_3	= MEIXmpControlIOMaskUSER3_IN,
	MEIControlInputUSER_4	= MEIXmpControlIOMaskUSER4_IN,
	MEIControlInputUSER_5	= MEIXmpControlIOMaskUSER5_IN,
	MEIControlInputXESTOP	= MEIXmpControlIOMaskXESTOP
} MEIControlInput;

typedef enum {
	MEIControlOutputUSER_0	= MEIXmpControlIOMaskUSER0_OUT,
	MEIControlOutputUSER_1	= MEIXmpControlIOMaskUSER1_OUT,
	MEIControlOutputUSER_2	= MEIXmpControlIOMaskUSER2_OUT,
	MEIControlOutputUSER_3	= MEIXmpControlIOMaskUSER3_OUT,
	MEIControlOutputUSER_4	= MEIXmpControlIOMaskUSER4_OUT,
	MEIControlOutputUSER_5	= MEIXmpControlIOMaskUSER5_OUT
} MEIControlOutput;

typedef enum {
	MEIControlIoBitINVALID = -1,

	MEIControlIoBitUSER_0_IN,
	MEIControlIoBitUSER_1_IN,
	MEIControlIoBitUSER_2_IN,
	MEIControlIoBitUSER_3_IN,
	MEIControlIoBitUSER_4_IN,
	MEIControlIoBitUSER_5_IN,
	MEIControlIoBitXESTOP,
	MEIControlIoBitUSER_0_OUT,
	MEIControlIoBitUSER_1_OUT,
	MEIControlIoBitUSER_2_OUT,
	MEIControlIoBitUSER_3_OUT,
	MEIControlIoBitUSER_4_OUT,
	MEIControlIoBitUSER_5_OUT,
	MEIControlIoBitLAST,
	
	MEIControlIoBitINPUT_FIRST = MEIControlIoBitUSER_0_IN,
	MEIControlIoBitINPUT_LAST = MEIControlIoBitXESTOP + 1,

	MEIControlIoBitOUTPUT_FIRST = MEIControlIoBitUSER_0_OUT,
	MEIControlIoBitOUTPUT_LAST = MEIControlIoBitUSER_5_OUT + 1,

	MEIControlIoBitFIRST = MEIControlIoBitINVALID + 1
} MEIControlIoBit;


typedef enum {
	MEIControlMemoryTypeCOMMAND = MPIControlMemoryTypeLAST,

	MEIControlMemoryTypeLAST,
	MEIControlMemoryTypeFIRST = MPIControlMemoryTypeLAST
} MEIControlMemoryType;

typedef struct MEIControlService {
	unsigned long	mask;
	unsigned long	pattern;
} MEIControlService;


#define MEIControlSTRING_MAX (128)

typedef struct MEIControlInfoMpi {
	char		version[MEIControlSTRING_MAX+1]; /* +1 for null termination character */
	long		fwVersion;
	long		fwOption;
} MEIControlInfoMpi;

typedef struct MEIControlInfoFirmwareZMP {
	long						boot0Version;
	long						boot0Revision;
	long						zbootVersion;
	long						zbootRevision;
} MEIControlInfoFirmwareZMP;

typedef struct MEIControlInfoFirmware {
	long						version;		/* MEIXmpVERSION_EXTRACT(SoftwareID) */
	long						option;			/* MEIXmpOPTION_EXTRACT(Option) */
	char						revision;		/* ('A' - 1) + MEIXmpREVISION_EXTRACT(SoftwareID) */
	long						subRevision;	/* MEIXmpSUB_REV_EXTRACT(Option) */
	long						branchId;
	MEIControlInfoFirmwareZMP	zmp;
} MEIControlInfoFirmware;

typedef struct MEIControlInfoPld {
	 char	version[MEIControlSTRING_MAX];
	 char	option[MEIControlSTRING_MAX];
} MEIControlInfoPld;

typedef struct MEIControlInfoRincon {
	char	version[MEIControlSTRING_MAX];
	char	package[MEIControlSTRING_MAX];
} MEIControlInfoRincon;

typedef struct MEIControlInfoHardware {
	char 		modelNumber[MEIControlSTRING_MAX];
	char 		serialNumber[MEIControlSTRING_MAX];
	char 		type[MEIControlSTRING_MAX];	
} MEIControlInfoHardware;

typedef struct MEIControlInfoDriver {
	char	version[MEIControlSTRING_MAX];
} MEIControlInfoDriver;

typedef struct MEIControlInfoIoDigitalIn {
	MPI_BOOL	supported;
	const char	*name;
} MEIControlInfoIoDigitalIn;

typedef struct MEIControlInfoIoDigitalOut {
	MPI_BOOL	supported;
	const char	*name;
} MEIControlInfoIoDigitalOut;

typedef struct MEIControlInfoIo {
	MEIControlInfoIoDigitalIn	digitalIn[MPIControlInMAX+1];
	MEIControlInfoIoDigitalOut	digitalOut[MPIControlOutMAX+1];
} MEIControlInfoIo;

typedef struct MEIControlInfo {
	MEIControlInfoMpi		mpi;
	MEIControlInfoFirmware	firmware;
	MEIControlInfoPld		pld;
	MEIControlInfoRincon	rincon;	
	MEIControlInfoHardware	hardware;
	MEIControlInfoDriver	driver;
	MEIControlInfoIo		io;
} MEIControlInfo;


typedef enum {
	MEIControlMessageFIRMWARE_INVALID = MPIControlMessageLAST,
	MEIControlMessageFIRMWARE_VERSION_NONE,
	MEIControlMessageFIRMWARE_VERSION,
	MEIControlMessageFPGA_SOCKETS,
	MEIControlMessageBAD_FPGA_SOCKET_DATA,
	MEIControlMessageNO_FPGA_SOCKET,
	MEIControlMessageINVALID_BLOCK_COUNT,
	MEIControlMessageSYNQNET_OBJECTS,
	MEIControlMessageSYNQNET_STATE,
	MEIControlMessageIO_BIT_INVALID,
	
	/* MPI is used for new errors added to the 03.04 branch. */
	MPIControlMessageSTATUS_CACHING_DISABLED,
	MPIControlMessageSAMPLE_RATE_CHANGE_WITH_STEPPER_MOTORS,

	MEIControlMessageLAST

} MEIControlMessage;

typedef struct MEIControlStatistics {
	double	maxForegroundTime;
	double	maxBackgroundTime;
	double	maxDelta;
	double	avgBackgroundRate;
	double	avgBackgroundTime;
	double	backgroundTime;
} MEIControlStatistics;

typedef enum {
	MEIControlTraceFIRST		= MEITraceLAST << 1,

	MEIControlTraceDYNA_ALLOC	= MEIControlTraceFIRST << 0,

	MEIControlTraceLAST			= MEIControlTraceFIRST << 1
} MEIControlTrace;

MPI_DECL1 long MPI_DECL2
	meiControlBlockGive(MPIControl	control,
						long		blockNumber);

MPI_DECL1 long MPI_DECL2
	meiControlBlockTake(MPIControl	control,
						long		blockNumber,
						MPIWait		timeout);

MPI_DECL1 long MPI_DECL2
	meiControlCycleWait(MPIControl	control,
						long		count);

MPI_DECL1 long MPI_DECL2
	meiControlExtMemAvail(MPIControl	control,
						  long			*size);

MPI_DECL1 long MPI_DECL2
	meiControlInfo(MPIControl		control,
				   MEIControlInfo	*info);

MPI_DECL1 long MPI_DECL2
	meiControlMemoryToFile(MPIControl  control,
						   const char *fileName);	

/* Returns the controller's sampleRate */
MPI_DECL1 long MPI_DECL2
	 meiControlSampleRate(MPIControl	 control,
						  double		*sampleRate);

MPI_DECL1 long MPI_DECL2
	meiControlFPGADefaultGet(MPIControl control,
							 MEIPlatformSocketInfo *socketInfo,
							 MEIControlFPGA *fpga);

MPI_DECL1 long MPI_DECL2
	meiControlFPGAFileOverride(MPIControl control,
			  				   MEIControlFPGA *fpga,
			  				   const char *overrideFile,
			  				   MEIPlatformSocketInfo *socketInfo);

MPI_DECL1 long MPI_DECL2
	meiControlGateGet(MPIControl	control,
					  long			gate,
					  MPI_BOOL		*closed);

MPI_DECL1 long MPI_DECL2
	meiControlGateSet(MPIControl	control,
					  long			gate,
					  MPI_BOOL		closed);

MPI_DECL1 long MPI_DECL2
	meiControlInit(MPIControl	control,
				   const char	*mpiVersion);

MPI_DECL1 long MPI_DECL2
	meiControlLinkGet(MPIControl		control,
					  MEIFlash			flash,
					  MPIHandle			handle,
					  MEIXmpLinkBuffer	**linkBuffer);

MPI_DECL1 long MPI_DECL2
	meiControlLinkGive(MPIControl control);

MPI_DECL1 long MPI_DECL2
	meiControlLinkMapSet(MPIControl	control);

MPI_DECL1 long MPI_DECL2
	meiControlLinkSet(MPIControl	control,
					  MEIFlash		flash);

MPI_DECL1 long MPI_DECL2
	meiControlLinkTake(MPIControl	control,
					   MPIWait		timeout);

MPI_DECL1 MEIPlatform MPI_DECL2
	meiControlPlatform(MPIControl	control);

MPI_DECL1 long MPI_DECL2
	meiControlRecorderCancel(MPIControl		   control,
							 long			   recorderNumber);

MPI_DECL1 long MPI_DECL2
	meiControlRecorderStatus(MPIControl		   control,
						     long			   recorderNumber,
							 MPIRecorderStatus *status);

MPI_DECL1 long MPI_DECL2
	meiControlSampleCounter(MPIControl	control,
							long		*sampleCounter);

MPI_DECL1 long MPI_DECL2
	meiControlSampleWait(MPIControl	control,
						 long		count);

MPI_DECL1 long MPI_DECL2
	meiControlSamplesToSeconds(MPIControl	control,
							   long			samples,
							   float		*seconds);

MPI_DECL1 long MPI_DECL2
	meiControlSecondsToSamples(MPIControl	control,
							   float		seconds,
							   long			*samples);

MPI_DECL1 long MPI_DECL2
	meiControlStatisticsReset(MPIControl	control);

MPI_DECL1 long MPI_DECL2
	meiControlStatistics(MPIControl				control,
						 MEIControlStatistics	*statistics);

MPI_DECL1 long MPI_DECL2
	meiControlVersionMismatchOveride(MPIControl control);

MPI_DECL1 long MPI_DECL2
	meiMapFileLoadVerify(MEIMap	map);

/* Event */
typedef enum {
	/* System events */
	MEIEventTypeCONTROL_HOST_PROCESS_TIME_EXCEEDED = MPIEventTypeLAST,	/* 16 */
	MEIEventTypeCONTROL_FAN,											/* 17 */

	/* Motor events */
	MEIEventTypeLIMIT_USER0,											/* 18 */
	MEIEventTypeLIMIT_USER1,											/* 19 */
	MEIEventTypeLIMIT_USER2,											/* 20 */
	MEIEventTypeLIMIT_USER3,											/* 21 */
	MEIEventTypeLIMIT_USER4,											/* 22 */
	MEIEventTypeLIMIT_USER5,											/* 23 */
	MEIEventTypeLIMIT_USER6,											/* 24 */
	MEIEventTypeLIMIT_USER7,											/* 25 */
	MEIEventTypeLIMIT_USER8,											/* 26 */
	MEIEventTypeLIMIT_USER9,											/* 27 */
	MEIEventTypeLIMIT_USER10,											/* 28 */
	MEIEventTypeLIMIT_USER11,											/* 29 */
	MEIEventTypeLIMIT_USER12,											/* 30 */
	MEIEventTypeLIMIT_USER13,											/* 31 */
	MEIEventTypeLIMIT_USER14,											/* 32 */
	MEIEventTypeLIMIT_USER15,											/* 33 */

	/* Motion events */
	MEIEventTypeMOTION_OUT_OF_FRAMES,									/* 34 */

	/* Axis events */
	MEIEventTypeIN_POSITION_COARSE,										/* 35 */
	MEIEventTypeIN_POSITION_FINE,										/* 36 */
	MEIEventTypeSETTLED													/* 36 */ = MEIEventTypeIN_POSITION_FINE,
	MEIEventTypeAT_TARGET,												/* 37 */
	MEIEventTypeFRAME,							   						/* 38 */
																		
	/* SynqNet events */												
	MEIEventTypeSYNQNET_DEAD,											/* 39 */
	MEIEventTypeSYNQNET_RX_FAILURE,										/* 40 */
	MEIEventTypeSYNQNET_TX_FAILURE,										/* 41 */
	MEIEventTypeSYNQNET_NODE_FAILURE,									/* 42 */
	MEIEventTypeSYNQNET_RECOVERY,										/* 43 */				

	/* SqNode events */
	MEIEventTypeSQNODE_IO_ABORT,										/* 44 */
	MEIEventTypeSQNODE_NODE_DISABLE,									/* 45 */
	MEIEventTypeSQNODE_NODE_ALARM,										/* 46 */
	MEIEventTypeSQNODE_ANALOG_POWER_FAULT,								/* 47 */
	MEIEventTypeSQNODE_USER_FAULT,										/* 48 */
	MEIEventTypeSQNODE_NODE_FAILURE,									/* 49 */
	MEIEventTypeSQNODE_IO_FAULT,										/* 50 */
																		
    /* CAN events */													
    MEIEventTypeCAN_BUS_STATE,											/* 51 */
    MEIEventTypeCAN_RECEIVE_OVERRUN,									/* 52 */
    MEIEventTypeCAN_EMERGENGY,											/* 53 */
    MEIEventTypeCAN_NODE_BOOT,											/* 54 */
    MEIEventTypeCAN_HEALTH,												/* 55 */
    MEIEventTypeCAN_DIGITAL_INPUT,										/* 56 */
    MEIEventTypeCAN_ANALOG_INPUT,										/* 57 */

	MEIEventTypeMOTION_START,											/* 58 */ /* Out of sequence */

	MEIEventTypeLAST,
	MEIEventTypeFIRST = MPIEventTypeLAST,

	MEIEventTypeCONTROL_FIRST	= MEIEventTypeCONTROL_HOST_PROCESS_TIME_EXCEEDED,
	MEIEventTypeCONTROL_LAST	= MEIEventTypeCONTROL_FAN,

	MEIEventTypeMOTOR_FIRST		= MEIEventTypeLIMIT_USER0,
	MEIEventTypeMOTOR_LAST		= MEIEventTypeLIMIT_USER15 + 1,

	MEIEventTypeMOTION_FIRST	= MEIEventTypeMOTION_OUT_OF_FRAMES,
	MEIEventTypeMOTION_LAST		= MEIEventTypeMOTION_OUT_OF_FRAMES + 1,

	MEIEventTypeAXIS_FIRST		= MEIEventTypeIN_POSITION_COARSE,
	MEIEventTypeAXIS_LAST		= MEIEventTypeFRAME + 1,

	MEIEventTypeSQNODE_FIRST	= MEIEventTypeSQNODE_IO_ABORT,
	MEIEventTypeSQNODE_LAST		= MEIEventTypeSQNODE_IO_FAULT + 1,

	MEIEventTypeSYNQNET_FIRST   = MEIEventTypeSYNQNET_DEAD,
	MEIEventTypeSYNQNET_LAST    = MEIEventTypeSYNQNET_RECOVERY + 1
} MEIEventType;

#define meiEventMaskALL(mask)		mpiEventMaskBitSET((mask), MEIEventTypeCONTROL_HOST_PROCESS_TIME_EXCEEDED, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCONTROL_FAN, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER0, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER1, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER2, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER3, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER4, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER5, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER6, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER7, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER8, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER9, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER10, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER11, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER12, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER13, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER14, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER15, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeMOTION_OUT_OF_FRAMES, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeIN_POSITION_COARSE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeIN_POSITION_FINE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeAT_TARGET, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeFRAME, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_DEAD, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_RX_FAILURE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_TX_FAILURE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_NODE_FAILURE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_RECOVERY, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_IO_ABORT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_NODE_DISABLE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_NODE_ALARM, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_ANALOG_POWER_FAULT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_USER_FAULT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_NODE_FAILURE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_IO_FAULT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_BUS_STATE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_RECEIVE_OVERRUN, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_EMERGENGY, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_NODE_BOOT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_HEALTH, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_DIGITAL_INPUT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_ANALOG_INPUT, 1)
#define meiEventMaskCONTROL(mask)	mpiEventMaskBitSET((mask), MEIEventTypeCONTROL_HOST_PROCESS_TIME_EXCEEDED, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCONTROL_FAN, 1)
#define meiEventMaskAXIS(mask)		mpiEventMaskBitSET((mask), MEIEventTypeIN_POSITION_COARSE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeIN_POSITION_FINE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeAT_TARGET, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeFRAME, 1)
#define meiEventMaskMOTION(mask)	mpiEventMaskBitSET((mask), MEIEventTypeMOTION_OUT_OF_FRAMES, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeMOTION_START, 1)
#define meiEventMaskMOTOR(mask)		mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER0, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER1, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER2, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER3, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER4, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER5, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER6, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER7, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER8, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER9, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER10, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER11, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER12, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER13, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER14, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeLIMIT_USER15, 1)
#define meiEventMaskSYNQNET(mask)	mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_DEAD, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_RX_FAILURE,1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_TX_FAILURE,1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_NODE_FAILURE,1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSYNQNET_RECOVERY,1)
#define meiEventMaskSQNODE(mask)	mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_IO_ABORT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_NODE_DISABLE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_NODE_ALARM, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_ANALOG_POWER_FAULT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_NODE_FAILURE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_USER_FAULT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeSQNODE_IO_FAULT, 1)
#define meiEventMaskCAN(mask)		mpiEventMaskBitSET((mask), MEIEventTypeCAN_BUS_STATE, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_RECEIVE_OVERRUN, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_EMERGENGY, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_NODE_BOOT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_HEALTH, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_DIGITAL_INPUT, 1), \
										mpiEventMaskBitSET((mask), MEIEventTypeCAN_ANALOG_INPUT, 1)

/*
	The address of an MEIEventNotifyData{} structure is passed
	as the third (void *external) argument to mpi'Object'EventNotify[GS]et().
	The address array contains host-based XMP addresses,
	the contents of which are returned in MEIEventStatusInfo{}.data{}.
*/
typedef struct MEIEventNotifyData {
	void	*address[MEIXmpSignalUserData];
} MEIEventNotifyData;

typedef struct MEIEventStatusInfo {
	union {
		MPIHandle	handle;	/* generic */
		MPIAxis		axis;	/* MEIEventTypeAXIS_FIRST  ... MEIEventTypeAXIS_LAST - 1 */
		long		node;	/* MEIEventTypeCAN_FIRST   ... MEIEventTypeCAN_LAST - 1  */
		long		number;	/* MPIEventTypeMOTION
							   MPIEventTypeMOTOR_FIRST ... MPIEventTypeMOTOR_LAST - 1
							   MEIEventTypeMOTOR_FIRST ... MEIEventTypeMOTOR_LAST - 1 */
		long		value;	/* MPIEventTypeEXTERNAL */
	} type;

	MEIXmpSignalID signalID;

	/* Contents of addresses specified by MEIEventNotifyData{} */
	union {
		long sampleCounter;
		struct {
			long sampleCounter;
		} motion;
		struct {
			long sampleCounter;
			long positionError;
			MEIInt64 actualPosition;
			MEIInt64 commandPosition;
		} axis;
        struct {
            /* Data associated with the CAN event. */
            long data[4];
        } can;
        struct {
			long sampleCounter;
			long dedicatedIn;
			MEIInt64 encoderPosition;
		} motor;
		long word[MEIXmpSignalUserData];
	} data;
} MEIEventStatusInfo;

typedef enum {
	MEIEventTraceFIRST = MEITraceLAST << 1,

	MEIEventTraceLAST  = MEIEventTraceFIRST << 1
} MEIEventTrace;

/* EventMgr */
typedef MPIEmpty	MEIEventMgrConfig;

typedef enum {
	MEIEventMgrTraceFIRST = MEITraceLAST << 1,

	MEIEventMgrTraceLAST  = MEIEventMgrTraceFIRST << 1
} MEIEventMgrTrace;

typedef struct MEIEventMgrServiceConfig {
	MPI_BOOL	allProcesses;	/* TRUE => collect events from all processes, else EventMgr process only */
} MEIEventMgrServiceConfig;

MPI_DECL1 long MPI_DECL2
	meiEventMgrServiceConfigGet(MPIEventMgr					eventMgr,
								MEIEventMgrServiceConfig	*config);

MPI_DECL1 long MPI_DECL2
	meiEventMgrServiceConfigSet(MPIEventMgr					eventMgr,
								MEIEventMgrServiceConfig	*config);

/* Filter */
typedef enum {
	MEIFilterGainIndexINVALID	= -1,
	
	/* Gain table index for normal firmware. */
	MEIFilterGainIndexNO_MOTION	= MEIXmpGainNOT_MOVING,
	MEIFilterGainIndexACCEL		= MEIXmpGainACCEL,
	MEIFilterGainIndexDECEL		= MEIXmpGainDECEL,
	MEIFilterGainIndexVELOCITY	= MEIXmpGainCONSTANT_VEL,

	/* Gain table index for Custom 1 firmware. */
	MEIFilterGainIndexSTOPPING2	= MEIXmpGainSTOPPED2,
	MEIFilterGainIndexSTOPPING1	= MEIXmpGainSTOPPED1,
	MEIFilterGainIndexSETTLING	= MEIXmpGainSETTLING,
	MEIFilterGainIndexMOVING	= MEIXmpGainMOVING,
	MEIFilterGainIndexSTOPPING3	= MEIXmpGainSTOPPED3,

	/* Gain table index for Custom 5 firmware. */
	MEIFilterGainIndexMIN		= MEIXmpGainMIN,
	MEIFilterGainIndexMAX		= MEIXmpGainMAX,
	MEIFilterGainIndexNONE		= MEIXmpGainNONE,
	MEIFilterGainIndexSLOPE		= MEIXmpGainSLOPE,

	MEIFilterGainIndexLAST		= MEIXmpGainLAST,
	MEIFilterGainIndexALL		= MEIFilterGainIndexLAST,	/* used for gain get/set() */
	MEIFilterGainIndexFIRST		= MEIFilterGainIndexINVALID + 1,

	MEIFilterGainIndexDEFAULT	= MEIFilterGainIndexNO_MOTION
} MEIFilterGainIndex;

typedef enum {
    MEIDataTypeINVALID = 0, /* this should stay zero - static arrays init to zero by default */

    MEIDataTypeCHAR,
    MEIDataTypeSHORT,
    MEIDataTypeUSHORT,
    MEIDataTypeLONG,
    MEIDataTypeULONG,
    MEIDataTypeFLOAT,
    MEIDataTypeDOUBLE,
	MEIDataTypeINT64,
	MEIDataTypeUINT64,

    MEIDataTypeLAST,
    MEIDataTypeFIRST   = MEIDataTypeINVALID + 1
} MEIDataType;

typedef	struct MEIFilterGainPID {
	struct {
		float	proportional;	/* Kp */
		float	integral;		/* Ki */
		float	derivative;		/* Kd */
	} gain;
	struct {
		float	position;		/* Kpff */
		float	velocity;		/* Kvff */
		float	acceleration;	/* Kaff */
		float	friction;		/* Kfff */
	} feedForward;
	struct {
		float	moving;			/* MovingIMax */
		float	rest;			/* RestIMax */
	} integrationMax;
	long	dRate;				/* DRate */
	struct {
		float	limit;			/* OutputLimit */
		float	limitHigh;		/* OutputLimitHigh */
		float	limitLow;		/* OutputLimitLow */
		float	offset;			/* OutputOffset */
	} output;
	struct {
		float	positionFFT;	/* Ka0 */
		float	filterFFT;		/* Ka1 */
		float	velocityFFT;	/* Ka2 */
	} noise;
} MEIFilterGainPID;

typedef	enum {
	MEIFilterGainPIDCoeffINVALID = -1,

	MEIFilterGainPIDCoeffGAIN_PROPORTIONAL,		/* Kp */
	MEIFilterGainPIDCoeffGAIN_INTEGRAL,			/* Ki */
	MEIFilterGainPIDCoeffGAIN_DERIVATIVE,		/* Kd */

	MEIFilterGainPIDCoeffFEEDFORWARD_POSITION,		/* Kpff */
	MEIFilterGainPIDCoeffFEEDFORWARD_VELOCITY,		/* Kvff */
	MEIFilterGainPIDCoeffFEEDFORWARD_ACCELERATION,	/* Kaff */
	MEIFilterGainPIDCoeffFEEDFORWARD_FRICTION,		/* Kfff */

	MEIFilterGainPIDCoeffINTEGRATIONMAX_MOVING,	/* MovingIMax */
	MEIFilterGainPIDCoeffINTEGRATIONMAX_REST,	/* RestIMax */

	MEIFilterGainPIDCoeffDRATE,					/* DRate */

	MEIFilterGainPIDCoeffOUTPUT_LIMIT,			/* OutputLimit */
	MEIFilterGainPIDCoeffOUTPUT_LIMITHIGH,		/* OutputLimitHigh */
	MEIFilterGainPIDCoeffOUTPUT_LIMITLOW,		/* OutputLimitLow */
	MEIFilterGainPIDCoeffOUTPUT_OFFSET,			/* OutputOffset */

	MEIFilterGainPIDCoeffNOISE_POSITIONFFT,		/* Ka0 */
	MEIFilterGainPIDCoeffNOISE_FILTERFFT,		/* Ka1 */
	MEIFilterGainPIDCoeffNOISE_VELOCITYFFT,		/* Ka2 */

	MEIFilterGainPIDCoeffLAST,
	MEIFilterGainPIDCoeffFIRST = MEIFilterGainPIDCoeffINVALID + 1
} MEIFilterGainPIDCoeff;

static MEIDataType MEIFilterGainTypePID[MPIFilterCoeffCOUNT_MAX] =
{
    MEIDataTypeFLOAT, /* Kp               */
    MEIDataTypeFLOAT, /* Ki               */
    MEIDataTypeFLOAT, /* Kd               */

    MEIDataTypeFLOAT, /* Kpff             */
    MEIDataTypeFLOAT, /* Kvff             */
    MEIDataTypeFLOAT, /* Kaff             */
    MEIDataTypeFLOAT, /* Kfff             */

    MEIDataTypeFLOAT, /* MovingIMax       */
    MEIDataTypeFLOAT, /* RestIMax         */

    MEIDataTypeLONG,  /* DRate            */

    MEIDataTypeFLOAT, /* OutputLimit      */
    MEIDataTypeFLOAT, /* OutputLimitHigh  */
    MEIDataTypeFLOAT, /* OutputLimitLow   */
    MEIDataTypeFLOAT, /* OutputOffset     */
    MEIDataTypeFLOAT, /* Ka0              */
    MEIDataTypeFLOAT, /* Ka1              */
    MEIDataTypeFLOAT  /* Ka2              */
};

typedef	struct MEIFilterGainPIV {
	struct {
		float	proportional;	/* Kpp */
		float	integral;		/* Kip */
	} gainPosition;
	struct {
		float	proportional;	/* Kpv */
	} gainVelocity1;
	struct {
		float	position;		/* Kpff */
		float	velocity;		/* Kvff */
		float	acceleration;	/* Kaff */
		float	friction;		/* Kfff */
	} feedForward;
	struct {
		float	moving;			/* MovingIMax */
		float	rest;			/* RestIMax */
	} integrationMax;
	struct {
		float	feedback;		/* Kdv */
	} gainVelocity2;
	struct {
		float	limit;			/* OutputLimit */
		float	limitHigh;		/* OutputLimitHigh */
		float	limitLow;		/* OutputLimitLow */
		float	offset;			/* OutputOffset */
	} output;
	struct {
		float	integral;		/* Kiv */
		float	integrationMax;	/* VintMax */
	} gainVelocity3;
	struct {
		float	positionFFT;	/* Ka0 */
		float   smoothing;		/* Ka1 */
		float	filterFFT;		/* Ka2 */
	} noise;
} MEIFilterGainPIV;

typedef	enum {
	MEIFilterGainPIVCoeffINVALID = -1,

	MEIFilterGainPIVCoeffGAINPOSITION_PROPORTIONAL,	/* Kpp */
	MEIFilterGainPIVCoeffGAINPOSITION_INTEGRAL,		/* Kip */

	MEIFilterGainPIVCoeffGAINVELOCITY_PROPORTIONAL,	/* Kpv */

	MEIFilterGainPIVCoeffFEEDFORWARD_POSITION,		/* Kpff */
	MEIFilterGainPIVCoeffFEEDFORWARD_VELOCITY,		/* Kvff */
	MEIFilterGainPIVCoeffFEEDFORWARD_ACCELERATION,	/* Kaff */
	MEIFilterGainPIVCoeffFEEDFORWARD_FRICTION,		/* Kfff */

	MEIFilterGainPIVCoeffINTEGRATIONMAX_MOVING,		/* MovingIMax */
	MEIFilterGainPIVCoeffINTEGRATIONMAX_REST,		/* RestIMax */

	MEIFilterGainPIVCoeffGAINVELOCITY_FEEDBACK,		/* Kdv */

	MEIFilterGainPIVCoeffOUTPUT_LIMIT,				/* OutputLimit */
	MEIFilterGainPIVCoeffOUTPUT_LIMITHIGH,			/* OutputLimitHigh */
	MEIFilterGainPIVCoeffOUTPUT_LIMITLOW,			/* OutputLimitLow */
	MEIFilterGainPIVCoeffOUTPUT_OFFSET,				/* OutputOffset */

	MEIFilterGainPIVCoeffGAINVELOCITY_INTEGRAL,			/* Kiv */
	MEIFilterGainPIVCoeffGAINVELOCITY_INTEGRATIONMAX,	/* Vintmax */

	MEIFilterGainPIVCoeffNOISE_POSITIONFFT,			/* Ka0 */
	MEIFilterGainPIVCoeffSMOOTHINGFILTER_GAIN,		/* Ka1 */
	MEIFilterGainPIVCoeffNOISE_FILTERFFT,			/* Ka2 */

	MEIFilterGainPIVCoeffLAST,
	MEIFilterGainPIVCoeffFIRST = MEIFilterGainPIVCoeffINVALID + 1
} MEIFilterGainPIVCoeff;

static MEIDataType MEIFilterGainTypePIV[MPIFilterCoeffCOUNT_MAX] =
{
    MEIDataTypeFLOAT, /* Kpp              */
    MEIDataTypeFLOAT, /* Kip              */

    MEIDataTypeFLOAT, /* Kpv              */

    MEIDataTypeFLOAT, /* Kpff             */
    MEIDataTypeFLOAT, /* Kvff             */
    MEIDataTypeFLOAT, /* Kaff             */
    MEIDataTypeFLOAT, /* Kfff             */

    MEIDataTypeFLOAT, /* MovingIMax       */
    MEIDataTypeFLOAT, /* RestIMax         */

    MEIDataTypeFLOAT, /* Kdv              */

    MEIDataTypeFLOAT, /* OutputLimit      */
    MEIDataTypeFLOAT, /* OutputLimitHigh  */
    MEIDataTypeFLOAT, /* OutputLimitLow   */
    MEIDataTypeFLOAT, /* OutputOffset     */

    MEIDataTypeFLOAT, /* Kiv              */
    MEIDataTypeFLOAT, /* Vintmax          */

    MEIDataTypeFLOAT, /* Ka0              */
	MEIDataTypeFLOAT, /* Ka1              */
	MEIDataTypeFLOAT  /* Ka2              */
};

typedef	struct MEIFilterGainStep {
	long	stepCommand;
} MEIFilterGainStep;

typedef	enum {
	MEIFilterGainStepCoeffINVALID = -1,

	MEIFilterGainStepCoeffSTEP_COMMAND,				/* Step Command */

	MEIFilterGainStepCoeffLAST,
	MEIFilterGainStepCoeffFIRST = MEIFilterGainStepCoeffINVALID + 1
} MEIFilterGainStepCoeff;

typedef struct MEIFilterConfig {
	char					userLabel[MEIObjectLabelCharMAX+1]; /* +1 for NULL terminator */
   
	MEIXmpAlgorithm			Algorithm;

	MEIXmpAxisInput			Axis[MEIXmpFilterAxisInputs];

	long					*VelPtr;

	MEIXmpSwitchType		GainSwitchType;
	float					GainDelay;
	long					GainWindow;
	MEIXmpSwitchType		PPISwitchType;
	MEIXmpPPIMode			PPIMode;
	float					PPIDelay;
	long					PPIWindow;
	MEIXmpIntResetConfig	ResetIntegratorConfig;
	float					ResetIntegratorDelay;

	MEIXmpFilterForm		PostFilterForm;
	MEIXmpPostFilter		PostFilter;
} MEIFilterConfig;

typedef enum {
	MEIFilterTraceFIRST = MEITraceLAST << 1,

	MEIFilterTraceLAST  = MEIFilterTraceFIRST << 1
} MEIFilterTrace;

/* Filter -- Postfilter */

#define MEIPi		(3.141592653589793238462643383279502884)
#define	MEI2Pi		(6.283185307179586476925286766559005768)
#define MEIRoot2	(1.4142135623730950488016887242097)
#define MEIe		(2.7182818284590452353602874713527)

typedef enum {
	MEIFilterTypeINVALID = -1,

	MEIFilterTypeUNITY_GAIN, /* B0 = 1    B1=B2=A1=A2 = 0    (effectively acting as no filter) */
	MEIFilterTypeSINGLE_ORDER,
	MEIFilterTypeLOW_PASS,
	MEIFilterTypeHIGH_PASS,
	MEIFilterTypeNOTCH,
	MEIFilterTypeRESONATOR,
	MEIFilterTypeLEAD_LAG,
	MEIFilterTypeZERO_GAIN,	/* b0=b1=b2=a1=a2 = 0   (this does act as a filter.... zeroing the output) */
	MEIFilterTypeBIQUAD,

	/* Only valid for setting.  Reading will not return these types */
	MEIFilterTypeDIGITAL_BIQUAD,
	MEIFilterTypePOLES_ZEROS,
	MEIFilterTypeDIGITAL_POLES_ZEROS,

	MEIFilterTypeUNKNOWN,	/* algorithm couldn't figure out what this filter was from the coeffs! Use the digital information */ 
	MEIFilterTypeLAST,
	MEIFilterTypeFIRST = MEIFilterTypeINVALID + 1

} MEIFilterType;

typedef enum{
		MEIFilterFormINVALID = -1,

		MEIFilterFormIIR,
		MEIFilterFormBIQUAD,
		MEIFilterFormSS_BIQUAD,
		MEIFilterFormINT_BIQUAD,
		MEIFilterFormINT_SS_BIQUAD,

		MEIFilterFormLAST,
		MEIFilterFormFIRST = MEIFilterTypeINVALID + 1

} MEIFilterForm;

#define MEIMaxBiQuadSections	(6)

typedef struct MEIPostfilterSection {

	MEIFilterType	type;
	MEIFilterForm	form;

	struct {

		struct {
			double breakPoint;		/* Hz */
		} lowPass;

		struct {
			double breakPoint;		/* Hz */
		} highPass;

		struct {
			double centerFrequency;	/* Hz */
			double bandwidth;		/* Hz */
		} notch;

		struct {
			double centerFrequency;	/* Hz */
			double bandwidth;		/* Hz */
			double gain;			/* dB */
		} resonator;

		struct {
			double lowFrequencyGain;	/* dB */
			double highFrequencyGain;	/* dB */
			double centerFrequency;		/* Hz */
		} leadLag;

		struct {
			double a1;
			double a2;
			double b0;
			double b1;
			double b2;
		} biquad;

		struct {
			double a1;
			double a2;
			double b0;
			double b1;
			double b2;
		} digitalBiquad;

		struct {
			long poleCount;
			long zeroCount;
			struct {
				double real;
				double imag;
			} pole[2];
			struct {
				double real;
				double imag;
			} zero[2];
		} polesZeros;

		struct {
			long poleCount;
			long zeroCount;
			struct {
				double real;
				double imag;
			} pole[2];
			struct {
				double real;
				double imag;
			} zero[2];
		} digitalPolesZeros;

		struct {
			double d1;
			double c1;
			double c2;
			double a2;
			double a1;
			double b1;
		} stateSpaceBiquad;

		struct {
			long numberOfCoefficients;
			double coeff[MEIXmpMAX_PostFilterSize];
		} iir;

	} data;

} MEIPostfilterSection;

/* Get all sections of a postfilter */
MPI_DECL1 long MPI_DECL2
	 meiFilterPostfilterGet(MPIFilter				 filter,
							long					*sectionCount,
							MEIPostfilterSection	*sections);


/* Set multiple sections of a postfilter.  Sections 0 through (numberOfSections-1) */
MPI_DECL1 long MPI_DECL2
	 meiFilterPostfilterSet(MPIFilter				 filter,
							long					 sectionCount,
							MEIPostfilterSection	*sections);

/* Get a section of a postfilter */
MPI_DECL1 long MPI_DECL2
	 meiFilterPostfilterSectionGet(MPIFilter			 filter,
								   long					 sectionNumber,
								   MEIPostfilterSection	*section);


/* Set a section of a postfilter */
MPI_DECL1 long MPI_DECL2
	 meiFilterPostfilterSectionSet(MPIFilter			 filter,
								   long					 sectionNumber,
								   MEIPostfilterSection	*section);

/* Motion */

typedef enum {
	MEIMotionMessageRESERVED0 = MPIMotionMessageLAST,
	MEIMotionMessageRESERVED1,
	MEIMotionMessageRESERVED2,
	MEIMotionMessageNO_AXES_MAPPED,
	MEIMotionMessageBAD_PATH_DATA,
	MEIMotionMessageBAD_MOVE_DELTA,

	MEIMotionMessageLAST
} MEIMotionMessage;

typedef enum {
	MEIMotionTraceFIRST = MEITraceLAST << 1,

	MEIMotionTraceSTATUS = MEIMotionTraceFIRST << 0,
	MEIMotionTracePARAMS = MEIMotionTraceFIRST << 1,

	MEIMotionTraceLAST  = MEIMotionTraceFIRST << 2
} MEIMotionTrace;

typedef struct	MEIMotionConfig {
	char	userLabel[MEIObjectLabelCharMAX+1]; /* +1 for NULL terminator */
	
	long	axisCount;
	long	axisNumber[MEIXmpMAX_COORD_AXES];
	double	blendLimit;
} MEIMotionConfig;

/* Custom Motion Attributes */

typedef enum {
	MEIMotionAttrLAST	= MPIMotionAttrFIRST,

	MEIMotionAttrEVENT			= MEIMotionAttrLAST - 1,
	MEIMotionAttrFINAL_VEL		= MEIMotionAttrLAST - 2,
	MEIMotionAttrNO_REVERSAL	= MEIMotionAttrLAST - 3,
	MEIMotionAttrHOLD			= MEIMotionAttrLAST - 4,
	MEIMotionAttrHOLD_LESS		= MEIMotionAttrLAST - 5,
	MEIMotionAttrHOLD_GREATER	= MEIMotionAttrLAST - 6,
	MEIMotionAttrOUTPUT			= MEIMotionAttrLAST - 7,

	MEIMotionAttrFIRST			= MEIMotionAttrLAST - 7,

	MEIMotionAttrCOUNT	= MEIMotionAttrLAST - MEIMotionAttrFIRST
} MEIMotionAttr;

typedef enum {
	MEIMotionAttrMaskEVENT			= mpiMotionAttrMaskBIT(MEIMotionAttrEVENT),
	MEIMotionAttrMaskFINAL_VEL		= mpiMotionAttrMaskBIT(MEIMotionAttrFINAL_VEL),
	MEIMotionAttrMaskNO_REVERSAL	= mpiMotionAttrMaskBIT(MEIMotionAttrNO_REVERSAL),
	MEIMotionAttrMaskHOLD			= mpiMotionAttrMaskBIT(MEIMotionAttrHOLD),
	MEIMotionAttrMaskHOLD_LESS		= mpiMotionAttrMaskBIT(MEIMotionAttrHOLD_LESS),
	MEIMotionAttrMaskHOLD_GREATER	= mpiMotionAttrMaskBIT(MEIMotionAttrHOLD_GREATER),
	MEIMotionAttrMaskOUTPUT			= mpiMotionAttrMaskBIT(MEIMotionAttrOUTPUT),

	MEIMotionAttrMaskALL	= ((-1 << MEIMotionAttrFIRST) & ~MPIMotionAttrMaskALL)
} MEIMotionAttrMask;

/* Motion Attribute Parameters */

typedef enum {
	MEIMotionAttrHoldTypeINVALID	= -1,

	MEIMotionAttrHoldTypeNONE,
	MEIMotionAttrHoldTypeGATE,
	MEIMotionAttrHoldTypeINPUT,
	MEIMotionAttrHoldTypeMOTOR_GENERAL_IO,
	MEIMotionAttrHoldTypeMOTOR_DEDICATED_IO,
	MEIMotionAttrHoldTypeAXIS_POSITION_ACTUAL,
	MEIMotionAttrHoldTypeAXIS_POSITION_COMMAND,
	MEIMotionAttrHoldTypeUSER_ADDRESS,

	MEIMotionAttrHoldTypeLAST,
	MEIMotionAttrHoldTypeFIRST = MEIMotionAttrHoldTypeINVALID + 1
} MEIMotionAttrHoldType;

typedef union {
	long		gate;
	struct {
		long	*input;
		long	mask;
		long	pattern;
	} input;
	struct {
		long	number;
		long	mask;
		long	pattern;
	} motor;
	struct {
		long	number;
		long	position;
	} axis;
	struct {
		long	*address;
		long	mask;
		long	pattern;
	} user;
} MEIMotionAttrHoldSource;

typedef	struct MEIMotionAttrHold {
	MEIMotionAttrHoldType	type;
	MEIMotionAttrHoldSource	source;

	float	timeout;
} MEIMotionAttrHold;

typedef enum {
	MEIMotionAttrOutputTypeINVALID	= -1,

	MEIMotionAttrOutputTypeNONE,
	MEIMotionAttrOutputTypeMOTOR,
	MEIMotionAttrOutputTypeOUTPUT,

	MEIMotionAttrOutputTypeLAST,
	MEIMotionAttrOutputTypeFIRST = MEIMotionAttrOutputTypeINVALID + 1
} MEIMotionAttrOutputType;

typedef	struct MEIMotionAttrOutput {
	MEIMotionAttrOutputType	type;
	union {
		long	*output;
		long	motor;
	} as;
	long	offMask;
	long	onMask;
	long	pointIndex;					/* MEIMotionAttrMaskOUTPUT for path motion - point index for turning on output - used with point lists */
} MEIMotionAttrOutput;

typedef struct MEIMotionAttributes {
	MPIEventMask		eventMask;		/* MEIMotionAttrMaskEVENT */
	double				*finalVelocity;	/* MEIMotionAttrMaskFINAL_VEL */
	MEIMotionAttrHold	*hold;			/* MEIMotionAttrMaskHOLD */
	long				*outputCount;	/* MEIMotionAttrMaskOUTPUT for path motion - number of outputs - per axis */
	MEIMotionAttrOutput	*output;		/* MEIMotionAttrMaskOUTPUT for path and non path motion - outputs - per axis */
} MEIMotionAttributes;

#define	meiMotionAttrMaskATTRIBUTES_REQUIRED(attrMask) \
			((attrMask) & \
			 (MEIMotionAttrMaskEVENT | \
			  MEIMotionAttrMaskFINAL_VEL | \
			  MEIMotionAttrMaskHOLD |	 \
			  MEIMotionAttrMaskHOLD_LESS |	 \
			  MEIMotionAttrMaskHOLD_GREATER |	 \
			  MEIMotionAttrMaskOUTPUT))			   

/* Custom Motion Types */

typedef enum {
	MEIMotionTypeFRAME	= MPIMotionTypeLAST,
	MEIMotionTypeUSER1,
	MEIMotionTypeUSER2,
	MEIMotionTypeUSER3,
	MEIMotionTypeUSER4,
	
	MEIMotionTypeLAST,
	MEIMotionTypeFIRST	= MPIMotionTypeLAST,

	MEIMotionTypeUSER_FIRST = MEIMotionTypeUSER1,
	MEIMotionTypeUSER_LAST = MEIMotionTypeUSER4 + 1,
	MEIMotionTypeUSER_COUNT = (MEIMotionTypeUSER_LAST - MEIMotionTypeUSER_FIRST)
} MEIMotionType;

typedef	struct MEIMotionFrame {
	long		pointCount;
	MEIXmpFrame	*frame;

	MPIMotionPoint	point;
} MEIMotionFrame;

typedef	struct MEIMotionParams {
	MEIMotionFrame	frame;

	MPIMotionAttributes	attributes;
	MEIMotionAttributes	attributesMEI;
} MEIMotionParams;


typedef long  				/* Callback prototype */
	 (MPI_DEF2 *MEIMotionActionFunction)(MPIMotion	motion,
							   MPIMotionType type,
							   MEIXmpAction	xmpAction);
MPI_DECL1 long MPI_DECL2
	meiMotionExecute(MPIMotion			motion,
					 MPIMotionType		type,
					 MPIMotionParams	*params,
					 MEIXmpAction		action);

		/* Callback prototype */
typedef long (MPI_DEF2 *MEIMotionUserExecuteFunction)(MPIMotion	motion,
										  MPIMotionType		type,
										  MPIMotionParams	*params,
										  MEIXmpAction		action);

MPI_DECL1 long MPI_DECL2
	meiMotionUserExecuteFunction(MPIMotionType type,
						  MEIMotionUserExecuteFunction function, MEIMotionUserExecuteFunction *oldFunction);

MPI_DECL1 long MPI_DECL2
	meiMotionFrameBufferLoad(MPIMotion	motion,
							 MPI_BOOL	initial,			/* TRUE/FALSE */
							 MPI_BOOL	lock,				/* TRUE/FALSE */
							 MPI_BOOL	frameLowEvent);		/* TRUE/FALSE */

typedef struct MEIMotionFrameBufferStatus {
	long	size;
	long	frameCount;
} MEIMotionFrameBufferStatus;

MPI_DECL1 long MPI_DECL2
	meiMotionFrameBufferStatus(MPIMotion	motion,
							   MEIMotionFrameBufferStatus	*status);

MPI_DECL1 long MPI_DECL2
	meiMotionLogClose(MPIMotion motion);

MPI_DECL1 long MPI_DECL2
	meiMotionLogOpen(MPIMotion motion,
					 long maxEntries);

MPI_DECL1 long MPI_DECL2
	meiMotionLogPrint(MPIMotion motion,
					 const char *fileName);

MPI_DECL1 long MPI_DECL2
	meiMotionParamsValidate(MPIMotion		motion,
							MPIMotionType	type,
							MPIMotionParams	*params,
							MEIXmpAction	action,
							long			*points);

MPI_DECL1 MEIMotionActionFunction MPI_DECL2
  meiMotionActionFunction(MPIMotion motion,
						  MEIMotionActionFunction function);

/* Motor */

typedef enum {
	MEIMotorMessageMOTOR_NOT_ENABLED= MPIMotorMessageLAST,
	MEIMotorMessageSECONDARY_ENCODER_NA,
	MEIMotorMessageHARDWARE_NOT_FOUND,
	MEIMotorMessageSTEPPER_INVALID,
	MEIMotorMessageDISABLE_ACTION_INVALID,
	MEIMotorMessagePULSE_WIDTH_INVALID,
	MEIMotorMessageFEEDBACK_REVERSAL_NA,
	MEIMotorMessageFILTER_DISABLE_NA,
	MEIMotorMessagePHASE_FINDING_FAILED,
	MEIMotorMessageDEMAND_MODE_UNSUPPORTED,
	MEIMotorMessageDEMAND_MODE_NOT_SET,
	MEIMotorMessagePOSITION_FEEDBACK_OFFSET_UNSUPPORTED,
	MEIMotorMessagePOSITION_FEEDBACK_OFFSET_SET_FAILED,
	MEIMotorMessagePOSITION_FEEDBACK_OFFSET_UNINITIALIZED,

	MEIMotorMessageLAST
} MEIMotorMessage;

/* Motor amplifier faults  */
#define MEIMotorAmpFaultsMAX	(32)	/* max faults per motor	  */
#define MEIMotorAmpMsgMAX		(200)	/* max chars per message  */

typedef char MEIMotorAmpFaultMsg[MEIMotorAmpMsgMAX];

typedef struct MEIMotorAmpFaults {
	long				count;
	long				code[MEIMotorAmpFaultsMAX];
	MEIMotorAmpFaultMsg	message[MEIMotorAmpFaultsMAX];
} MEIMotorAmpFaults;


MPI_DECL1 long MPI_DECL2  
	meiMotorAmpFault(MPIMotor			motor,
					 MEIMotorAmpFaults	*fault);

MPI_DECL1 long MPI_DECL2  
	meiMotorAmpFaultClear(MPIMotor motor);

/* Motor amplifier  warnings */
#define MEIMotorAmpWarningsMAX	(32)	/* max Warnings per motor	  */

typedef char MEIMotorAmpWarningMsg[MEIMotorAmpMsgMAX];

typedef struct MEIMotorAmpWarnings {
	long					count;
	long					code[MEIMotorAmpWarningsMAX];
	MEIMotorAmpWarningMsg	message[MEIMotorAmpWarningsMAX];
} MEIMotorAmpWarnings;


MPI_DECL1 long MPI_DECL2  
	meiMotorAmpWarning(MPIMotor			motor,
					   MEIMotorAmpWarnings	*warning);

MPI_DECL1 long MPI_DECL2  
	meiMotorAmpWarningClear(MPIMotor motor);


MPI_DECL1 long MPI_DECL2  
	meiMotorMultiTurnReset(MPIMotor motor);


typedef MEIXmpLimitData	MEIMotorEventConfig;

typedef enum MEIMotorStepperPulseType {
	MEIMotorStepperPulseTypeINVALID = -1,

	MEIMotorStepperPulseTypeSTEP,	
	MEIMotorStepperPulseTypeDIR,
	
	MEIMotorStepperPulseTypeCW,	
	MEIMotorStepperPulseTypeCCW,

	MEIMotorStepperPulseTypeQUADA,
	MEIMotorStepperPulseTypeQUADB,

	MEIMotorStepperPulseTypeLAST,
	MEIMotorStepperPulseTypeFIRST =  MEIMotorStepperPulseTypeINVALID + 1

} MEIMotorStepperPulseType;

typedef struct MEIMotorStepperPulse {
	MEIMotorStepperPulseType	type;
	MPI_BOOL					invert;
} MEIMotorStepperPulse;

typedef struct MEIMotorStepper {
	MPI_BOOL				loopback;	/* TRUE = FPGA pulse feedback, FALSE = encoder feedback */
	MEIMotorStepperPulse	pulseA;
	MEIMotorStepperPulse	pulseB;
	double					pulseWidth;	/* output pulse width  (sec), 10^(-7) < pulseWidth < 10^(-3) */
	
} MEIMotorStepper;

typedef struct MEIMotorEncoderRatio {
	long	A;	  /* denominator */
	long	B;	  /* numerator */
} MEIMotorEncoderRatio;

typedef struct MEIMotorEncoderModulo {
	MPI_BOOL	reverse;	/* set to TRUE if external encoder is already moduloed at a non 32 bit boundary */
	long		value;
} MEIMotorEncoderModulo;

typedef enum MEIMotorEncoderType {
	MEIMotorEncoderTypeINVALID = -1,

	MEIMotorEncoderTypeQUAD_AB = MEIXmpMotorEncoderConfigTypeQUAD_AB,
	MEIMotorEncoderTypeDRIVE = MEIXmpMotorEncoderConfigTypeDRIVE,
	MEIMotorEncoderTypeSSI = MEIXmpMotorEncoderConfigTypeSSI,

	MEIMotorEncoderTypeLAST,
	MEIMotorEncoderTypeFIRST = MEIMotorEncoderTypeINVALID + 1

} MEIMotorEncoderType;


typedef enum MEIMotorSsiInput{
	MEIMotorSsiInputINVALID = -1,
	
	MEIMotorSsiInputENC_A,
	MEIMotorSsiInputENC_B,
	MEIMotorSsiInputENC_I,
	MEIMotorSsiInputGPIO_0,
	MEIMotorSsiInputGPIO_1,
	MEIMotorSsiInputGPIO_2,
	MEIMotorSsiInputGPIO_3,
	MEIMotorSsiInputGPIO_4,
	MEIMotorSsiInputGPIO_5,
	MEIMotorSsiInputGPIO_6,

	MEIMotorSsiInputLAST,
	MEIMotorSsiInputFIRST = MEIMotorSsiInputINVALID,

} MEIMotorSsiInput;



typedef struct MEIMotorEncoderSsiConfig {
	MEIMotorSsiInput	input;
	long				bitCount;
	long				baudRate; /* units = hertz */
	MPI_BOOL			brokenWireEnable;
} MEIMotorEncoderSsiConfig;

typedef struct MEIMotorEncoder {
	MEIMotorEncoderType				type;
	long							encoderPhase;     /* 0 => normal, else reversed */
	long							filterDisable;	  /* 0 => quad filter enabled, else not enabled */
	MEIMotorEncoderRatio			ratio;
	MEIMotorEncoderModulo			modulo;
	MEIMotorEncoderSsiConfig		ssiConfig;
} MEIMotorEncoder;

typedef struct MEIMotorDacChannelConfig {
	float				Offset;		/* volts */
	float				Scale;
	MEIXmpDACInputType	InputType;
	MEIXmpGenericValue	*Input;
} MEIMotorDacChannelConfig;

typedef struct MEIMotorDacConfig {
	MEIXmpDACPhase				Phase;
	MEIMotorDacChannelConfig	Cmd;
	MEIMotorDacChannelConfig	Aux;
} MEIMotorDacConfig;

typedef struct MEIMotorStatusOutput {
	long    		*outPtr;
	MEIXmpIOMask	ioMask[MEIXmpMotorStatusOutputs];
} MEIMotorStatusOutput;

typedef enum MEIMotorIoType {
	MEIMotorIoTypeINVALID = -1,

	MEIMotorIoTypeOUTPUT,

	MEIMotorIoTypePULSE_A,
	MEIMotorIoTypePULSE_B,
	MEIMotorIoTypeCOMPARE_0,
	MEIMotorIoTypeCOMPARE_1,
	MEIMotorIoTypeSOURCE5,
	MEIMotorIoTypeBRAKE, 
	MEIMotorIoTypeSSI_CLOCK0,
	MEIMotorIoTypeSSI_CLOCK1,
	MEIMotorIoTypeOUTPUT_DISABLED,

	MEIMotorIoTypeSOURCE10,
	MEIMotorIoTypeSOURCE11,
	MEIMotorIoTypeSOURCE12,
	MEIMotorIoTypeSOURCE13,
	MEIMotorIoTypeSOURCE14,
	MEIMotorIoTypeSOURCE15,

	MEIMotorIoTypeINPUT,

	MEIMotorIoTypeLAST,
	MEIMotorIoTypeFIRST = MEIMotorIoTypeINVALID+1,

	MEIMotorIoTypeSOURCE1 = MEIMotorIoTypePULSE_A,
	MEIMotorIoTypeSOURCE2 = MEIMotorIoTypePULSE_B,
	MEIMotorIoTypeSOURCE3 = MEIMotorIoTypeCOMPARE_0,
	MEIMotorIoTypeSOURCE4 = MEIMotorIoTypeCOMPARE_1,
	MEIMotorIoTypeSOURCE6 = MEIMotorIoTypeBRAKE,
	MEIMotorIoTypeSOURCE7 = MEIMotorIoTypeSSI_CLOCK0,
	MEIMotorIoTypeSOURCE8 = MEIMotorIoTypeSSI_CLOCK1,
	MEIMotorIoTypeSOURCE9 = MEIMotorIoTypeOUTPUT_DISABLED
} MEIMotorIoType;

typedef enum MEIMotorIoTypeMask {
	MEIMotorIoTypeMaskOUTPUT		= (1<<MEIMotorIoTypeOUTPUT),
	MEIMotorIoTypeMaskSOURCE1		= (1<<MEIMotorIoTypeSOURCE1),
	MEIMotorIoTypeMaskSOURCE2		= (1<<MEIMotorIoTypeSOURCE2),
	MEIMotorIoTypeMaskSOURCE3		= (1<<MEIMotorIoTypeSOURCE3),
	MEIMotorIoTypeMaskSOURCE4		= (1<<MEIMotorIoTypeSOURCE4),
	MEIMotorIoTypeMaskSOURCE5		= (1<<MEIMotorIoTypeSOURCE5),
	MEIMotorIoTypeMaskSOURCE6		= (1<<MEIMotorIoTypeSOURCE6),
	MEIMotorIoTypeMaskSOURCE7		= (1<<MEIMotorIoTypeSOURCE7),
	MEIMotorIoTypeMaskSOURCE8		= (1<<MEIMotorIoTypeSOURCE8),
	MEIMotorIoTypeMaskSOURCE9		= (1<<MEIMotorIoTypeSOURCE9),
	MEIMotorIoTypeMaskSOURCE10		= (1<<MEIMotorIoTypeSOURCE10),
	MEIMotorIoTypeMaskSOURCE11		= (1<<MEIMotorIoTypeSOURCE11),
	MEIMotorIoTypeMaskSOURCE12		= (1<<MEIMotorIoTypeSOURCE12),
	MEIMotorIoTypeMaskSOURCE13		= (1<<MEIMotorIoTypeSOURCE13),
	MEIMotorIoTypeMaskSOURCE14		= (1<<MEIMotorIoTypeSOURCE14),
	MEIMotorIoTypeMaskSOURCE15		= (1<<MEIMotorIoTypeSOURCE15),

	MEIMotorIoTypeMaskINPUT			= (1<<MEIMotorIoTypeINPUT),

	MEIMotorIoTypeMaskPULSE_A		= (1<<MEIMotorIoTypePULSE_A),
	MEIMotorIoTypeMaskPULSE_B		= (1<<MEIMotorIoTypePULSE_B),
	MEIMotorIoTypeMaskCOMPARE_0		= (1<<MEIMotorIoTypeCOMPARE_0),
	MEIMotorIoTypeMaskCOMPARE_1		= (1<<MEIMotorIoTypeCOMPARE_1),
	MEIMotorIoTypeMaskBRAKE			= (1<<MEIMotorIoTypeBRAKE),
	MEIMotorIoTypeMaskSSI_CLOCK0	= (1<<MEIMotorIoTypeSSI_CLOCK0),
	MEIMotorIoTypeMaskSSI_CLOCK1	= (1<<MEIMotorIoTypeSSI_CLOCK1),
	MEIMotorIoTypeMaskOUTPUT_DISABLED = (1<<MEIMotorIoTypeOUTPUT_DISABLED),


} MEIMotorIoTypeMask;

typedef struct MEIMotorIoConfig {
	MEIMotorIoType			Type;		
} MEIMotorIoConfig;

typedef enum MEIMotorFaultBit {
	MEIMotorFaultBitINVALID	=	-1,

	MEIMotorFaultBitAMP_FAULT,	 
	MEIMotorFaultBitDRIVE_FAULT,
	MEIMotorFaultBitWATCHDOG_FAULT,
	MEIMotorFaultBitCHECKSUM_ERROR,
	MEIMotorFaultBitFEEDBACK_FAULT,
	MEIMotorFaultBitAMP_NOT_POWERED,
	MEIMotorFaultBitDRIVE_NOT_READY,
	MEIMotorFaultBitFEEDBACK_FAULT_SECONDARY,
	
	MEIMotorFaultBitLAST,
	MEIMotorFaultBitFIRST = MEIMotorFaultBitINVALID + 1
} MEIMotorFaultBit;

typedef enum MEIMotorFaultMask {
	MEIMotorFaultMaskAMP                      =	(1<< MEIMotorFaultBitAMP_FAULT),	 
	MEIMotorFaultMaskDRIVE                    =	(1<< MEIMotorFaultBitDRIVE_FAULT),
	MEIMotorFaultMaskWATCHDOG                 =	(1<< MEIMotorFaultBitWATCHDOG_FAULT),
	MEIMotorFaultMaskCHECKSUM                 =	(1<< MEIMotorFaultBitCHECKSUM_ERROR),
	MEIMotorFaultMaskFEEDBACK                 =	(1<< MEIMotorFaultBitFEEDBACK_FAULT),
	MEIMotorFaultMaskAMP_NOT_POWERED          =	(1<< MEIMotorFaultBitAMP_NOT_POWERED),
	MEIMotorFaultMaskDRIVE_NOT_READY          =	(1<< MEIMotorFaultBitDRIVE_NOT_READY),
	MEIMotorFaultMaskFEEDBACK_FAULT_SECONDARY =	(1<< MEIMotorFaultBitFEEDBACK_FAULT_SECONDARY)
} MEIMotorFaultMask;

typedef struct MEIMotorFaultConfig {
	MEIMotorFaultMask	faultMask;	/* sets the MEIMotorDedicatedInAMP_FAULT state */
} MEIMotorFaultConfig;	

typedef enum MEIMotorIoConfigIndex {
	MEIMotorIoConfigIndexINVALID = -1,
		
	MEIMotorIoConfigIndex0,
	MEIMotorIoConfigIndex1,
	MEIMotorIoConfigIndex2,
	MEIMotorIoConfigIndex3,
	MEIMotorIoConfigIndex4,
	MEIMotorIoConfigIndex5,
	MEIMotorIoConfigIndex6,
	MEIMotorIoConfigIndex7,
	MEIMotorIoConfigIndex8,
	MEIMotorIoConfigIndex9,
	MEIMotorIoConfigIndex10,
	MEIMotorIoConfigIndex11,
	MEIMotorIoConfigIndex12,
	MEIMotorIoConfigIndex13,
	MEIMotorIoConfigIndex14,
	MEIMotorIoConfigIndex15,
	MEIMotorIoConfigIndex16,
	MEIMotorIoConfigIndex17,
	MEIMotorIoConfigIndex18,
	MEIMotorIoConfigIndex19,
	MEIMotorIoConfigIndex20,
	MEIMotorIoConfigIndex21,
	MEIMotorIoConfigIndex22,
	MEIMotorIoConfigIndex23,
	MEIMotorIoConfigIndex24,
	MEIMotorIoConfigIndex25,
	MEIMotorIoConfigIndex26,
	MEIMotorIoConfigIndex27,
	MEIMotorIoConfigIndex28,
	MEIMotorIoConfigIndex29,
	MEIMotorIoConfigIndex30,
	MEIMotorIoConfigIndex31,


	MEIMotorIoConfigIndexLAST,
	MEIMotorIoConfigIndexFIRST = MEIMotorIoConfigIndexINVALID + 1
} MEIMotorIoConfigIndex;

typedef enum MEIMotorDisableAction {
	MEIMotorDisableActionINVALID = -1,

	MEIMotorDisableActionNONE,
	MEIMotorDisableActionCMD_EQ_ACT,

	MEIMotorDisableActionLAST,
	MEIMotorDisableActionFIRST = MEIMotorDisableActionINVALID + 1
} MEIMotorDisableAction;

typedef enum MEIMotorDemandMode {
	MEIMotorDemandModeINVALID = -1,
	
	MEIMotorDemandModeTORQUE,
	MEIMotorDemandModeVELOCITY,
	MEIMotorDemandModeANALOG,
	MEIMotorDemandModeANALOG_DUAL_DAC,
	MEIMotorDemandModeCUSTOM,
	MEIMotorDemandModeLAST,
	MEIMotorDemandModeFIRST = MEIMotorDemandModeINVALID +1
} MEIMotorDemandMode;

typedef struct MEIMotorConfig {
	char					userLabel[MEIObjectLabelCharMAX+1]; /* +1 for NULL terminator */

	MEIMotorDemandMode		demandMode;
	MEIMotorEncoder			Encoder[MEIXmpMotorEncoders];

	MEIMotorStatusOutput	StatusOutput;

	MEIMotorIoConfig		Io[MEIMotorIoConfigIndexLAST];
	
	MEIMotorFaultConfig		faultConfig;

	MEIMotorStepper			Stepper;
	MEIMotorDacConfig		Dac;

	MEIXmpCommutationBlock	Commutation;		/* read-only from field Theta to end */

	MEIXmpLimitData			Limit[MEIXmpLimitLAST];

	MPIAction				nodeFailureAction;	 
	MPIAction				userFaultAction;	/* see MEISqNodeConfigUserFault{} structure */
	MEIMotorDisableAction	disableAction;

} MEIMotorConfig;

typedef struct MEIMotorDacChannelStatus {
	float	level;	/* volts */
} MEIMotorDacChannelStatus;

typedef struct MEIMotorDacStatus {
	MEIMotorDacChannelStatus	cmd;
	MEIMotorDacChannelStatus	aux;
} MEIMotorDacStatus;

typedef struct MEIMotorStepperStatus {
	MPI_BOOL	pulseLockLost; /* TRUE if Pulse jitter logic has lost lock , otherwise FALSE */
	MPI_BOOL	pulseStatus;   /* TRUE if a pulse was generated incorrectly, otherwise FALSE */
}MEIMotorStepperStatus;

typedef struct MEIMotorStatus {
	MEIMotorDacStatus		dac;
	MEIMotorFaultMask		faultMask;
	MEIMotorStepperStatus	stepper;
} MEIMotorStatus;

/* Phase Finding Structures */
typedef enum MEIMotorPhaseFindState  {
	MEIMotorPhaseFindStateIN_PROGRESS,
	MEIMotorPhaseFindStateSUCCESS,
	MEIMotorPhaseFindStateFAILURE,
}MEIMotorPhaseFindState;

typedef struct MEIMotorPhaseFindDriveMsg {
/* drive specific code and message reporting information 
   on the success or failure, see drive module for details */
	long				code;		
	char*				message;	
} MEIMotorPhaseFindDriveMsg;

typedef struct  MEIMotorPhaseFindStatus {
	MEIMotorPhaseFindState		state;
	MEIMotorPhaseFindDriveMsg		msg;

}MEIMotorPhaseFindStatus;

MPI_DECL1 long MPI_DECL2
	meiMotorPhaseFindStart(MPIMotor		motor);

MPI_DECL1 long MPI_DECL2
	meiMotorPhaseFindStatus(MPIMotor	motor,
							MEIMotorPhaseFindStatus*  status);

MPI_DECL1 long MPI_DECL2
	meiMotorPhaseFindAbort(MPIMotor motor);


typedef enum {
	MEIMotorTraceFIRST = MEITraceLAST << 1,

	MEIMotorTraceLAST  = MEIMotorTraceFIRST << 1
} MEIMotorTrace;

MPI_DECL1 long MPI_DECL2
	meiMotorCommutationModeGet(MPIMotor			motor,
							   MEIXmpCommMode	*mode);

MPI_DECL1 long MPI_DECL2
	meiMotorCommutationModeSet(MPIMotor			motor,
							   MEIXmpCommMode	mode);

MPI_DECL1 long MPI_DECL2
	meiMotorDacConfigGet(MPIMotor		   motor,
						 MEIMotorDacConfig *config);

MPI_DECL1 long MPI_DECL2
	meiMotorDacConfigSet(MPIMotor		   motor,
						 MEIMotorDacConfig *config);

MPI_DECL1 long MPI_DECL2
	meiMotorDacFlashConfigGet(MPIMotor			motor,
							  void				*flash,
				    		  MEIMotorDacConfig *config);

MPI_DECL1 long MPI_DECL2
	meiMotorDacFlashConfigSet(MPIMotor			motor,
							  void				*flash,
				    		  MEIMotorDacConfig *config);

MPI_DECL1 long MPI_DECL2
	meiMotorEncoderRatio(MPIControl				control,
						 long					motorNumber,
						 long					encoderNumber,
						 MEIMotorEncoderRatio	*ratio);

MPI_DECL1 long MPI_DECL2
	meiMotorEncoderReset(MPIMotor	motor);


MPI_DECL1 long MPI_DECL2
	meiMotorDemandModeGet(MPIMotor motor, 
						  MEIMotorDemandMode* mode); 

MPI_DECL1 long MPI_DECL2
	meiMotorDemandModeSet(MPIMotor motor, 
						  MEIMotorDemandMode mode);


typedef enum MEIMotorInfoNodeType {
	MEIMotorInfoNodeTypeINVALID = -1,

	MEIMotorInfoNodeTypeNONE,
	MEIMotorInfoNodeTypeSQNODE,

	MEIMotorInfoNodeTypeLAST,
	MEIMotorInfoNodeTypeFIRST = MEIMotorInfoNodeTypeINVALID + 1
} MEIMotorInfoNodeType;

typedef struct MEIMotorInfoDedicatedIn {
	MPI_BOOL supported;
} MEIMotorInfoDedicatedIn;

typedef struct MEIMotorInfoDedicatedOut {
	MPI_BOOL supported;
} MEIMotorInfoDedicatedOut;

typedef struct MEIMotorInfoGeneralIo {
	MPI_BOOL			supported;
	char				*name;
	MEIMotorIoTypeMask	validTypes;
} MEIMotorInfoGeneralIo;

typedef struct MEIMotorInfo {
	MEIMotorInfoNodeType nodeType;
	struct {
		long			 networkNumber;
		long			 nodeNumber;
		long			 driveIndex;
	} sqNode;

	long	captureCount;
	long	encoderCount;
	long	probeCount;

	MEIMotorInfoDedicatedIn		dedicatedIn[MPIMotorDedicatedInLAST];
	MEIMotorInfoDedicatedOut	dedicatedOut[MPIMotorDedicatedOutLAST];
	MEIMotorInfoGeneralIo		generalIo[MPIMotorGeneralIoLAST];
} MEIMotorInfo;

MPI_DECL1 long MPI_DECL2
	meiMotorInfo(MPIMotor		motor,
				 MEIMotorInfo	*info);

MPI_DECL1 long MPI_DECL2
	meiMotorStatus(MPIControl	control,
				   long			motorNumber,
				   MPIStatus	*status,
				   void		 	*external);

/* Notify */
typedef enum {
	MEINotifyTraceFIRST = MEITraceLAST << 1,

	MEINotifyTraceTHREAD = MEINotifyTraceFIRST << 0,

	MEINotifyTraceLAST  = MEINotifyTraceFIRST << 1
} MEINotifyTrace;

/* Object */
#define	meiObjectMessageGET(object)				(((MEIObject)(object))->message)
#define	meiObjectMessageSET(object,messageId)	(((MEIObject)(object))->message = (messageId))

#define	meiObjectTimeoutGET(object)			(((MEIObject)(object))->timeout)
#define	meiObjectTimeoutSET(object,TIMEOUT)	(((MEIObject)(object))->timeout = (TIMEOUT))

#define	meiObjectTraceGET(object)			(((MEIObject)(object))->trace)
#define	meiObjectTraceSET(object,mask)		(((MEIObject)(object))->trace = (mask))

#define	meiObjectNextGET(object)			(((MEIObject)(object))->link.next)
#define	meiObjectNextSET(object,obj)		(((MEIObject)(object))->link.next = (obj))

#define	meiObjectPreviousGET(object)		(((MEIObject)(object))->link.previous)
#define	meiObjectPreviousSET(object,obj)	(((MEIObject)(object))->link.previous = (obj))

#define	meiObjectNEXT(object)		meiObjectNextGET(object)
#define	meiObjectPREVIOUS(object)	meiObjectPreviousGET(object)

MPI_DECL1 long MPI_DECL2
	meiObjectTraceGet(MPIHandle		handle,
					  MEITraceMask	*traceMask);

MPI_DECL1 long MPI_DECL2
	meiObjectTraceSet(MPIHandle		handle,
					  MEITraceMask	traceMask);

/* Probe */

typedef enum {
	MEIProbeMessageLAST = MPIProbeMessageLAST
} MEIProbeMessage;

typedef enum {
	MEIProbeTraceFIRST = MEITraceLAST << 1,
		
	MEIProbeTraceLAST  = MEIProbeTraceFIRST << 1
} MEIProbeTrace;

typedef struct MEIProbeAddress {
	long	*primaryPosition;
	long	*secondaryPosition;
	long	*time;
	long	*status;
	short	*data;
} MEIProbeAddress;

typedef struct MEIProbeInfo {
	MEIProbeAddress	address;
} MEIProbeInfo;

MPI_DECL1 long MPI_DECL2
	meiProbeInfo(MPIProbe		probe,
				 MEIProbeInfo	*info);

/* Recorder */
typedef enum MEIRecorderTriggerIndex {
	MEIRecorderTriggerIndexINVALID = -1,

	MEIRecorderTriggerIndexSTART,
	MEIRecorderTriggerIndexSTOP,
	
	MEIRecorderTriggerIndexLAST,
	MEIRecorderTriggerIndexFIRST = MEIRecorderTriggerIndexINVALID + 1
} MEIRecorderTriggerIndex;

typedef enum MEIRecorderTriggerType {
	MEIRecorderTriggerTypeINVALID = -1,

	MEIRecorderTriggerTypeDISABLED,
	MEIRecorderTriggerTypeUSER,
	
	MEIRecorderTriggerTypeLAST,
	MEIRecorderTriggerTypeFIRST = MEIRecorderTriggerTypeINVALID + 1
} MEIRecorderTriggerType;

typedef enum MEIRecorderTriggerCondition {
	MEIRecorderTriggerConditionINVALID = -1,

	MEIRecorderTriggerConditionNONE,
	MEIRecorderTriggerConditionEQ,
	MEIRecorderTriggerConditionGREATER_THAN_OR_EQ,
	MEIRecorderTriggerConditionLESS_THAN_OR_EQ,
	MEIRecorderTriggerConditionNOT_EQ,
	MEIRecorderTriggerConditionCHANGE,

	MEIRecorderTriggerConditionLAST,
	MEIRecorderTriggerConditionFIRST = MEIRecorderTriggerConditionINVALID + 1,

	MEIRecorderTriggerConditionMATCH = MEIRecorderTriggerConditionEQ,
	MEIRecorderTriggerConditionREPEAT = 0x80000000
} MEIRecorderTriggerCondition;

typedef struct	MEIRecorderTriggerUser {
	MEIRecorderTriggerCondition	condition;
	long						*addr;
	unsigned long				mask;
	unsigned long				pattern;
	unsigned long				count;
} MEIRecorderTriggerUser;

typedef struct MEIRecorderTrigger {
	MEIRecorderTriggerType type;
	union {
		MEIRecorderTriggerUser user;
	} attributes;
} MEIRecorderTrigger;

typedef struct	MEIRecorderConfig {
	MEIRecorderTrigger	trigger[MEIRecorderTriggerIndexLAST];
} MEIRecorderConfig;

typedef struct MEIRecorderRecordAxis {
	long		sample;		/* sample number */
	float		dac;		/* voltage */
	MEIInt64	actual;		/* actual position */
	MEIInt64	command;	/* command position */
} MEIRecorderRecordAxis;

typedef struct MEIRecorderRecordFilter {
	long		sample;		/* sample number */
	long		positionError;	/* position error */
	float		dac;		/* voltage */
} MEIRecorderRecordFilter;

#define MEIRecorderMAX_AXIS_RECORDS   (5)
#define MEIRecorderMAX_FILTER_RECORDS (10)

typedef union {
	MEIRecorderRecordAxis		axis[MEIRecorderMAX_AXIS_RECORDS];
	MEIRecorderRecordFilter		filter[MEIRecorderMAX_FILTER_RECORDS];
} MEIRecorderRecord;

typedef enum {
	MEIRecorderRecordTypeAXIS = MPIRecorderRecordTypeLAST,
	MEIRecorderRecordTypeFILTER,

	MEIRecorderRecordTypeLAST,
	MEIRecorderRecordTypeFIRST = MPIRecorderRecordTypeLAST
} MEIRecorderRecordType;

typedef enum {
	MEIRecorderTraceFIRST = MEITraceLAST << 1,

	MEIRecorderTraceRECORD_GET	= MEIRecorderTraceFIRST << 0,
	MEIRecorderTraceSTATUS		= MEIRecorderTraceFIRST << 1,
	MEIRecorderTraceOVERFLOW	= MEIRecorderTraceFIRST << 2,

	MEIRecorderTraceLAST  = MEIRecorderTraceFIRST << 3
} MEIRecorderTrace;

/* Sequence */
typedef MPIEmpty	MEISequenceConfig;

typedef enum {
	MEISequenceTraceFIRST = MEITraceLAST << 1,

	MEISequenceTraceLOAD = MEISequenceTraceFIRST << 0,

	MEISequenceTraceLAST  = MEISequenceTraceFIRST << 1
} MEISequenceTrace;

MPI_DECL1 long MPI_DECL2
	meiSequenceCompile(MPISequence sequence);

/* SynqNet */
MPI_DECL1 long MPI_DECL2				
	meiSynqNetCableNumToNodePort(MEISynqNet     synqNet,
								 long           cableNumber,
								 long			*nodeNumber,
								 MEINetworkPort *port);

MPI_DECL1 long MPI_DECL2				
	meiSynqNetNodePortToCableNum(MEISynqNet     synqNet,
								 long			nodeNumber,
								 MEINetworkPort port,
								 long           *cableNumber);

/* MechaWare */
#define MEIMechaWareSIZE	(16384)

#define MEIFeatureH

#if defined(__cplusplus)
}
#endif

#ifndef MEI_NO_DEPRECATIONS
#include "meiDeprecated.h"
#endif

#endif
