/* xmpcanbl.h */

/* Copyright(c) 1991-2006 by Motion Engineering, Inc.  All rights reserved.
 *
 * This software  contains proprietary and  confidential information  of
 * Motion Engineering Inc., and its suppliers.  Except as may be set forth
 * in the license agreement under which  this software is supplied, use,
 * disclosure, or  reproduction is prohibited without the prior express
 * written consent of Motion Engineering, Inc.
 */

/*
 *	RCSHeader
 */

#if !defined(_XMPCANBL_H)
#define _XMPCANBL_H

/* == Bootloader memory map. =============================================== */
/* The bootloader uses the DPR with a different memory map.  The following */
/* definitions are used to map the DPR when using the bootloader.          */

/* Different codes are written into the signature field depending on */
/* if the bootloader or the firmware are running. */
#define	MEIXmpCanBOOT_SIGNATURE  (0xF00D)

/* Command register constants */
#define MEIXmpCommandNULL             (0x0000)
#define MEIXmpCommandPROGRAM          (0x0001)
#define MEIXmpCommandVERIFY           (0x0002)
#define MEIXmpCommandSUCCESS          (0x0003)
#define MEIXmpCommandPROGRAMMED       (0x0004)
#define MEIXmpCommandREADY            (0x0005)
#define MEIXmpCommandVALID_ADDRESS    (0x0006)
#define MEIXmpCommandEXECUTE          (0x0007)

#define MEIXmpCommandINVALID_COMMAND  (0x0010)
#define MEIXmpCommandINVALID_APP_FLAG (0x0011)
#define MEIXmpCommandFAILED_CLEAR1    (0x0012)
#define MEIXmpCommandFAILED_CLEAR2    (0x0013)
#define MEIXmpCommandFAILED_CLEAR3    (0x0014)
#define MEIXmpCommandFAILED_ERASE1    (0x0015)
#define MEIXmpCommandFAILED_ERASE2    (0x0016)
#define MEIXmpCommandFAILED_ERASE3    (0x0017)
#define MEIXmpCommandFAILED_PROGRAM   (0x0018)
#define MEIXmpCommandINVALID_ADDRESS  (0x0019)

/* Valid Application Flag */
#define MEIXmpCanVALID_APP_KEY     (0x5a5a)
#define MEIXmpCanVALID_APP_ADDR    (0x1000)
#define MEIXmpCanBOOT_VER_ADDR     (0x0040)

/* This is a map of the DPR connected to the 2407 CAN Controller.            */
/* This map is used when the bootloader is working.                          */
/* The DPR is a 16bit wide device mapped into the low 16bit of 32bit memory. */
/* The notMapped fields are the top 16bit which do not contain any data.     */
#define BlockSizeMAX 0xfe0
typedef struct MEIXmpCanBoot {

  	/* A location that is repeatedly updated with "MEIXmpCanSIGNATURE". */
  	/* If the host clears this location and it is refereshed with       */
  	/* "MEIXmpCanBOOT_SIGNATURE" then the host can assume that the CAN  */
  	/* firmware is executing.                                           */
	unsigned int volatile Signature;

    /* A copy of the CAN bootloader version. */
    unsigned int volatile BootVersion;

    /* Commands to and responses back from the bootloader.  The valuse are */
    /* defined above as the MEIXmpCommandXXX macros.                       */
    unsigned int volatile Command;

    /* This boolean flag is set by the host to prematurely stop the firmware */
    /* upgrade, at the request of the user.                                  */
    unsigned int volatile Abort;

    /* The start address of a block of firmware. */
    unsigned int volatile Address;

    /* The length of a block of firmware */
    unsigned int volatile Length;

    /* A block to firmware */
    unsigned int volatile Data[BlockSizeMAX];

    /* Unused space in the DPR. */
    unsigned int Unused[ 0x17 ];

    /* Force a bootload flag. */
    unsigned int volatile ForceBoot;

    /* The CAN controller writting to this addres will generate an interrupt */
    /* on the host.  The host clears the interrupt by reading or writing     */
    /* this address.                                                         */
    unsigned int volatile CAN2HostInterrupt;
    
    /* The host writting to this address will generate an interrupt on the */
    /* CAN controller.  The CAN controller clears the interrupt by reading */
    /* or writing this address.                                            */
    unsigned int volatile Host2CANInterrupt;

} MEIXmpCanBoot;

#endif  /* _XMPCANBL_H */
