/*********************************************************************   See the "Users Manual" at emc2/docs/Hal_Introduction.pdf*** This module provides a single module that can monitor heartbeats from* several other modules (like a charge pump signal), to make sure that* no monitored process or HAL thread dies or takes too long between* execution cycles************************************************************************ Author: Stephen Wille Padnos (swpadnos AT sourceforge DOT net)* License: GPL Version 2* Created on: Jue 19, 2010* System: Linux** Copyright (c) 2010 All rights reserved.*********************************************************************/#include "rtapi.h"		/* RTAPI realtime OS API */#include "rtapi_app.h"		/* RTAPI realtime module decls */#include "hal.h"		/* HAL public API decls */#include "stdmpi.h"#include "stdmei.h"//#include "apputil.h"#define AXIS_COUNT      (2)#define POINT_COUNT     (150)    /* Number of points in trajectory */#define FLUSH_COUNT     (50)#define TIME            (0.01)   /* Time between points (seconds) */#define POSITION_DELTA  (10.0)  /* counts for each point *//* module information */MODULE_AUTHOR("Xu blight");MODULE_DESCRIPTION("MEI card Component for EMC HAL");MODULE_LICENSE("GPL");int axisnum=1;			// must specify a count on the loadrt lineRTAPI_MP_INT(axisnum, "Number of axis");/************************************************************************                STRUCTURES AND GLOBAL VARIABLES                       *************************************************************************///double position[axisnum][POINT_COUNT]; /* Simple trapezoidal profile velocity path */double point[POINT_COUNT + 1];/* Command line arguments and defaults */long    axisNumber[8] = { 0,1,2,3,4,5,6,7  };long    motionNumber[9] ={ 0,1,2,3,4,5,6,7,8};/* Data needed for each input */typedef struct {	MPIControl      		control;                /* Control object */	// MPIAxis         		axis[axisnum];   /* Array of axis objects */    	MPIMotion       		motion;                 /* Motion object */    	MPINotify       		notifytotal;                 /* Event notification handle */    	MPIEventMgr     		eventMgr;               /* Event manager handle */    	MPIMotionParams 		params;                 /* MPI motion parameters */ 	MPIAxisConfig   		axisconfig;	MPIFilterConfig 		filterconfig;	MPIMotorConfig  		motorconfig;    	MEIEventMgrServiceConfig    	serviceConfig;	MPIEventMask                	eventMask;    	long    			returnValue;    	MPIControlType      		controlType;    	MPIControlAddress   		controlAddress;	double 				time[POINT_COUNT];	double 				position[2 * POINT_COUNT];	long				argIndex;    	long				pointIndex;    	long				axisIndex;    	MPI_BOOL    			motionDone;    	     	double  			initialPosition[2];    	double 				pos[3];// = ;	long 				poscount;	long 				axisposcount;	hal_u32_t 			slice_time;     //ns 	//hal_bit_t *startmotion;		/* pin: enable input */	hal_bit_t 			*mode; //0:single axis ,1: path	hal_u32_t 			*axismask; //bit 0 1 2 3 4 5 6 7 8 : axis x y z u v w a b c	hal_bit_t 			*cmd_enable;		/* pin: only runs while this is high (like an enable) */	hal_bit_t 			*cmd_reset;		/* pin: only runs while this is high (like an enable) */		//  hal_float_t *command;	/* pin: commanded value */		//  double prev_error;		/* previous error for differentiator */		//  hal_float_t *error_d;	/* opt. pin: differentiated error */		//  double prev_cmd;		/* previous command for differentiator */		//  double limit_state;		/* +1 or -1 if in limit, else 0.0 */		//  hal_float_t *cmd_d;		/* opt. pin: differentiated command */		//   char prev_ie;   		//  hal_s32_t c_secs, c_nsecs;	/* internal:  elapsed seconds and nanoseconds */  		//  hal_u32_t t_secs, t_nsecs;	/* internal:  seconds and nanoseconds for timeout */  		//  hal_bit_t last;		/* internal:  last value of the input pin */} mei_data;/* Base data for a weighted summer. */typedef struct {  	hal_float_t *act_pos;		/* output pin: high if all inputs are toggling, low otherwise */  	hal_u32_t *error_code;  	//  hal_float_t *cmd_pos; 	float cmd_pos;    	MPIAxis         axis ;   /* Array of axis objects */    	MPIMotion       motion ;                 /* Motion object */	MPIFilter       filter;	MPIMotor	motor;    	MPINotify       notify;                 /* Event notification handle */    	MPIEventMgr     eventMgr;               /* Event manager handle */    	MPIMotionParams params;                 /* MPI motion parameters */ 	double position[POINT_COUNT];	double time[POINT_COUNT];	} axis_data;/* other globals */static int comp_id;		/* component ID */ mei_data *mei_t;axis_data *axis_t;int motionstatus;int motionstatusbak;int flag;/************************************************************************                  LOCAL FUNCTION DECLARATIONS                         *************************************************************************/static void process(void *arg, long period);static void set_timeouts(void *arg, long period);/************************************************************************                       INIT AND EXIT CODE                             *************************************************************************/int rtapi_app_main(void){    int  n,  axisIndex;    long    returnValue,retval;    /* check that there's at least one valid input requested */    if (axisnum<1) {      rtapi_print_msg(RTAPI_MSG_ERR, "MEI: ERROR: must specify at least one input\n");      return -1;    }    if (axisnum>6) {		rtapi_print_msg(RTAPI_MSG_ERR, "MEI: ERROR: specify axis number too large than 6 \n");		return -1;    } 	    /* have good config info, connect to the HAL */    comp_id = hal_init("mei");    if (comp_id < 0) {	rtapi_print_msg(RTAPI_MSG_ERR,	    "MEI_COMP: ERROR: hal_init() failed (Return code %d)\n", comp_id);	return -1;    }    /* allocate shared memory for watchdog global and pin info */    mei_t = hal_malloc(sizeof(mei_data));    if (mei_t == 0) {	rtapi_print_msg(RTAPI_MSG_ERR,	    "MEI_COMP: ERROR: hal_malloc() for common data failed\n");	hal_exit(comp_id);	goto err;    }    /* allocate shared memory for watchdog global and pin info */    axis_t = hal_malloc(axisnum*sizeof(axis_data));    if (axis_t == 0) {	rtapi_print_msg(RTAPI_MSG_ERR,	    "MEI_COMP: ERROR: hal_malloc() for axis data failed\n");	hal_exit(comp_id);	goto err;    }	/* Obtain a Control handle */    // mei_t.controlType=MPIControlTypeDEFAULT;          mei_t->control =  mpiControlCreate(MPIControlTypeDEFAULT,  NULL);    if(mpiControlValidate(mei_t->control)==0) {	  	rtapi_print_msg(RTAPI_MSG_ERR,						"MEI_COMP: ERROR: mpiControlCreate() failed\n");		hal_exit(comp_id);		goto err;	}	    /* Initialize the controller */    returnValue = mpiControlInit(mei_t->control); 	  if (returnValue != 0) {		rtapi_print_msg(RTAPI_MSG_ERR, "MEI_COMP: ERROR: mei init  failed\n");		hal_exit(comp_id);		goto err;    }		    for (axisIndex = 0; axisIndex < axisnum; axisIndex++) {		if ((mei_t->axismask && 0x1<<axisIndex)==0) continue;		/* Create motion object */	//	axis_t[axisIndex].motion= mpiMotionCreate(control, long(axisIndex), NULL);	//	if(mpiMotionValidate(axis_t[axisIndex].motion)==0);	//	{	//	rtapi_print_msg(RTAPI_MSG_ERR,	//					"MEI_COMP: ERROR: mpiMotionCreate() for motion %d  failed\n",axisIndex);	//	hal_exit(comp_id);	//	goto err;	//	}		/* Create Axis objects */        axis_t[axisIndex].axis =  mpiAxisCreate(mei_t->control, axisIndex) ;		if(mpiAxisValidate(axis_t[axisIndex].axis)!=0) {			rtapi_print_msg(RTAPI_MSG_ERR, "MEI_COMP: ERROR: mpiAxisCreate() for axis %d failed\n",axisIndex);			hal_exit(comp_id); goto err;		 }		/* Create motor objects */        axis_t[axisIndex].motor= mpiMotorCreate(mei_t->control,  axisIndex );		if(mpiMotorValidate(axis_t[axisIndex].motor)!=0) {			rtapi_print_msg(RTAPI_MSG_ERR, "MEI_COMP: ERROR: mpimotorCreate() for motor %d failed\n",axisIndex);			hal_exit(comp_id); goto err;		 }		/* Create filter objects */        axis_t[axisIndex].filter = mpiFilterCreate(mei_t->control, axisIndex  );		if(mpiFilterValidate(axis_t[axisIndex].filter)!=0) {			rtapi_print_msg(RTAPI_MSG_ERR, "MEI_COMP: ERROR: mpiFilterCreate() for filter %d failed\n",axisIndex);			hal_exit(comp_id); goto err;		 }		returnValue = mpiAxisConfigGet(axis_t[axisIndex].axis,&(mei_t->axisconfig),NULL);		returnValue = mpiMotorConfigGet(axis_t[axisIndex].motor,&(mei_t->motorconfig),NULL);		returnValue = mpiFilterConfigGet(axis_t[axisIndex].filter,&(mei_t->filterconfig),NULL);		//strcpy_s(&(mei_t->axisconfig).userLabel,itoa(axisIndex));		//strcpy_s(&(mei_t->motorconfig).userLabel,"y");		//strcpy_s(&(mei_t->filterconfig).userLabel,"z");		returnValue = mpiAxisConfigSet(axis_t[axisIndex].axis,&(mei_t->axisconfig),NULL);		returnValue = mpiMotorConfigSet(axis_t[axisIndex].motor,&(mei_t->motorconfig),NULL);		returnValue = mpiFilterConfigSet(axis_t[axisIndex].filter,&(mei_t->filterconfig),NULL); 		// returnValue = mpiMotionAxisAppend(axis_t[axisIndex].motion,  axis_t[axisIndex].axis);		// if (returnValue!=0){  rtapi_print_msg(RTAPI_MSG_ERR, "MEI_COMP: ERROR: mpiMotionAxisAppend() for axis %d  failed\n",axisIndex);		//	 hal_exit(comp_id);  goto err;		// }		 /* Read initial command position */		 returnValue = mpiAxisCommandPositionGet(axis_t[axisIndex].axis,  &(mei_t->initialPosition[axisIndex]));		 if (returnValue!=0){			 rtapi_print_msg(RTAPI_MSG_ERR,			 "MEI_COMP: ERROR: read init pos mpiAxisCommandPositionGet() for axis %d  failed\n",axisIndex);			 hal_exit(comp_id);			 goto err;		 }        // axis_t[axisIndex].poscount=0;    }	/*total motion create */		/* Initialize motion params structure */		mei_t->params.pt.pointCount = POINT_COUNT;		mei_t->params.pt.position = mei_t->position;//		mei_t->params.pt.time = mei_t->time;//		mei_t->params.pt.point.retain = FALSE;      /* Flush frame buffer after execution */		mei_t->params.pt.point.final = 0;      /* Last point */		mei_t->params.pt.point.emptyCount = 10; /* Start E-Stop if frames left to execute*/    mei_t->motion = mpiMotionCreate(mei_t->control, axisnum, NULL);		if(mpiMotionValidate(mei_t->motion)!=0);		{		rtapi_print_msg(RTAPI_MSG_ERR,						"MEI_COMP: ERROR: mpiMotionCreate() for motion %d  failed\n",axisnum);		hal_exit(comp_id);		goto err;		}   for (axisIndex = 0; axisIndex < axisnum; axisIndex++) {		if ((*(mei_t->axismask) && 0x1<<axisIndex)==0) continue;        returnValue = mpiMotionAxisAppend(mei_t->motion,  axis_t[axisIndex].axis);		 if (returnValue!=0){  rtapi_print_msg(RTAPI_MSG_ERR, "MEI_COMP: ERROR: mpiMotionAxisAppend() for axis %d to motiontotal  failed\n",axisIndex);			 hal_exit(comp_id);  goto err;		 }   }     /* export pins params for all inputs */   // for (n = 0; n < axisnum; n++) {      //	if ((*(mei_t->axismask) && 0x1<<n)==0) continue;      // retval=hal_pin_float_newf(HAL_IN, &(axis_t[n].cmd_pos), comp_id, "mei.axis%d.cmd_pos", n);      //   if (retval != 0) {	//	   rtapi_print_msg(RTAPI_MSG_ERR, "MEI_COMP: ERROR: couldn't create input pin mei.axis%d.cmd_pos\n", n);	//	   goto err;}	//}	   retval=hal_pin_bit_newf(HAL_IN, &(mei_t->cmd_enable), comp_id, "mei.cmd_enable");       if (retval != 0) {	   rtapi_print_msg(RTAPI_MSG_ERR,  "MEI_COMP: ERROR: couldn't create input pin mei.cmd_enable\n");	   goto err;       }       retval=hal_pin_bit_newf(HAL_IN, &(mei_t->mode), comp_id, "mei.mode");       if (retval != 0) {	   rtapi_print_msg(RTAPI_MSG_ERR,  "MEI_COMP: ERROR: couldn't create input pin mei.mode\n");	   goto err;       }	   	   retval=hal_pin_bit_newf(HAL_IN, &(mei_t->cmd_reset), comp_id, "mei.cmd_reset");       if (retval != 0) {	   rtapi_print_msg(RTAPI_MSG_ERR,  "MEI_COMP: ERROR: couldn't create input pin mei.cmd_reset\n");	   goto err;       }            retval=hal_pin_u32_newf(HAL_IN, &(mei_t->axismask), comp_id, "mei.axismask");       if (retval != 0) {	   rtapi_print_msg(RTAPI_MSG_ERR,  "MEI_COMP: ERROR: couldn't create input pin mei.axismask\n");	   goto err;       }    /* export "global" pins */    for (n = 0; n < axisnum; n++) {		if ((*(mei_t->axismask) && 0x1<<n)==0) continue;     retval=hal_pin_u32_newf(HAL_OUT, &(axis_t[n].error_code), comp_id,  "mei.axis%d.error_code", n);     if (retval != 0) {	  rtapi_print_msg(RTAPI_MSG_ERR,   "MEI_COMP: ERROR: couldn't create output pin mei.axis%d.error_code\n",n);	  goto err;     }     retval=hal_pin_float_newf(HAL_IN, &(axis_t[n].act_pos), comp_id, "mei.axis%d.act_pos", n);     if (retval != 0) {	  rtapi_print_msg(RTAPI_MSG_ERR,  "MEI_COMP: ERROR: couldn't create input pin mei.axis%d.act_pos\n",n);	  goto err;     }  }    /* export functions */    retval = hal_export_funct("mei.process", process, 0, 0, 0, comp_id);    if (retval != 0) {	rtapi_print_msg(RTAPI_MSG_ERR,  "MEI_COMP: ERROR: process funct export failed\n");	goto err;    }    retval = hal_export_funct("mei.set-timeouts", set_timeouts, 0, 1, 0, comp_id);    if (retval != 0) {	rtapi_print_msg(RTAPI_MSG_ERR,  "MEI_COMP: ERROR: set_timeouts funct export failed\n");	goto err;    } 	 motionstatus=0;mei_t->poscount=0;mei_t->axisposcount=0;motionstatusbak=0;    rtapi_print_msg(RTAPI_MSG_INFO,	"MEI_COMP: installed mei with %d axis\n", axisnum);    hal_ready(comp_id);    return 0;    err:    hal_exit(comp_id);    return -1;}void rtapi_app_exit(void){    hal_exit(comp_id);}/************************************************************************                     REALTIME FUNCTIONS                               *************************************************************************//*  */static void process(void *arg, long period){    int n;    long    returnValue,flushnum,countdown=-1;     MPI_BOOL enableState;	  //enable motor    for (n = 0; n < axisnum; n++) {		if ((mei_t->axismask && 0x1<<n)==0) continue;        if (*(mei_t->cmd_enable)==1 ){			 motionstatus=1;         //set start flag , if enable on ,then start motion object	         returnValue = mpiMotorAmpEnableGet(axis_t[n].motor,&enableState);    //msgCHECK(returnValue);	            if (enableState == FALSE)	returnValue = mpiMotorAmpEnableSet(axis_t[n].motor, TRUE);    //    msgCHECK(returnValue);		     }}	if(*(mei_t->cmd_enable)==0){motionstatus=0;}    if (motionstatus==1 && motionstatusbak==0) ////start motion ride edge	{  		flushnum=FLUSH_COUNT+50; mei_t->poscount=0;motionstatusbak=motionstatus;	}  	if ((motionstatus+motionstatusbak )>0 )		{				mei_t->position[mei_t->poscount]=axis_t[n].cmd_pos;                        mei_t->time[mei_t->poscount]=TIME;			mei_t->poscount=mei_t->poscount+1;	}	if (motionstatus==1 && motionstatusbak==1 && mei_t->poscount>=flushnum) { 				for (n = 0; n < axisnum; n++) {				if ((*(mei_t->axismask) && 0x1<<n)==0) continue;				if (flushnum>FLUSH_COUNT){			       returnValue = mpiMotionStart(mei_t->motion, MPIMotionTypePT,&(mei_t->params));			       mei_t->poscount=0;flushnum=FLUSH_COUNT;					mei_t->params.pt.point.final = 0;      /* Last point */				 //  mei_t.params.pt.point.emptyCount = 10; /* Start E-Stop if frames left to execute				}else				{    /* Modify motion */                  returnValue =  mpiMotionModify(mei_t->motion,  MPIMotionTypePT | MPIMotionAttrMaskAPPEND,  &(mei_t->params)); 			       mei_t->poscount=0;flushnum=FLUSH_COUNT;				}			}	}	if (motionstatus==0 && motionstatusbak==1) //stop motion 	{		mei_t->params.pt.point.final = 1;      /* Last point */        mei_t->params.pt.pointCount =mei_t->poscount;        returnValue =  mpiMotionModify(mei_t->motion,  MPIMotionTypePT | MPIMotionAttrMaskAPPEND,  &(mei_t->params));		motionstatusbak=0;countdown=mei_t->poscount;mei_t->poscount=0;	}    if (countdown>=0 && motionstatusbak==0 && motionstatusbak==0)	{		countdown=countdown-1;		if (countdown==0){                        			for (n = 0; n < axisnum; n++) {		          if ((*(mei_t->axismask) && 0x1<<n)==0) continue; 			      returnValue = mpiMotorAmpEnableSet(axis_t[n].motor, FALSE);    // 		    }		  countdown=-1;	  }	}	if (*(mei_t->cmd_reset)==1 )	{ 		returnValue = mpiMotionAction(mei_t->motion , MPIActionRESET);    //msgCHECK(returnValue); 	}}static void set_timeouts(void *arg, long period){    int n, i;    hal_float_t temp;    if (*(mei_t->cmd_enable)==1 && flag==0)	{  		 for (n = 0; n < axisnum; n++) {		 axis_t[n].cmd_pos=i*2;i+=1;	    }	   if (i>=10000 ) flag=1;	}    if (*(mei_t->cmd_enable)==1 && flag==1)	{  		 for (n = 0; n < axisnum; n++) {		 axis_t[n].cmd_pos=i*2;i-=1;	    }	   if (i<=1) flag=0;	}	if (*(mei_t->cmd_enable)==0) {flag=0;i=0;}}/************************************************************************                   LOCAL FUNCTION DEFINITIONS                         *************************************************************************/